-- This files converts intranet absences and old calendar items
-- to the new calendar model.



-- First add a constraint that the end date must be after the start
-- date. Delete all rows that don't match.
DECLARE
  CURSOR c_vacations IS
    select uv.vacation_id
      from user_vacations uv
     where uv.start_date > uv.end_date;

BEGIN
   for rec in c_vacations loop
	delete from user_vacations uv where uv.vacation_id = rec.vacation_id;
   END LOOP;
   EXECUTE IMMEDIATE 'alter table user_vacations add constraint user_vacations_date_ck check(end_date >= start_date)';
END;
/


-- Inserts the specified vacation_id into the calendar as an event
-- Currently (9/7/100), this procedure is only used during the data model upgrade process.
-- The intranet module uses a tcl wrapper to call the tcl calendar api to insert other items.
-- mbryzek -- 9/7/100
create or replace procedure im_add_vacation_to_calendar ( p_vacation_id IN integer ) 
IS
  i 				integer;
  v_start_julian_date 		integer;
  v_end_julian_date 		integer;
  v_creator_id			integer;
  v_instance_id			integer;
  v_number_days			integer;
  v_group_id 			integer;
  v_user_id			user_vacations.user_id%TYPE;
  v_vacation_type		user_vacations.vacation_type%TYPE;
  v_description			user_vacations.description%TYPE;
  v_start_date			date;
  v_this_start_date		date;
  v_last_modified		date;
  v_name			varchar(4000);
BEGIN 
  -- just pick the first user as the creator (should be the system account)
  select min(u.user_id) into v_creator_id
    from users u;

   select ug.group_id into v_group_id
     from user_groups ug 
    where ug.short_name='employee';

  select to_char(uv.start_date,'J'), to_char(uv.end_date,'J'),
         uv.vacation_type, uv.user_id, uv.description,
	 uv.last_modified,
	 u.first_names || ' ' || substr(u.last_name, 0,1)
    into v_start_julian_date, v_end_julian_date, 
	 v_vacation_type, v_user_id, v_description,
	 v_last_modified, v_name
    from user_vacations uv, users u
   where uv.vacation_id = p_vacation_id
     and uv.user_id = u.user_id;

  if v_end_julian_date < v_start_julian_date then
	-- data integrity error
 	raise_application_error(-20000,'start date is after end date for vacation ' || p_vacation_id);
  end if;

  for i in v_start_julian_date..v_end_julian_date loop
	select to_date(i,'J') into v_this_start_date from dual;
	v_instance_id := cal_ins_item_on_id_table( 'USER_VACATIONS',
				  p_vacation_id,
				  v_this_start_date,
			          null,
			          v_name || ' (' || v_vacation_type || ')',
				  v_user_id,
				  v_group_id,
				  v_description,
				  'f',
				  '/intranet/absences/edit?vacation_id=' || p_vacation_id,
				  null,
				  't',
				  v_creator_id,
				  'f');
	update cal_items
	set creation_date = v_last_modified
	where item_id = v_instance_id
	   or parent_id = v_instance_id;
  end loop;

END;
/
show errors

-- Now loop through all registered vacations and regsiter them as events
-- note that we select only those that don't exist... this is very important :)
-- We also 
DECLARE
  v_ctr       		integer;
  v_group_id  		integer;
  v_commit_every_n	integer;

  -- limit to max vacations of 60 days... otherwise it's too hard to insert all those rows
  CURSOR c_vacations IS
    select uv.vacation_id
      from user_vacations uv
     where uv.vacation_id not in (select map.on_what_id 
                                    from cal_item_db_row_map map
                                   where map.on_which_table = 'USER_VACATIONS')
       and uv.end_date - uv.start_date <= 60;
BEGIN

   -- commit after how many rows?
   v_commit_every_n := 1;

   select ug.group_id into v_group_id
     from user_groups ug 
    where ug.short_name='employee';

   -- create the employees group calendar
   insert into cal_groups
   (group_id)
   select v_group_id
     from dual
    where not exists (select 1 
                        from cal_groups cg
                       where cg.group_id = v_group_id);

   v_ctr := 0;
 
   for rec in c_vacations loop
	im_add_vacation_to_calendar(rec.vacation_id);

        v_ctr := v_ctr + 1;
        IF v_ctr >= v_commit_every_n THEN
 		-- commit every v_commit_every_n rows
		v_ctr := 0;
		commit;
	END IF;

   end loop;
END;
/


-- Now we have to convert old calendar items to new calendar items.

-- First copy calendar_categories into cal_item_types
insert into cal_item_types 
(type_id, scope, group_id, user_id, type, enabled_p)
(select category_id, scope, group_id, user_id, category, enabled_p
 from calendar_categories
 where category_id not in (select type_id from cal_item_types));

-- Now alter cal_id_sequence if necessary
DECLARE
  v_incr_cal_id INTEGER;
BEGIN
  select max(type_id) + 1 into v_incr_cal_id 
  from cal_item_types;

  select v_incr_cal_id - cal_id_sequence.nextval
  into v_incr_cal_id
  from dual;
 
  if v_incr_cal_id > 0 then
    EXECUTE IMMEDIATE 'alter sequence cal_category_id_sequence increment by '
    || v_incr_cal_id;
  end if;
END;
/

-- Declare procedure to convert calendar items
-- Inserts the specified calendar_id into the new calendar
-- Possibly inserts multiple events if start_date < end_date
-- Currently (2000-09-13), this procedure is only used during the data model upgrade process.
-- This procedure is modeled on im_add_vacation_to_calendar by mbryzek
create or replace procedure im_add_item_to_calendar ( p_calendar_id IN integer ) 
IS
  i 			integer;
  v_start_julian_date 	integer;
  v_end_julian_date 	integer;
  v_parent_id		integer;
  v_item_id		integer;
  v_number_days		integer;
  v_group_id 		integer;
  v_type_id		integer;
  v_title		calendar.title%TYPE;
  v_description		calendar.body%TYPE;
  v_html_p		calendar.html_p%TYPE;
  v_start_date		date;
  v_end_date		date;
  v_event_url		calendar.event_url%TYPE;
  v_event_email		calendar.event_email%TYPE;
  v_related_link_url	cal_items.related_link_url%TYPE;
  v_country		country_codes.country_name%TYPE;
  v_usps_abbrev		calendar.usps_abbrev%TYPE;
  v_zip_code		calendar.zip_code%TYPE;
  v_creation_date	date;
  v_creator_id		integer;
  v_this_start_date	date;
BEGIN 
   -- Get employee calendar group
   select ug.group_id into v_group_id
     from user_groups ug 
    where ug.short_name='employee';

  -- Pull out item columns
  select to_char(c.start_date,'J'), to_char(c.end_date,'J'),
         c.category_id, c.title, c.body, c.html_p, c.event_url, 
	 c.event_email, cc.country_name, c.usps_abbrev, c.zip_code, 
	 c.creation_user, c.creation_date
    into v_start_julian_date, v_end_julian_date, 
	 v_type_id, v_title, v_description, v_html_p, v_event_url,
	 v_event_email, v_country, v_usps_abbrev, v_zip_code,
	 v_creator_id, v_creation_date
    from calendar c, country_codes cc
   where c.calendar_id = p_calendar_id
     and c.country_code = cc.iso(+);

  if v_end_julian_date < v_start_julian_date then
	-- data integrity error
 	raise_application_error(-20000,'start date is after end date for calendar item ' || p_calendar_id);
  end if;

  -- Need to massage some data before doing the insert
  if v_event_url is not null then
    v_related_link_url := v_event_url;
    if v_event_email is not null then
      v_description := v_description || '

' || 'email: ' || v_event_email;
    end if;
  elsif v_event_email is not null then
    v_related_link_url := 'mailto:' || v_event_email;
  end if;

  -- Insert two blank lines, then location info
  if v_usps_abbrev != '' or v_zip_code != '' or v_country != '' then
    v_description := v_description || '

' || v_usps_abbrev || ' ' || v_zip_code || ' ' || v_country;
  end if;

  -- Insert one item for each day of event
  for i in v_start_julian_date..v_end_julian_date loop
	select to_date(i,'J') into v_this_start_date from dual;
	v_item_id := cal_ins_item_on_id_table( 'CALENDAR',
				  p_calendar_id,
				  v_this_start_date,
			          null,
			          v_title,
				  null,
				  v_group_id,
				  v_description,
				  v_html_p,
				  v_related_link_url,
				  null,
				  'f',
				  v_creator_id,
				  'f');
	update cal_items
	set creation_date = v_creation_date
	where item_id = v_item_id
	   or parent_id = v_item_id;
  end loop;

  -- insert calendar type info
  insert into cal_item_type_map
  (item_id, type_id)
  values
  (v_item_id, v_type_id);
END;
/
show errors

-- Loop through calendar items converting to cal_items in Employees calendar
-- Assume Employees calendar was created above
DECLARE
  v_ctr       		integer;
  v_group_id  		integer;
  v_commit_every_n	integer;

  -- Ignore items that aren't approved
  -- Limit to max durations of 30 days... otherwise it's too hard to 
  -- insert all those rows and does anything more than a month make sense?
  CURSOR c_calendar IS
    select c.calendar_id
      from calendar c
     where c.calendar_id not in (select map.on_what_id 
                                   from cal_item_db_row_map map
                                  where map.on_which_table = 'CALENDAR')
       and c.end_date - c.start_date <= 30
       and approved_p = 't';
BEGIN
   -- commit after how many rows?
   v_commit_every_n := 1;

   select ug.group_id into v_group_id
     from user_groups ug 
    where ug.short_name='employee';


   v_ctr := 0;
 
   for rec in c_calendar loop
	im_add_item_to_calendar(rec.calendar_id);

        v_ctr := v_ctr + 1;
        IF v_ctr >= v_commit_every_n THEN
 		-- commit every v_commit_every_n rows
		v_ctr := 0;
		commit;
	END IF;
   END loop;
END;
/

