# packages/calendar/calendar-api.tcl

ad_library {
    API for other packages/modules to call into calendar.

    @creation-date 2000-08-18
    @author W. Scott Meeks (smeeks@arsdigita.com)
    @cvs-id $Id: calendar-api-procs.tcl,v 1.1.1.1 2001/01/24 18:03:15 tarik Exp $
}

# For a more user-friendly error log...
util_report_library_entry

# It doesn't seem like you can set the default value for a keyword argument
# to something that needs to be evaluated, so this simplifies setting
# [db_null] as the default value for arguments.
proc default_db_null {args} {
    foreach arg $args {
	if {![uplevel "exists_and_not_null $arg"]} {
	    uplevel "set $arg {[db_null]}"
	}
    }
}

ad_proc -public cal_insert_item {
    -parent_id
    -item_id
    -title
    -valid_start_date
    -valid_end_date
    -start_date_sql
    -end_date_sql
    -date
    -start_time
    -end_time
    {-duration 60}
    -user_id
    -group_id
    -description
    -html_p
    -related_link_url
    -related_link_text
    -redirect_to_related_link_p
    -creator_id:required
    -on_what_id
    -on_which_table
    -editable_p
} {
    Inserts a new calendar item into the appropriate calendar.
    Does not do validation of arguments.

    @param -parent_id and @param -item_id support passing in a parent_id 
    and item_id for double-click protection.  If @param -parent_id 
    isn't passed in, then both will be generated. Specifying @param -parent_id
    without @param -item_id is an error.

    @param -title 

    @param -valid_start_date and @param -valid_end_date are used to pass in 
    start and end values that can be passed in as bind variables for extra
    security.

    @param -start_date_sql and @param -end_date_sql are used to directly 
    specify the start_date and end_date to be inserted into the DB.  These
    parameters override the following parameters.

    These parameters are used to calculate start_date and end_date based on a 
    combination of date, time and duration:
    @param -date assumes 'YYYY-MM-DD'.
    @param -start_time and @param -end_time assume 'HH24:MI'.  If 
    @param -start_time is not specified, then a No Time item will be created
    and the value of @param -end_time will be ignored.
    If @param -end_time is not specified, then the value of @param -duration 
    will be used.

    At a bare minimum, either @param -start_date_sql or @param -date must be 
    specified.

    @param -user_id indicates a personal item.
    @param -group_id indicates a group item.
    Specifying neither or both is currently undefined but will not cause an
    error.

    @param -description 
    @param -html_p indicates whether or not description has html tags;
    default is 'f'

    @param -related_link_url if not specified, then no link will appear in the 
    item details. 
    @param -related_link_text if not specified, then the value of 
    -related_link_url will be displayed as the link text.
    @param -redirect_to_related_link_p if 'f' then @param title will be 
    displayed for the item on the various views as a link to details about 
    the items. @param related_link_url and @param related_link_text will be 
    used to display a link on the details page.
    if 't' then @param related_link_url and @param related_link_text will be 
    used to form the link that appears in the various views.
    The default is 'f'.

    @param -creator_id is required.  Generally it will be the logged in user.

    @param -on_what_id and @param -on_which_table throws an error if one but 
    not both are specified.  If both are specified, then a row will be added
    to the mapping table.

    @param -editable_p indicates whether or not the item can be edited by 
    anyone or only deleted.  

    Throws error if insert fails.
    Returns item_id (from whence parent_id can be derived) on success.
} {
    # Do argument interpretation in TCL, then dispatch to appropriate
    # PL/SQL function.
    
    # Set defaults
    default_db_null title user_id group_id description html_p related_link_url related_link_text redirect_to_related_link_p editable_p

    if ![exists_and_not_null end_date_sql] {
	set end_date_sql "NULL"
    }

    # Figure out start_date, end_date
    set date_format "'YYYY-MM-DD HH24:MI'"
    
    if [exists_and_not_null valid_start_date] {
	set start_date_sql "to_date(:valid_start_date, $date_format)"
	set end_date_sql "to_date(:valid_end_date, $date_format)"
    }
    if ![exists_and_not_null start_date_sql] {
	if [exists_and_not_null date] {
	    if {[exists_and_not_null start_time]} {
		set start_date_sql "to_date('$date $start_time', $date_format)"

		if [exists_and_not_null end_time] {
		    set end_date_sql "to_date('$date $end_time', $date_format)"
		} else {
		    # Oracle date arithmetic uses fractional days
		    set end_date_sql "$start_date_sql + $duration / (24 * 60)"
		}
	    } else {
		set start_date_sql "to_date('$date')"
		set end_date_sql "NULL"
	    }
	} else {
	    error "Must specify either -date or -start_date_sql."
	}
    }
    # on_what_id/on_which_table
    if {[exists_and_not_null on_what_id] && [exists_and_not_null on_which_table]} {
	if {[exists_and_not_null parent_id]} {
	    if ![exists_and_not_null item_id] {
		error "Must specify item_id if parent_id is specified"
	    }
	    # Have to do the steps separately
	    db_transaction {
		db_dml cal_insert_item_with_ids "
		BEGIN
		cal_insert_item_with_ids(:parent_id, :item_id,
		$start_date_sql, $end_date_sql,
		:title, :user_id, :group_id, :description, :html_p,
		:related_link_url, :related_link_text, 
		:redirect_to_related_link_p,
		:creator_id, :editable_p);
		END;
		"
		db_dml cal_insert_db_row_map "
		INSERT INTO cal_item_db_row_map
		(item_id, on_which_table, on_what_id)
		VALUES
		(:item_id, upper(trim(:on_which_table)), :on_what_id)
		"
	    }
	} else {
	    set item_id [db_exec_plsql cal_ins_item_on_id_table "
	    BEGIN
	    :1 := cal_ins_item_on_id_table(:on_which_table, :on_what_id, 
	    $start_date_sql, $end_date_sql,
	    :title, :user_id, :group_id, :description, :html_p, 
	    :related_link_url, :related_link_text, :redirect_to_related_link_p,
	    :creator_id, :editable_p);
	    END;
	    "]
	}
    } elseif {[exists_and_not_null on_what_id] || [exists_and_not_null on_which_table]} {
	# Only one specified, throw error.
	error "Only one of on_what_id/on_which_table specified."
    } else {
	if {[exists_and_not_null parent_id]} {
	    if ![exists_and_not_null item_id] {
		error "Must specify item_id if parent_id is specified"
	    }
	    db_dml cal_insert_item_with_ids "
	    BEGIN
	    cal_insert_item_with_ids(:parent_id, :item_id,
	    $start_date_sql, $end_date_sql,
	    :title, :user_id, :group_id, :description, :html_p,
	    :related_link_url, :related_link_text, :redirect_to_related_link_p,
	    :creator_id, :editable_p);
	    END;
	    "
	} else {
	    set item_id [db_exec_plsql cal_insert_item "
	    BEGIN
	    :1 := cal_insert_item($start_date_sql, $end_date_sql,
	    :title, :user_id, :group_id, :description, :html_p,
	    :related_link_url, :related_link_text, :redirect_to_related_link_p,
	    :creator_id, :editable_p);
	    END;
	    "]
	}
    }
    db_release_unused_handles
    return $item_id
}

# NOTE: This proc is incomplete.  It should not be used at this time.
ad_proc -private cal_update_item {
    -parent_id
    -item_id
    -on_what_id
    -on_which_table
    -modifying_user:required
    -title
    -start_date_sql
    -end_date_sql
    -date
    -start_time
    -end_time
    -description
    -related_link_url
    -related_link_text
    -editable_p:boolean
} {
    Updates an existing calendar item.
    Does not do validation of arguments.

    Either parent_id, item_id, or on_what_id/on_which_table must be
    specified.  If parent_id but not item_id is specified for a repeating
    item, then all instances will be updated.

    @param -modifying_user is required.  Generally it will be the logged in user.

    @param -start_date_sql and @param -end_date_sql are used to directly 
    specify the start_date and end_date to be inserted into the DB.  These
    parameters override the following parameters.

    These parameters are used to calculate start_date and end_date based on a 
    combination of date, time and duration:
    @param date assumes 'YYYY-MM-DD.
    @param -start_time and @param -end_time assume 'HH24:MI'.  
    If @param -start_time is passed an empty string, then the item will be
    changed to a No Time item.

    @param title
    @param -description
    @param -related_link_url if not specified, then no link will appear in the 
    item details. 
    @param -related_link_text if not specified, then the value of -related_link_url
    will be displayed as the link text.
    @param -on_what_id and @param -on_which_table throws an error if one but 
    not both are specified.  If both are specified, then any corresponding 
    items will be updated.
    @param -editable_p indicates whether or not the item can be edited by 
    anyone or only deleted.  

    Throws error if update fails.
    Returns 1 on success.
} {
    # Process arguments.

    # Set defaults
    default_db_null title end_date_sql user_id group_id description related_link_url related_link_text redirect_to_related_link_p editable_p

    # Figure out start_date, end_date
    set date_format "'YYYY-MM-DD HH24:MI'"

    if ![exists_and_not_null start_date_sql] {
	if [exists_and_not_null date] {
	    if {[exists_and_not_null start_time]} {
		set start_date_sql "to_date('$date $start_time', $date_format)"

		if [exists_and_not_null end_time] {
		    set end_date_sql "to_date('$date $end_time', $date_format)"
		} else {
		    # Oracle date arithmetic uses fractional days
		    set end_date_sql "$start_date_sql + $duration / (24 * 60)"
		}
	    } else {
		set start_date_sql "to_date('$date')"
		set end_date_sql "[db_null]"
	    }
	} else {
	    set start_date_sql "[db_null]"
	}
    }

    # on_what_id/on_which_table vs. item_id vs. parent_id
    if {[exists_and_not_null on_what_id] && [exists_and_not_null on_which_table]} {
	if {[exists_and_not_null parent_id] || [exists_and_not_null item_id]} {
	    error "Cannot specify parent_id or item_id if on_what_id and on_which_table are specified."
	}
	db_foreach cal_item_id_from_id_table "
	select item_id
	from cal_item_db_row_map
	where on_what_id = :on_what_id
	  and on_which_table = upper(trim(:on_which_table))
	" {
	    cal_update_item $item_id $group_id $modifying_user \
		    $start_date_sql $end_date_sql $title $description \
		    $related_link_url $related_link_text \
	            $redirect_to_related_link_p 
	}
    } elseif [exists_and_not_null item_id] {
	    cal_update_item $item_id $group_id $modifying_user \
		    $start_date_sql $end_date_sql $title $description \
		    $related_link_url $related_link_text $redirect_to_related_link_p 
    } elseif [exists_and_not_null parent_id] {
	# This is not correct.  Need to properly handle setting time
	cal_update_all_items $parent_id $group_id $modifying_user \
		    $start_date_sql $end_date_sql $title $description \
		    $related_link_url $related_link_text \
		    $redirect_to_related_link_p $editable_p
    } else {
	error "Did not specify what item(s) to update."
    }
}

proc cal_update_item { 
    item_id group_id modifying_user 
    start_date_sql end_date_sql title description 
    related_link_url related_link_text redirect_to_related_link_p 
} {
    # Pull in parent_id to check write access
    set parent_id [db_string cal_get_item_parent "
    select parent_id
    from cal_items
    where item_id = :item_id
    "]

    if {[calendar_item_write_access_p $parent_id $group_id]} {
	# We've cleared security...
	db_dml cal_update_item "
	UPDATE cal_items
	SET title = :title,
	description = :description,
	start_date = $start_date_sql,
	end_date = $end_date_sql,
	related_link_url = :related_link_url,
	related_link_text = :related_link_text,
	redirect_to_related_link_p = :redirect_to_related_link_p,
	last_modifying_user = :modifying_user,
	last_modified = sysdate
	WHERE item_id = :item_id"
    } else {
	error "Insufficient privileges to update this item."
    }
    db_release_unused_handles
    return 1
}

# Still need to deal with properly setting time....
proc cal_update_all_items {
    parent_id group_id modifying_user 
    start_date_sql end_date_sql title description 
    related_link_url related_link_text redirect_to_related_link_p editable_p 
} {
    if {[calendar_item_write_access_p $parent_id $group_id]} {
	# We've cleared security...

	# Need to figure out the time offsets....
	db_transaction {
	    db_dml cal_update_parent "
	    UPDATE cal_items
	    SET title = :title,
	    description = :description,
	    related_link_url = :related_link_url,
	    related_link_text = :related_link_text,
	    redirect_to_related_link_p = :redirect_to_related_link_p,
	    editable_p = :editable_p
	    WHERE parent_id=:parent_id"

	    # Reset the instances to nulls so that
	    # they inherit properly and set the time
	    db_dml cal_update_items "
	    UPDATE cal_item_instances
	    SET title = NULL,
	    description = NULL,
	    start_date = $start_date_sql,
	    end_date = $end_date_sql,
	    related_link_url = NULL,
	    related_link_text = NULL,
	    redirect_to_related_link_p = NULL,
	    last_modifying_user = :modifying_user,
	    last_modified = sysdate
	    WHERE parent_id = :parent_id"
	}
    } else {
	error "Insufficient privileges to update this item."
    }
    db_release_unused_handles
    return 1
}

ad_proc -public cal_groups_with_calendars {} {
    Returns a list of {group_id1 group_name1 group_id2 group_name2 ....} pairs
    for groups that have calendars, suitable for passing to foreach.  E.g.
    foreach {id name} [cal_groups_with_calendars] { ... }
} {
    set result [list]

    db_foreach cal_groups_with_calendars "
    select ug.group_id, ug.group_name, lower(ug.group_name)
    from user_groups ug, cal_groups cg
    where ug.group_id = cg.group_id
    order by lower(ug.group_name)
    " {
	lappend result $group_id $group_name
    }
    db_release_unused_handles
    return $result
}

ad_proc -public cal_delete_mapped_instances {
    -user_id
    on_which_table
    on_what_id
} {
    Finds and deletes all calendar instances and associated rows which have
    a mapping in the database.
    @param on_which_table 
    @param on_what_id 
    @param -user_id       Limits deletion to only those items with this user_id
} {
    if [exists_and_not_null user_id] {
	set cal_item_list [db_list_of_lists cal_mapped_items "
	select parent_id, ii.item_id
	from cal_item_db_row_map m, cal_items ii, cal_user_group_item_map im
	where m.item_id = ii.item_id
	  and on_what_id = :on_what_id
	  and on_which_table = upper(trim(:on_which_table))
	  and im.user_id = :user_id
	  and im.item_id = parent_id
	order by parent_id
	"]
    } else {
	set cal_item_list [db_list_of_lists cal_mapped_items "
	select parent_id, ii.item_id
	from cal_item_db_row_map m, cal_items ii
	where m.item_id = ii.item_id
	  and on_what_id = :on_what_id
	  and on_which_table = upper(trim(:on_which_table))
	order by parent_id
	"]
    }
    foreach pair $cal_item_list {
	set parent_id [lindex $pair 0]
	set item_id [lindex $pair 1]
	cal_delete_item $item_id

	# Check for deleting last instance
	if {[exists_and_not_null old_parent_id] && $old_parent_id != $parent_id} {
	    if {[db_1row cal_count_items "
	    select count(item_id)
	    from cal_items
	    where parent_id = :parent_id
	    "] == 0} {
		cal_delete_parent_item $parent_id
	    }
	}
	set old_parent_id $parent_id
    }
    # Check for deleting last instance
    if {[exists_and_not_null old_parent_id] && $old_parent_id != $parent_id} {
	if {[db_1row cal_count_items "
	select count(item_id)
	from cal_items
	where parent_id = :parent_id
	"] == 0} {
	    cal_delete_parent_item $parent_id
	}
    }
}

ad_proc -public cal_delete_item {
    item_id
} {
    Delete a particular item and all associated rows.
} {
    db_transaction {
	# Delete the instance and dependencies
	db_dml cal_item_type_map_delete "
	delete from cal_item_type_map
	where item_id = :item_id"
	db_dml cal_item_db_row_map_delete "
	delete from cal_item_db_row_map
	where item_id = :item_id"
	db_dml cal_item_delete "
	delete from cal_items
	where item_id = :item_id"
    }
    db_release_unused_handles
}

ad_proc -public cal_delete_parent_item {
    parent_id
} {
    Delete all instances of a particular item and all associated rows.
} {
    db_transaction {
	# Delete dependent rows.	
	db_dml cal_user_group_item_map_delete "
	delete from cal_user_group_item_map	
	where item_id = :parent_id"
	db_dml cal_item_type_map_delete "
	delete from cal_item_type_map
	where item_id in (select item_id
	                  from cal_items
	                  where parent_id = :parent_id)"
	db_dml cal_item_db_row_map_delete "
	delete from cal_item_db_row_map
	where item_id in (select item_id
	                  from cal_items
	                  where parent_id = :parent_id)"
	db_dml cal_alerts_delete "
	delete from cal_alerts
	where item_id = :parent_id"
	db_dml cal_items_delete "
	delete from cal_items
	where parent_id = :parent_id"
	
	# Now delete parent
	db_dml cal_parent_delete "
	delete from cal_items
	where item_id = :parent_id"
    }
    db_release_unused_handles
}


ad_proc -public cal_insert_repeating_item {
    { -parent_id "" }
    { -item_id "" }
    { -title "" }
    -start_date:required
    -end_date:required
    { -start_time "" }
    { -end_time "" }
    {-duration 60}
    { -user_id "" }
    { -group_id "" }
    { -description "" }
    { -related_link_url "" }
    { -related_link_text "" }
    { -redirect_to_related_link_p "" }
    { -related_url "" }
    { -related_url_text "" }
    { -related_url_p "" }
    { -creator_id "" }
    { -creation_user ""}
    { -on_what_id "" }
    { -on_which_table "" }
    { -editable_p "" }
} {
    Wrapper for cal_insert_item that simply calls it repeatedly for
    each day between start_date and end_date. 

    @param start_date First date of the event (Ansi format)
    @param end_date Last date of the event (Ansi format)
} {
    # The data model changed after the intranet changes were released
    # This changed some of the parameter names, but we support the old names
    # too for backward compatibility.  The new names take precendence over the
    # old ones should both be present.
    if [empty_string_p $related_link_url] {
	set related_link_url $related_url
    }
    if [empty_string_p $related_link_text] {
	set related_link_text $related_url_text
    }
    if [empty_string_p $redirect_to_related_link_p] {
	set redirect_to_related_link_p $related_url_p
    }
    if [empty_string_p $creator_id] {
	set creator_id $creation_user
    }

    # We need julian dates to do calendar insert
    db_1row cal_julian_dates \
	"select to_char(to_date(:start_date,'YYYY-MM-DD'),'J') as julian_start_date,
	        to_char(to_date(:end_date,'YYYY-MM-DD'),'J') as julian_end_date
           from dual"
    
    for { set i $julian_start_date } { $i <= $julian_end_date } { incr i } {
	# Insert one calendar item for every date between start and end
	cal_insert_item  -parent_id $parent_id \
		-item_id $item_id \
		-title $title \
		-start_date_sql "to_date($i,'J')" \
		-start_time $start_time \
		-end_time $end_time \
		-duration $duration \
		-user_id $user_id \
		-group_id $group_id \
		-description $description \
		-related_link_url $related_link_url \
		-related_link_text $related_link_text \
		-redirect_to_related_link_p $redirect_to_related_link_p \
		-creator_id $creator_id \
		-on_what_id $on_what_id \
		-on_which_table $on_which_table \
		-editable_p $editable_p
    }
}


