ad_library {
    Utility functions for Calendar package
    
    @creation-date Before June 2000
    @author W. Scott Meeks (smeeks@arsdigita.com)
    @author Bryan Che (bryanche@arsdigita.com)
    @author Tarik Alatovic (tarik@arsdigita.com)
    @author David Eison (eison@arsdigita.com)
    @author Karl Goldstein (karlg@arsdigita.com)

    @cvs-id $Id: calendar-procs.tcl,v 1.16 2001/02/07 01:20:14 tarik Exp $
}

# TODO
# The following is a list of minor but useful improvements that should be made
# * General code clean up, particularly in mini-calendar and day view
# * Investigate cal_get_dates query
# * Clean up cal_related_link_html
# * Investigate cal_hours_select query
# * Turn cal_current_days query into a memoized proc
# * Investigate whether calendar_item_aev still needs to be split up.
# * Turn cal_get_repeat_dates into a cleaner memoized proc
# * Figure out display of repeat information in item view
# * Investigate whether or not compress_day_view_p is being used properly.
#   It currently doesn't seem to be correctly sticky.
# * Consider memoizing cal_get_user_info
# * Consider making cal_group_schedule not depend on portals and clean
#   up the query.
# * Push bind vars for dates further if possible.
# * calendar_form_transform_datetime should use regexp to validate input
# * Fix calendar_types_widget to validate scope
# * Provide for updating only future instances of repeating items.
# * Change instances where we select dates to make sure they are alway to_char
#   a specific format because the default could be different on different 
#   platforms.

# DATE-NOTE
# You will see Tcl variables being used for some dates in SQL statements.   
# This is because these dates are being generated and modified as SQL fragments
# (e.g. "to_char(:start_date_def, 'YYYY-MM-DD HH24:MI:SS') + (:duration)/(24*60)")
# that evaluate to the correct date. Otherwise, separate DB accesses
# would be required to generate the dates to be passed in as pure bind vars.
# However, note that bind vars are used for any dates being passed in directly
# from user input, e.g. start_date, end_date, and duration.
# For the values generated by calendar_form_transform_datetime (used to add or
# edit items), these dates are checked with a select from dual and the resulting
# values are passed via bind variables.
#
# Bind variables are used for all other variables in SQL statements.


# For a more user-friendly error log...
util_report_library_entry


# This appears to be a debugging proc, not actually used in the implementation
proc my_ns_return { msg } {
    ns_return 200 text/html "<html><title></title><body><hr><hr><pre>$msg</pre><hr><hr></body></html>"
    uplevel {
	return
    }
}


#
# Data entry validation routines
#

ad_proc calendar_validate_boolean { field_name string } {
    Validate boolean variable
} {
    if {$string != "t" && $string != "f"} {
	error "The entry for $field_name, \"$string\" is not a boolean"
    }
    return $string
}

ad_proc calendar_validate_number {field_name string} {
    Throws an error if the string isn't a number; 
    otherwise strips any leading zeros (so this won't work for octals) and returns the result.
} {
    if { ![regexp {^[0-9]+.?[0-9]*$} $string] } {
	error "The entry for $field_name, \"$string\" is not a number"
    }
    # trim leading zeros, so as not to confuse Tcl
    set string [string trimleft $string "0"]
    if { [empty_string_p $string] } {
	# but not all of the zeros
	return "0"
    }
    return $string
}

ad_proc calendar_parse_oracle_error_for_null_constraint_violation { {} error_message } {
    This function parses an oracle error message looking for a violation of a null constraint.
    It will return the name of the first violating column, or an empty string if no violation found.
} {
    if {[regexp {cannot insert NULL[^.]*.[^.]*.\"([^\"]*)\"} $error_message x constraint_column]} {
	return $constraint_column
    } else {
	return ""
    }
}

ad_proc calendar_parse_oracle_error_for_unique_constraint_violation { {} error_message } {
    This function parses an oracle error message looking for a violation of a unique constraint.
    It will return 1 if a violation is found, 0 otherwise.
} {
    if {[regexp {unique constraint \(([^\)]*)\) violated} $error_message x constraint_name]} {
	return 1
    } else {
	return 0
    }
}

ad_proc cal_mini_calendar_widget { 
    {} base_url {current_view ""} {current_date ""} {group_id 0}} {
	"This proc creates a mini calendar useful for navigating various 
	calendar views.  It takes a base url, which is the url to which this 
	mini calendar will navigate.  This proc will set 2 variables in that
	url's environment: the current view and the current date.  Valid views 
	are list, day, week, month, and year.  
	The current_date must be formatted YYYY-MM-DD."} {
    #valid views are "list" "day" "week" "month" "year"
    if {![exists_and_not_null current_view]} {
	set current_view "day"
    }

    if {![exists_and_not_null current_date]} {
	set current_date [db_string cal_sysdate "select sysdate from dual"]
    }

    #get the current month, day, and the first day of the month
    db_1row cal_get_dates "select
    trim(to_char(sysdate, 'Month')) || ' ' || to_char(sysdate, 'DD, YYYY') as pretty_today,
    trim(to_char(month.current_month, 'Month')) as this_month,
    sysdate as today,
    to_char(next_year.year, 'YYYY') as next_year_year,
    next_year.year as next_year,
    to_char(prev_year.year, 'YYYY') as prev_year_year,
    prev_year.year as prev_year,
    next_month.month as next_month,
    prev_month.month as prev_month,
    to_char(next_month.month, 'YYYY') as next_month_year,
    to_char(next_month.month, 'MM') as next_month_month,
    to_char(next_month.month, 'DD') as next_month_day,
    to_char(prev_month.month, 'YYYY') as prev_month_year,
    to_char(prev_month.month, 'MM') as prev_month_month,
    to_char(prev_month.month, 'DD') as prev_month_day,
    month.current_month,
    to_char(month.current_month, 'MM') as current_month_num,
    to_char(day.current_day, 'fmDD') as current_day,
    to_char(day.current_day, 'YYYY') as current_year,
    to_char(trunc(month.current_month, 'DD'), 'fmDay') as month_start,
    to_char(last_day(month.current_month), 'dd') as last_day,
    trim(to_char(month.current_month, 'Month')) || ' ' || trim(to_char(month.current_month, 'YYYY')) as pretty_month,
    to_char(last_day(add_months(trunc(month.current_month, 'DD'),-1 )), 'dd') as prev_month_last_day
    from
    (select
    add_months(:current_date, 12) as year
    from dual) next_year,
    (select 
    add_months(:current_date, -12) as year
    from dual) prev_year,
    (select
    add_months(:current_date, 1) as month
    from dual) next_month,
    (select
    add_months(:current_date, -1) as month
    from dual) prev_month,
    (select trunc(to_date(:current_date), 'Month')
     as current_month
     from dual) month,
    (select trunc(to_date(:current_date), 'DD')
     as current_day
     from dual) day"

    #detect which view I'm in and show the appropriate box
    set view_nav_html "
    <tr align=center>    
     [ad_decode $current_view "list" "<td BGCOLOR=FFD700>" "<td>"]    
    <a href=\"$base_url?current_view=list&current_date=[ns_urlencode $current_date]&group_id=$group_id\">
    <font size=-1 color=blue>List</font></a>
    </td>
     [ad_decode $current_view "day" "<td BGCOLOR=FFD700>" "<td>"]    
    <a href=\"$base_url?current_view=day&current_date=[ns_urlencode $current_date]&group_id=$group_id\">
    <font size=-1 color=blue>Day</font></a>
    </td>
     [ad_decode $current_view "week" "<td BGCOLOR=FFD700>" "<td>"]
    <a href=\"$base_url?current_view=week&current_date=[ns_urlencode $current_date]&group_id=$group_id\">
    <font size=-1 color=blue>Week</font></a>
    </td>
     [ad_decode $current_view "month" "<td BGCOLOR=FFD700>" "<td>"]    
    <a href=\"$base_url?current_view=month&current_date=[ns_urlencode $current_date]&group_id=$group_id\">
    <font size=-1 color=blue>Month</font></a>
    </td>
     [ad_decode $current_view "year" "<td BGCOLOR=FFD700>" "<td>"]
    <a href=\"$base_url?current_view=year&current_date=[ns_urlencode $current_date]&group_id=$group_id\">
    <font size=-1 color=blue>Year</font></a>
    </td>
    </tr>
    "

    set return_html "
    <table border=1 cellpadding=2 cellspacing=0 width=160>

    $view_nav_html
    "

    #if this is a month or year view, show the current year in the main bar
    if {([string compare $current_view "month"] == 0) || ([string compare $current_view "year"] == 0)} {
		append return_html "
	<tr><TD NOWRAP ALIGN=CENTER bgcolor=lavender colspan=5>
        <a href=\"$base_url?current_view=$current_view&current_date=[ns_urlencode $prev_year]&group_id=$group_id\">
        <img border=0 width=11 height=11 src=\"/calendar/graphics/left\"></a>
        <TT><B>$current_year</B></TT>
	<a href=\"$base_url?current_view=$current_view&current_date=[ns_urlencode $next_year]&group_id=$group_id\">
        <img border=0 width=11 height=11 src=\"/calendar/graphics/right\"></a>
	</TD></tr>"
    }

    if {[string compare $current_view "month"] == 0} {
	#month view
	append return_html "
	<tr><td colspan=5>
	<TABLE BGCOLOR=ffffff CELLSPACING=3 CELLPADDING=1 BORDER=0>
	<tr>
	"

	set months_list [list January February March April May June July August September October November December]
	set months_days [list 31 28 31 30 31 30 31 31 30 31 30 31]
	set i 0
	while {$i < 12} {
	    set month [lindex $months_list $i]

	    #show 3 months in a row
	    if {([expr int(fmod($i, 3))] == 0) && ($i != 0)} {
		append return_html "</tr><tr>"
	    }
	    
	    if {[string compare $month $this_month] == 0} {
		append return_html "
		<td>
		<font size=-1><b>$month</b></font>
		</td>
		"
	    } else {
		set new_month_mon "[expr $i + 1]"
		if {[string length $new_month_mon] == 1} {
		    set new_month_mon "0$new_month_mon"
		} 
		set last_month_day [lindex $months_days $i]
		if {$current_day > $last_month_day} {
		    set new_current_day $last_month_day
		} else {
		    set new_current_day $current_day
		}
		set new_month "$current_year-$new_month_mon-$new_current_day"
		set month_link "
		<a href=\"$base_url?current_view=$current_view&current_date=[ns_urlencode $new_month]&group_id=$group_id\">
		<font size=-1 color=blue>$month</font></a>"

		append return_html "<td>$month_link</td>"
	    }
	
	    incr i	    
	}
	
	append return_html "</tr>"	    

	
    } elseif {[string compare $current_view "year"] == 0} {
	#year view
	append return_html "
	<tr><td colspan=5>
	<TABLE BGCOLOR=ffffff CELLSPACING=3 CELLPADDING=1 BORDER=0>
	<tr>
	"

	set i [expr $current_year - 2]
	set end_year [expr $current_year + 2]
	set count 0

	while {$i <= $end_year} {
	    
	    if {[string compare $i $current_year] == 0} {
		append return_html "
		<td>
		<font size=-1><b>$i</b></font>
		</td>
		"
	    } else {
		set new_year "$i-$current_month_num-$current_day"
		append return_html "
		<td>
		<a href=\"$base_url?current_view=$current_view&current_date=[ns_urlencode $new_year]&group_id=$group_id\"><font size=-1 color=blue>$i</font></a></td>"

	    }

	    incr count	    
	    incr i
	}
	
	append return_html "</tr>"
	
    } else {
	#list, day, week view
	append return_html "
	<tr><TD NOWRAP ALIGN=CENTER bgcolor=lavender colspan=5>
        <a href=\"$base_url?current_view=$current_view&current_date=[ns_urlencode $prev_month]&group_id=$group_id\">
        <img border=0 width=11 height=11 src=\"/calendar/graphics/left\"></a>
        <TT><B>$pretty_month</B></TT>
	<a href=\"$base_url?current_view=$current_view&current_date=[ns_urlencode $next_month]&group_id=$group_id\">
        <img border=0 width=11 height=11 src=\"/calendar/graphics/right\"></a>
	</TD></tr>

	<tr><td colspan=5>
	<TABLE BGCOLOR=ffffff CELLSPACING=3 CELLPADDING=1 BORDER=0>
	"

	#get all the sundays then all the mondays, etc
	#and puts <br> tags between the dates

	#make a week list so that I can translate days into numbers (with lindex)
	set week_list [list "Sunday" "Monday" "Tuesday" "Wednesday" "Thursday" "Friday" "Saturday"]
	
	#map indices from week_list to the lists below
	set week_map [list "sundays" "mondays" "tuesdays" "wednesdays" "thursdays" "fridays" "saturdays"]

	#these lists hold their respective dates.  So, sundays holds the dates of
	#all the sundays in the month
	set sundays "<td align=right><font size=-1><b>Su</b>"
	set mondays "<td align=right><font size=-1><b>Mo</b>"
	set tuesdays "<td align=right><font size=-1><b>Tu</b>"
	set wednesdays "<td align=right><font size=-1><b>We</b>"
	set thursdays "<td align=right><font size=-1><b>Th</b>"
	set fridays "<td align=right><font size=-1><b>Fr</b>"
	set saturdays "<td align=right><font size=-1><b>Sa</b>"

	#Use the first day of the month and assign 1 to its list.
	#For all the days before that day, pad the first index
	#with days from the last month.
	#Then, fill in all the rest of the lists

	set first_day_index [lsearch -exact $week_list $month_start]

	set pad_index $first_day_index
	while {$pad_index > 0} {
	    #pad the lists with days from the last month as necessary
	    set pad_index [expr $pad_index -1]

	    set day_list [lindex $week_map $pad_index]

	    set day_date "$prev_month_year-"
	    append day_date "$prev_month_month-"
	    if {[string length $prev_month_last_day] == 1} {
		append day_date "0$prev_month_last_day"
	    } else {
		append day_date "$prev_month_last_day"
	    }
	    set day_date_link "<a href=\"$base_url?current_view=$current_view&current_date=[ns_urlencode $day_date]&group_id=$group_id\"><font color=gray>$prev_month_last_day</font></a>"

	    append $day_list "<br>$day_date_link"

	    set prev_month_last_day [expr $prev_month_last_day - 1]
	}

	set i 1
	#we want to use 1-based indexing for the day_index so that we
	#can distinguish the first and last day of the week using the
	#mod function.
	set day_index [expr $first_day_index +1]
	while {$i <= $last_day} {
	    #fill in the lists
	    
	    #convert day_index back to 0-based indexing for lindex
	    set day_list [lindex $week_map [expr $day_index - 1]]

	    set day_date "$current_year-"
	    append day_date "$current_month_num-"
	    if {[string length $i] == 1} {
		append day_date "0$i"
	    } else {
		append day_date "$i"
	    }

	    set day_date_link "<a href=\"$base_url?current_view=$current_view&current_date=[ns_urlencode $day_date]&group_id=$group_id\"><font color=blue>$i</font></a>"

	    if {$i == $current_day} {
		#this is the current day, so format it differently
		append $day_list "<br><b>$i</b>"
	    } else {
		append $day_list "<br>$day_date_link"
	    }

	    incr i

	    set day_index [expr int(fmod($day_index+1, 7))]
	    if {$day_index == 0} {
		set day_index 7
	    }

	}   

	#special case: if day_index is 1, then the last day
	#was saturday since day_index was updated at the end
	#of the while loop.  So, we don't want to fill
	#in the rest of the week
	if {$day_index == 1} {
	    set day_index 8
 	}


	#pad the rest of the weekdays with dates from the following month
	set i 1
	while {$day_index <= 7} {
	    set day_list [lindex $week_map [expr $day_index - 1]]

	    set day_date "$next_month_year-"
	    append day_date "$next_month_month-"
	    if {[string length $i] == 1} {
		append day_date "0$i"
	    } else {
		append day_date "$i"
	    }
	    set day_date_link "<a href=\"$base_url?current_view=$current_view&current_date=[ns_urlencode $day_date]&group_id=$group_id\"><font color=gray>$i</font></a>"

	    append $day_list "<br>$day_date_link"
	    incr day_index
	    incr i
	}

	#close the weekday lists
	set i 0
	while {$i < 7} {
	    set day_list [lindex $week_map $i]
	    append $day_list "</font></td>\n"
	    incr i
	}

	append return_html "<tr valign=top>" $sundays $mondays $tuesdays $wednesdays $thursdays $fridays $saturdays "</tr>"

    }
    set today_url "$base_url?current_view=$current_view&current_date=[ns_urlencode $today]&group_id=$group_id"

    append return_html "
    <tr><td align=center colspan=7>
        <form method=get action=\"$base_url\">
        [philg_hidden_input current_view $current_view]
        [philg_hidden_input group_id $group_id]
        <center>
        <font size=-1>
        <input type=text size=11 name=current_date value=\"$current_date\">
        <input type=submit value=\"Go\"><br>
        (YYYY-MM-DD Format)
        </font>
        </center>
        </form>
    </td></tr>

    </table>
    </td></tr>
    
    <tr><td align=center bgcolor=lavender colspan=5>
     <font size=-1><a href=\"$today_url\"><font color=blue><b>Today</b></font></a>
    is $pretty_today</font>
    </td></tr>

    </table>
    "

    return $return_html
}

ad_proc cal_mini_month_calendar {current_year month {group_id 0} {view_mode "full"}} {
    if {[string compare $view_mode "modular"] == 0} {
	set month_link "?current_view=month&view_mode=modular&"
	set day_link "?current_view=day&view_mode=modular&"
    } else {
	#view_mode is "full"
	set month_link "?current_view=month&view_mode=full&"
	set day_link "?current_view=day&view_mode=full&"
    }

    #month is the 'MM' form of the current month
    #current_date is 'YYYY' format of current year
     
    set month_txt "$current_year-$month-01"

    db_1row cal_month_dates "select
    to_char(to_date(:month_txt), 'fmMonth') as this_month,
    to_char(last_day(to_date(:month_txt)), 'dd') as last_day,
    to_char(to_date(:month_txt), 'MM') as current_month_num,
    to_char(trunc(to_date(:month_txt), 'Month'), 'fmDay') as month_start,
    to_char(to_date(:month_txt), 'fmMonth') as this_month,
    to_char(add_months(to_date(:month_txt), 1), 'MM') as next_month_month,
    to_char(add_months(to_date(:month_txt), 1), 'YYYY') as next_month_year,
    to_char(last_day(add_months(to_date(:month_txt), -1)), 'dd') as prev_month_last_day,
    to_char(add_months(to_date(:month_txt), -1), 'YYYY') as prev_month_year,
    to_char(add_months(to_date(:month_txt), -1), 'MM') as prev_month_month
    from dual
    "

    set return_hml ""
    append return_html "
    <table border=1 cellpadding=1 cellspacing=0 >
    <tr align=center bgcolor=lavender>
    <td colspan=7>
    <a href=\"$month_link" "group_id=$group_id&current_date=$month_txt\">
    $this_month</a>
    </td>
    </tr>
    <tr>
    <td>
    <TABLE CELLSPACING=3 CELLPADDING=1 BORDER=0 width=100%>
    "
    
    #get all the sundays then all the mondays, etc
    #and puts <br> tags between the dates
    
    #make a week list so that I can translate days into numbers (with lindex)
    set week_list [list "Sunday" "Monday" "Tuesday" "Wednesday" "Thursday" "Friday" "Saturday"]
    
    #map indices from week_list to the lists below
    set week_map [list "sundays" "mondays" "tuesdays" "wednesdays" "thursdays" "fridays" "saturdays"]

    #these lists hold their respective dates.  So, sundays holds the dates of
    #all the sundays in the month
    set sundays "<td align=right><font size=-1><b>Su</b>"
    set mondays "<td align=right><font size=-1><b>Mo</b>"
    set tuesdays "<td align=right><font size=-1><b>Tu</b>"
    set wednesdays "<td align=right><font size=-1><b>We</b>"
    set thursdays "<td align=right><font size=-1><b>Th</b>"
    set fridays "<td align=right><font size=-1><b>Fr</b>"
    set saturdays "<td align=right><font size=-1><b>Sa</b>"

    #Use the first day of the month and assign 1 to its list.
    #For all the days before that day, pad the first index
    #with days from the last month.
    #Then, fill in all the rest of the lists

    set first_day_index [lsearch -exact $week_list $month_start]

    set pad_index $first_day_index
    while {$pad_index > 0} {
	#pad the lists with days from the last month as necessary
	set pad_index [expr $pad_index -1]

	set day_list [lindex $week_map $pad_index]

	set day_date "$prev_month_year-"
	append day_date "$prev_month_month-"
	if {[string length $prev_month_last_day] == 1} {
	    append day_date "0$prev_month_last_day"
	} else {
	    append day_date "$prev_month_last_day"
	}
	set day_date_link ""
	append day_date_link "<a href=\"$day_link" "group_id=$group_id&current_date=[ns_urlencode $day_date]\"><font color=gray>$prev_month_last_day</font></a>"

	append $day_list "<br>$day_date_link"

	set prev_month_last_day [expr $prev_month_last_day - 1]
    }

    set i 1
    #we want to use 1-based indexing for the day_index so that we
    #can distinguish the first and last day of the week using the
    #mod function.
    set day_index [expr $first_day_index +1]
    while {$i <= $last_day} {
	#fill in the lists
	
	#convert day_index back to 0-based indexing for lindex
	set day_list [lindex $week_map [expr $day_index - 1]]

	set day_date "$current_year-"
	append day_date "$current_month_num-"
	if {[string length $i] == 1} {
	    append day_date "0$i"
	} else {
	    append day_date "$i"
	}

	set day_date_link ""
	append day_date_link "<a href=\"$day_link" "group_id=$group_id&current_date=[ns_urlencode $day_date]\"><font color=blue>$i</font></a>"

	append $day_list "<br>$day_date_link"

	incr i

	set day_index [expr int(fmod($day_index+1, 7))]
	if {$day_index == 0} {
	    set day_index 7
	}

    }   

    #special case: if day_index is 1, then the last day
    #was saturday since day_index was updated at the end
    #of the while loop.  So, we don't want to fill
    #in the rest of the week
    if {$day_index == 1} {
	set day_index 8
    }

    #pad the rest of the weekdays with dates from the following month
    set i 1
    while {$day_index <= 7} {
	set day_list [lindex $week_map [expr $day_index - 1]]

	set day_date "$next_month_year-"
	append day_date "$next_month_month-"
	if {[string length $i] == 1} {
	    append day_date "0$i"
	} else {
	    append day_date "$i"
	}
	set day_date_link ""
	append day_date_link "<a href=\"$day_link" "group_id=$group_id&current_view=day&current_date=[ns_urlencode $day_date]\"><font color=gray>$i</font></a>"

	append $day_list "<br>$day_date_link"
	incr day_index
	incr i
    }

    #see if we need to add an extra row to the calendar
    set first_day_index [lsearch -exact $week_list $month_start]
    if {$first_day_index != 0} {
	#this is the date of the first sunday
	set first_sun [expr 8 - $first_day_index]
	set extra_week 1
    } else {
	set first_sun 1
	set extra_week 0
    }
    set num_weeks [expr int(ceil(double($last_day+1-$first_sun)/double(7)) + $extra_week)]
    if {$num_weeks < 6} {
	set i 0
	while {$i < 7} {
	    set day_list [lindex $week_map $i]
	    append $day_list "<br>&nbsp"
	    incr i
	}
    }

    #close the weekday lists
    set i 0
    while {$i < 7} {
	set day_list [lindex $week_map $i]
	append $day_list "</font></td>\n"
	incr i
    }

    append return_html "<tr valign=top>" $sundays $mondays $tuesdays $wednesdays $thursdays $fridays $saturdays "</tr></table></td></tr></table>"

}

ad_proc cal_year_calendar {
    {-group_id 0}
    {-view_mode "full"}
    current_year
    current_date
} {
    year_calendar proc
} {
    set return_html ""

    set i 1
    while {$i <= 12} {
	if {[string length $i] == 1} {
	    set month "0$i"
	} else {
	    set month $i
	}

	if {[expr int(fmod([expr $i -1], 3))] == 0} {
	    if {$i != 1} {
		append return_html "</tr>"
	    }
		
	    append return_html "<tr valign=top>"
	}

	append return_html "<td>
	[cal_mini_month_calendar $current_year $month $group_id $view_mode]
	</td>
	"

	incr i
    }

    append return_html "</tr>"

    return $return_html	
}

###################################################
#BEGIN PROCS FOR CREATING A DAY VIEW
###################################################

#procs for a table cell object

#a cell is a list of the following items:
#root start location in table
#name
#rowspan
#colspan
#ex: {2 3} {<a href="blah.tcl">name</a>} 3 2

ad_proc create_cell {i j text rowspan colspan } {
    return [list [list $i $j] $text $rowspan $colspan] 
}

ad_proc cell_get_root {cell} {
    return [lindex $cell 0]
}

ad_proc cell_get_text {cell} {
    return [lindex $cell 1]
}

ad_proc cell_get_rowspan {cell} {
    return [lindex $cell 2]
}

ad_proc cell_get_colspan {cell} {
    return [lindex $cell 3]
}
###
ad_proc cell_set_root {cell i j} {
    return [lreplace $cell 0 0 [list $i $j]]
}

ad_proc cell_set_text {cell text} {
    return [lreplace $cell 1 1 $text]
}

ad_proc cell_set_rowspan {cell rowspan} {
    return [lreplace $cell 2 2 $rowspan]
}

ad_proc cell_set_colspan {cell colspan} {
    return [lreplace $cell 3 3 $colspan]
}

##############################

##############################
#procs for making a table

ad_proc create_table {{num_rows 1} {num_cols 1}} {

    if {$num_rows < 1} {
	set num_rows 1
    }

    if {$num_cols < 1} {
	set num_cols 1
    }

    set table [list]
    
    set i 0
    while {$i < $num_cols} {
	set col [list]
	set j 0
	while {$j < $num_rows} {
	    set cell [create_cell $i $j "" -1 -1]
	    
	    lappend col $cell
	    incr j
	}
	lappend table $col
	incr i
    }

    return $table
}

#i: col; j: row
ad_proc table_get_cell {table i j} {
    return [lindex [lindex $table $i] $j]
}

ad_proc table_set_cell {table i j cell} {
    set col [lindex $table $i]
    set col [lreplace $col $j $j $cell]
    return [lreplace $table $i $i $col]
}

ad_proc table_add_col {table} {
    set num_cols [llength $table]
    set num_rows [llength [lindex $table 0]]

    set i 0
    set new_col [list]
    while {$i < $num_rows} {
	set cell [create_cell $num_cols $i "" 1 1]
	
	lappend new_col $cell

	incr i
    }

    return [lappend table $new_col]
}

ad_proc table_num_cols {table} {
    return [llength $table]
}

ad_proc table_num_rows {table} {
    return [llength [lindex $table 0]]
}

#returns the index of the first empty column in a row
#returns -1 if there is no empty column
ad_proc table_first_empty_col {table row} {
    set num_cols [llength $table]

    set i 0
    while {$i < $num_cols} {
	set cell [table_get_cell $table $i $row]
	if {[cell_get_text $cell] == ""} {
	    return $i
	}
	incr i
    }
    
    return -1
}

ad_proc table_next_filled_col {table row {col 0}} {
    set num_cols [llength $table]

    set i $col
    while {$i < $num_cols} {
	set cell [table_get_cell $table $i $row]
	if {[cell_get_text $cell] != ""} {
	    return $i
	}
	incr i
    }
    
    return -1
}

#returns how many columns are left to write in this row, starting with col
ad_proc table_filled_cols_left {table row {col 0}} {
    set num_cols [llength $table]

    set return_num 0

    set i $col
    while {$i < $num_cols} {
	set cell [table_get_cell $table $i $row]
	if {![empty_string_p [cell_get_text $cell] ]} {
	    incr return_num
	}
	incr i
    }
    return $return_num
}

ad_proc table_print_table {table} {
    set return_html "<table border=1>"
    set cols [table_num_cols $table]
    set rows [table_num_rows $table]
    set i 0
    while {$i < $rows} {
	set j 0
	append return_html "<tr>"
	while {$j < $cols} {
	    set cell [table_get_cell $table $j $i]
	    append return_html "<td nowrap><b>($j,$i)</b>$cell</td>"
	    incr j
	}
	append return_html "</tr>"
	incr i
    }
    append return_html "</table>"

    return $return_html
}

# This is a quick hack to support the intranet/absences monthly calendar
# Name will either be a single space, in which case it should not be
# displayed, or it will be of the form 
# "<first_names> <first char of last_name>"
proc cal_related_link_html {title name url text} {
    set result "<a href=\"$url\">"
    if [empty_string_p $text] {
	append result "$title"
    } else {
	append result "$text"
    }
    if {[string length $name] > 1} {
	append result " - $name"
    }
    append result "</a>"
    return $result
}

ad_proc calendar_day_view {{} current_date logged_in_user_id group_id {compress_day_view_p "t"} {begin_cal_hour 0} {end_cal_hour 23} {add_link "?current_view=day&"} {view_mode "modular"}} {
    returns a calendar day view
} {
    # Why was this here?  num_hour_rows will always be 24.
    # Maybe this was meant to select on hours_span from cal_user_prefs?
    # Anyway, just set it to 24 and figure this out later.
#      db_1row cal_hours_select "select
#      count(*) as num_hour_rows
#      from cal_hours"

    set num_hour_rows 24

    set table [create_table $num_hour_rows 1]

    if {$group_id == 0} {
	# user calendar
	set user_or_group_sql "and (m.user_id = :logged_in_user_id
                or m.group_id in 
                   (select ugm.group_id
                    from user_group_map ugm, cal_user_update_prefs ugp
                    where ugm.user_id = :logged_in_user_id
                      and ugp.user_id = ugm.user_id
                      and ugm.group_id = ugp.group_id
                      and ugp.update_type = 'automatically')
                )
	"
    } else {
	# Group calendar
	set user_or_group_sql "and m.group_id in (select ug2.group_id
                                                  from user_groups ug2
                                                  start with ug2.group_id= :group_id
                                                  connect by prior ug2.group_id = ug2.parent_group_id)"
    }
    db_foreach cal_get_time_items "
    select /*+ RULE */
    m.group_id as item_group_id, ug.short_name as item_group_name, ug.short_name as item_group_short_name, 
    ug.group_type as item_group_type,
    combine_type_names(ii.item_id, ', ') as type_list,
    ii.item_id, ii.parent_id,
    to_char(ii.start_date, 'fmHH24') as hour,
    to_char(ii.end_date, 'fmHH24') as end_hour,
    to_char(ii.end_date, 'fmMI') as end_minute,
    to_char(ii.start_date, 'YYYY-MM-DD HH24:MI') as start_time,
    to_char(ii.start_date, 'fmHH:fmMIam') as pretty_start_time,
    to_char(ii.end_date, 'YYYY-MM-DD HH24:MI') as end_time,
    to_char(ii.end_date, 'fmHH:fmMIam') as pretty_end_time,
    ii.start_date, ii.end_date,
    to_char(ii.end_date, 'fmMonth fmDD, YYYY') as pretty_end_day,
    decode(m.user_id, NULL, 'f', 't') as user_p,
    nvl(ii.title, i.title) as title,
    nvl(ii.related_link_url, i.related_link_url) as related_link_url, 
    nvl(ii.related_link_text, i.related_link_text) as related_link_text, 
    nvl(ii.redirect_to_related_link_p, i.redirect_to_related_link_p) as redirect_to_related_link_p
    from cal_items i, cal_items ii, 
         cal_user_group_item_map m, user_groups ug
    where ug.group_id(+) = m.group_id
      and i.item_id = ii.parent_id
      and ii.end_date is not NULL
      and ii.start_date between 
          to_date(:current_date) and
          to_date(:current_date) + (24 - 1/3600)/24
      and i.item_id = m.item_id
      $user_or_group_sql
    order by ii.start_date"  {
	#if there is a calendar item at this time...
	if {[exists_and_not_null title]} {
	    if {[string compare $start_date $end_date] == 0} {
		if {($end_minute > 0) && ($end_hour < $end_cal_hour)} {
		    set end_index [expr $end_hour + 1]
		} else {
		    set end_index $end_hour
		}
	    } else {
		set end_index 24
		#set end_index [expr $end_hour + 1]
	    }

	    #the number of rows this item takes
	    set row_span [expr $end_index - $hour]
	    if {$row_span < 1} {
		set row_span 1
		incr end_index
	    }
	    #i is the row, col_index is the col

	    #find out in what table column this cell belongs
	    set col_index 0
	    #iterate through the times for this item
	    set i $hour
	    while {$i < $end_index} {
		set tmp_col_index [table_first_empty_col $table $i]

		if {$tmp_col_index == -1} {
		    #add a new column to the table--no room for
		    #this entry
		    set table [table_add_col $table]
		    set col_index [expr [table_num_cols $table] - 1]
		    break
		}
		if {$tmp_col_index > $col_index} {
		    set col_index $tmp_col_index
		}
		incr i
	    }

	    #now, actually create the table cell
	    set action "item-view"
	    if {$start_date == $end_date} {
		set item_text "$pretty_start_time - $pretty_end_time"
	    } else {
		set item_text "$pretty_start_time - $pretty_end_day $pretty_end_time"
	    }
	    if {$redirect_to_related_link_p == "t"} {
		append item_text "<a href=\"$related_link_url\">"
		if [empty_string_p $related_link_text] {
		    append item_text "$title"
		} else {
		    append item_text "$related_link_text"
		}
	    } else {
		append item_text " <a href=\"[ad_calendar day_link][export_url_vars action item_id parent_id current_date view_mode_var]&group_id=$item_group_id\">$title</a>"
	    }
	    # ejc@arsdigita.com 7-27-2000
	    # if this is NOT a group portal, show the group name
	    # if types exist, show them in parentheses
  	    if {$group_id == 0 && ![empty_string_p $item_group_short_name]} {
		set group_home_link $item_group_name
	    } else {
		if { $group_id == $item_group_id  } {
		    set group_home_link ""
		} else {
		    set group_home_link $item_group_name
		}
	    }

	    if ![empty_string_p $type_list] {
		set item_type_list [list $type_list]
	    } else {
		set item_type_list ""
	    }
	    if {![empty_string_p $group_home_link] || ![empty_string_p $item_type_list]} {
		append item_text "
  		([join "$group_home_link $item_type_list" ": "])
  		"
	    }

	    set cell [create_cell $col_index $hour $item_text $row_span -1]
	    
	    set i $hour

	    while {$i < $end_index} {
		set table [table_set_cell $table $col_index $i $cell]  	    
		incr i
	    }
	}
    }

    #now generate the html table
    # This query should be turned into a proc and memoized.
    # In which case, sysdate needs to be pulled out separately.
    db_1row cal_current_days "
    select
    to_char(to_date(:current_date), 'fmDay fmMonth, fmDD YYYY') as pretty_current_day,
    to_date(:current_date) - 1 as previous_day,
    to_date(:current_date) + 1 as next_day,
    to_char(sysdate, 'fmHH24') as current_hour
    from dual
    "
    set return_html ""

    #add all-day items first
    set num_cols [table_num_cols $table]
    set all_day_html ""
    set add_link_p 1
    if {$group_id != 0} {
	if ![ad_calendar group_admin_p] {
	    set add_link_p 0
	}
    }

    if { $add_link_p } {
	append all_day_html "
	<tr>
	<td valign=top nowrap width=10%>
	<a href=\"[ad_calendar add_link]" "[ad_calendar view_mode_var]" "current_date=$current_date&start_date=$current_date%2000:00&end_date=&current_view=day&action=item-add\">No Time</a>
	</td>
	"
    } else {
	append all_day_html "
	<tr>
	<td valign=top nowrap width=10%>
	No Time
	</td>
	"
    }
	
    set all_day_items "
    <ul>"
    set count 0

    db_foreach cal_no_time_items "
    select
    m.group_id as item_group_id, ug.short_name as item_group_name, ug.short_name as item_group_short_name, 
    ug.group_type as item_group_type,
    combine_type_names(ii.item_id, ', ') as type_list,
    ii.item_id, ii.parent_id,
    decode(m.user_id, NULL, 'f', 't') as user_p,
    nvl(ii.title, i.title) as title,
    upper(nvl(ii.title, i.title)),
    nvl(ii.related_link_url, i.related_link_url) as related_link_url, 
    nvl(ii.related_link_text, i.related_link_text) as related_link_text, 
    nvl(ii.redirect_to_related_link_p, i.redirect_to_related_link_p) as redirect_to_related_link_p
    from cal_items i, cal_items ii, cal_user_group_item_map m, user_groups ug
    where ii.start_date =  to_date(:current_date)
      and ug.group_id(+) = m.group_id
      and i.item_id = ii.parent_id
      and i.item_id = m.item_id
      and ii.end_date is NULL
      $user_or_group_sql
    order by upper(nvl(ii.title, i.title))" {
	append all_day_items "<li>"
	
	if {"$redirect_to_related_link_p" == "t"} {
	    append all_day_items "[cal_related_link_html $title "" $related_link_url $related_link_text]"
	} else {
	append all_day_items "
	    <a href=\"[ad_calendar day_link]" "[ad_calendar view_mode_var]" "action=item-view&item_id=$item_id&parent_id=$parent_id&current_date=$current_date&group_id=[ad_calendar group_id]&current_view=day\">$title</a>"
	}

	if {$group_id == 0 && ![empty_string_p $item_group_short_name]} {
	    set group_home_link [list [cal_group_home_link $item_group_id $item_group_type $item_group_short_name $item_group_name]]
	} else {
	    set group_home_link ""
	}
	if ![empty_string_p $type_list] {
	    set item_type_list [list $type_list]
	} else {
	    set item_type_list ""
	}
	if {![empty_string_p $group_home_link] || ![empty_string_p $item_type_list]} {
	    append all_day_items "
	    ([join "$group_home_link $item_type_list" ": "])
	    "
	}
	incr count
    }

    set colspan [table_num_cols $table]
    if {$count > 0} {
	append all_day_html "<td colspan=$colspan>$all_day_items </ul>"
    } else {
	append all_day_html "<td bgcolor=\"DCDCDC\" colspan=$colspan>&nbsp;"
    }
    append all_day_html "</td></tr>"    

    if {([string compare $compress_day_view_p "f"] == 0) || (([string compare $compress_day_view_p "t"] == 0) && ($count > 0))} {
	append return_html $all_day_html
    }

    set i 0
    while {$i < $num_hour_rows} {
	set filled_cell_count 0
	set row_html ""
	set bgcolor_html ""
	if {$i == $current_hour} {
	    set bgcolor_html "bgcolor=\"FFF8DC\""
	}
	if {$i < 12} {
	    if {$i == 0} {
		set time "12:00 am"
	    } else {
		set time "$i:00 am"
	    }
	} else {
	    if {$i == 12} {
		set time "12:00 pm"
	    } else {
		set time "[expr $i - 12]:00 pm"
	    }	 
	}

	if {$i < 10} {
	    set fm_hour "0$i"
	} else {
	    set fm_hour "$i"
	}
	    
	set encoded_start_time [ns_urlencode "$current_date $fm_hour:00"]

	if { $add_link_p } {
	    append row_html "
	    <tr>
	    <td valign=top nowrap $bgcolor_html width=10%>
	    <a href=\"[ad_calendar add_link]" "[ad_calendar view_mode_var]" "current_date=$current_date&start_date=$encoded_start_time&action=item-add&current_view=day\">
	    $time</a></td>
	    "
	} else {
	    append row_html "
	    <tr>
	    <td valign=top nowrap $bgcolor_html width=10%>
	    $time</td>
	    "
	}

	set j 0
	set total_cols 0
	while {$j < $num_cols} {
	    set cell [table_get_cell $table $j $i]
	    set cell_text [cell_get_text $cell]
	    
	    #need to check if there is room for this empty cell
	    set filled_left [table_filled_cols_left $table $i $j] 
	    set filled_check [expr $num_cols - $filled_left - $total_cols]

	    #see if we need to write an empty table cell
	    if {[empty_string_p $cell_text] && ($filled_check >0)} {
		#this cell is empty, so we may write an empty tag
		
		#see how long to make the colspan for this cell
		set filled_index [table_next_filled_col $table $i $j]
		if {$filled_index == -1} {
		    set colspan [expr $num_cols - $j]
		} else {
		    set colspan [expr abs($j - $filled_index)]
		}

		if {$colspan > 0} {
		    append row_html "<td colspan=$colspan bgcolor=\"DCDCDC\">&nbsp;</td>"
		    set total_cols [expr $total_cols + $colspan]
		}
	    }

	    #see if we write the contents of this cell
	    set cell_root [cell_get_root $cell]
	    if {![empty_string_p $cell_text]} {
		incr filled_cell_count
		
		#get the rowspan
		set rowspan [cell_get_rowspan $cell]
		
		#see how long to make the colspan for this cell

		#start at the root row
		set k [lindex [cell_get_root $cell] 1]
		#go until this cell ends
		set cell_end_row [expr $k + [cell_get_rowspan $cell]]
		set filled_index [table_num_cols $table]
		while {$k < $cell_end_row} {
		    set tmp_filled_index [table_next_filled_col $table $k [expr $j + 1]]
		    if {($tmp_filled_index < $filled_index) && ($tmp_filled_index != -1)} {
			set filled_index $tmp_filled_index
		    }
		    incr k
		}

		if {$filled_index == -1} {
		    set colspan [expr $num_cols - $j]
		} else {
		    set colspan [expr $filled_index - $j]
		}

		#this is the root cell, so write its contents
		if {([lindex $cell_root 0] == $j) && ([lindex $cell_root 1] == $i)} {
		    append row_html "
		    <td valign=top rowspan=$rowspan colspan=$colspan>
		    $cell_text
		    </td>\n"	
		}

		set total_cols [expr $total_cols + $colspan]
	    }
	    incr j
	}
	append row_html "</tr>"
	
	if {($filled_cell_count > 0) || (([string compare $compress_day_view_p "f"] == 0) && ($i >= $begin_cal_hour) && ($i <= $end_cal_hour))} {
	    append return_html $row_html
	} 



	incr i    
    }

    if {![empty_string_p $return_html]} {
	set return_html "
	<table border=1 cellpadding=2 cellspacing=0 width=90%>
	$return_html
	</table>\n"
    }

    db_release_unused_handles 

    return $return_html
}

###################################################
#END PROCS FOR CREATING A DAY VIEW
###################################################

ad_proc -private calendar_validate_start_end {start_date end_date format} {
    Check that start_date and end_date resolve to valid dates.
    Return a list of the validate dates.
    This also keeps hacked sql from setting any variables as a side-effect
    of the select in the calling procs context.
} {
    db_1row cal_date_valid "
    select to_char($start_date, $format) as valid_start_date, 
           to_char($end_date, $format) as valid_end_date
    from dual
    "
    return [list $valid_start_date $valid_end_date]
}

ad_proc calendar_form_process { } {
    Process calendar form.
    We do an insert or an update based on new_item_p.
    If the insert fails, we assume a double-click.
    If that has no results, we do an update.  If that fails,
    we display an error message.
    We check that the user either owns the item being modified or
    is an admin for the specified group_id.
} {
    # Note that start_date and end_date widgets are handled
    # by a seperate proc, so they are not grabbed here.
    ad_page_variables { 
	parent_id
	item_id
	title
	description
	desc_html
	{all_day_p "f"}
	{duration ""}
	{repeat_p ""}
	{update_current_or_all ""}
	pass_through_current_view
	pass_through_current_date
	{group_id "[db_null]"}
	{user_id "[db_null]"}
	{scope ""}
	{new_item_p "f"}
	{return_url ""}
	{type_id_list -multiple-list}
    }

    set date_format_string "'YYYY-MM-DD HH24:MI'"

    set logged_in_user_id [ad_verify_and_get_user_id]
    set creator_id $logged_in_user_id
		    
    page_validation {
	if {$group_id != "[db_null]"} {
	    set group_id [validate_integer "Group ID" $group_id]
	    # quick hack to get around group_id being set to 0 for no group...
	    if {$group_id == 0} {
		set group_id "[db_null]"
	    }
	}
	if {$user_id != "[db_null]" } {
	    set user_id [validate_integer "User ID" $user_id]
	}
	set parent_id [validate_integer "Calendar ID" $parent_id]
	set item_id [validate_integer "Calendar ID" $item_id]
	if {[exists_and_not_null duration]} {
	    set duration [calendar_round_to_precision [calendar_validate_number "Duration" $duration] 5]
	}
	calendar_validate_boolean "No Time" $all_day_p
	if {! [exists_and_not_null title]} {
	    error "<li>You did not enter a title"
	}
	if {[empty_string_p $update_current_or_all] &&
	    $new_item_p == "f" &&
            [db_string cal_repeat_exists "
            SELECT 1
            FROM cal_items
            WHERE item_id = :parent_id
	      AND interval_type IS NOT NULL" -default 0]} {
	    error "<li>This is a repeating item.  You must choose to either update only the current instance or all instances."
	}
    }

    # See DATE-NOTE at top of file.
    # Get start_date, end_date
    if {$all_day_p == "t"} {
	# start_date is truncated to just the date due to the "t" parameter
	set start_date [lindex [calendar_form_transform_datetime -truncate_p "t" "start_date"] 0]
	set end_date "NULL"
    } else {
	set start_date [lindex [calendar_form_transform_datetime -truncate_p "f" "start_date"] 0]
	if {[exists_and_not_null duration]} {
	    set end_date "($start_date + ($duration)/(24*60))"
	} else {
	    set end_date [lindex [calendar_form_transform_datetime -truncate_p "f" -use_as_fallback "start_date" "end_date"] 0]
	}
    }

    #make sure the start dates and end dates are valid
    if {[catch {calendar_validate_start_end $start_date $end_date $date_format_string} dates]} {
	return [ad_return_complaint 1 "<li>You have entered an invalid time"]
    } else {
	set valid_start_date [lindex $dates 0]
	set valid_end_date [lindex $dates 1]
    }

    #make sure that the start time is before the end time
    if [db_string cal_date_check "
    SELECT 1
    FROM dual
    WHERE to_date(:valid_start_date, $date_format_string) > 
          to_date(:valid_end_date, $date_format_string)" -default 0] {
	return [ad_return_complaint 1 "<li>Your start time must be before your end time"]
    }

    if {$scope == "group"} {
	set user_id "[db_null]"
    }
    # Deal with formatting description
    switch $desc_html {
 	pre {
 	    set description "<pre wrap>[ad_quotehtml $description]</pre>"
 	    set html_p "t"
 	}
 	plain {
 	    set html_p "f"
 	}
 	html {
 	    set html_p "t"
 	}
    }

    # NOTE: cal_user_group_item_map is currently not updatable using the UI
    #ns_log notice "Before transaction."
    db_transaction {
	# If this is a new item, try to do inserts
	# If the inserts fail due to a unique
	# constraint, assume a double click and switch to doing an update.
	set succeeded 1
	set security_error_p "f"
	if {$new_item_p == "t" &&
	    [catch {cal_insert_item -item_id $item_id -parent_id $parent_id \
		    -title $title \
		    -valid_start_date "$valid_start_date" \
	            -valid_end_date "$valid_end_date" \
		    -user_id $user_id -group_id $group_id \
		    -description $description -html_p $html_p \
	            -creator_id $creator_id} message] == 0} {
		# We should be all set, nothing to do at this point
		#ns_log notice "After insert."
	} else {
	    #ns_log notice "Insert failed."

	    # We either know we don't have a new item, or we had an error
	    # on our inserts... if it was a unique constraint error, 
	    # try doing an update instead.
	    if {$new_item_p == "f" ||
	    [calendar_parse_oracle_error_for_unique_constraint_violation $message]} {
		# We do some security checking here -anyone can add a new item, but only
		# certain people (owners or group admins) can modify already existing items.
		set write_access_p [calendar_item_write_access_p $parent_id $group_id]
		if {$write_access_p} {
		    # We've cleared security...

		    # NOTE that the cal_user_group_item_map is NOT
		    # presently updated in this case, so we
		    # can't change item ownership here.
		    
		    # Okay, this gets complicated with repeating items....
		    if {$update_current_or_all == "all" ||
		        $repeat_p == "f"} {
			
			# Select fields that were actually changed
			db_1row item_select "
			SELECT i.title as item_title,
			       ii.title as instance_title,
			       i.description as item_description,
			       ii.description as instance_description,
 			       i.html_p as item_html_p,
 			       ii.html_p as instance_html_p
			FROM cal_items i, cal_items ii
			WHERE i.item_id = :parent_id
			  AND i.item_id = ii.parent_id
			  AND ii.item_id = :item_id
			"

			# if the new title is the same as the instance title
			# or the item title if instance is null, then don't
			# change it
			if {([string compare $title $instance_title] == 0) ||
			    ([empty_string_p $instance_title] &&
			([string compare $title $item_title] == 0))} {
			    set update_item_title $item_title
			    set update_instance_title_sql "title = :instance_title"
			} else {
			    set update_item_title $title
			    set update_instance_title_sql "title = NULL"
			}
			# if the new description is the same as the instance 
			# description or the item description if instance is
			# null, then don't change it
			if {([string compare $description $instance_description] == 0) ||
			    ([empty_string_p $instance_description] &&
			     ([string compare $description $item_description] == 0))} {
			    set update_item_description $item_description
			    set update_instance_description_sql "description = :instance_description"
			} else {
			    set update_item_description $description
			    set update_instance_description_sql "description = NULL"
			}
 			# if the new html_p is the same as the instance 
 			# html_p or the item html_p if instance is
 			# null, then don't change it
 			if {([string compare $html_p $instance_html_p] == 0) ||
 			    ([empty_string_p $instance_html_p] &&
 			     ([string compare $html_p $item_html_p] == 0))} {
 			    set update_item_html_p $item_html_p
 			    set update_instance_html_p_sql "html_p = :instance_html_p"
 			} else {
 			    set update_item_html_p $html_p
 			    set update_instance_html_p_sql "html_p = NULL"
 			}
			# This may not be exactly what is wanted as it doesn't 
			# update individual instances that have been changed.  
			# If we did want to blast individualized instances
			# we'd need to do an update, setting these fields to null.
			db_dml cal_item_update_parent "
			UPDATE cal_items
			SET title=:update_item_title,
			description=:update_item_description,
 			html_p = :update_item_html_p,
			creator_id=:creator_id,
			creation_date=sysdate
			WHERE item_id=:parent_id"

			# Reset the current instance to nulls so that
			# it inherits properly
			db_dml cal_item_update "
			UPDATE cal_items
			SET $update_instance_title_sql,
			$update_instance_description_sql,
 			$update_instance_html_p_sql
			WHERE item_id = :item_id"
			if {[db_resultrows] <= 0} {
			    db_abort_transaction
			}
			if {$repeat_p == "f"} {
			    # See DATE-NOTE at top of file
			    db_dml cal_item_update_dates "
			    UPDATE cal_items
			    SET start_date=to_date(:valid_start_date, $date_format_string),
			        end_date=to_date(:valid_end_date, $date_format_string)
			    WHERE item_id=:item_id
			    "
			    if {[db_resultrows] <= 0} {
				db_abort_transaction
			    }
			} else {
			    # Calculate time offset 
			    # If nonzero then time changed
			    # If greater than 1, then date changed
			    # See DATE-NOTE at top of file
			    db_1row cal_item_get_offsets "
			    SELECT (to_date(:valid_start_date, $date_format_string) - start_date) as start_offset,
			           (nvl(to_date(:valid_end_date, $date_format_string), end_date) - end_date) as end_offset
			    FROM cal_items
			    WHERE item_id = :item_id"
			    # ns_log notice "Offsets: $start_offset $end_offset"
			    if {$end_offset == "[db_null]" || \
				    $end_date == "NULL"} {
				# Update each instance with start_offset and
				# new end_date
				db_dml cal_items_update_start_date "
				UPDATE cal_items
				SET start_date = (start_date + :start_offset),
				end_date = to_date(:valid_end_date, $date_format_string)
				WHERE parent_id = :parent_id"
			    } elseif {$start_offset != 0 || $end_offset != 0} {
				# Update each instance with offsets
				db_dml cal_item_update_dates "
				UPDATE cal_items
				SET start_date = (start_date + :start_offset),
				end_date = (end_date + :end_offset)
				WHERE parent_id = :parent_id"
				if {[db_resultrows] <= 0} {
				    db_abort_transaction
				}
			    }
			}
		    } elseif {$update_current_or_all == "current"} {
			# Got to figure out what's changed
			# If time, then update instance
			# See DATE-NOTE at top of file
			if {[db_string cal_item_time_changed_p "
			SELECT 1
			FROM cal_items
			WHERE item_id = :item_id
			AND (start_date != to_date(:valid_start_date, $date_format_string)
			OR end_date != to_date(:valid_end_date, $date_format_string))
			" -default 0]} {
			    db_dml cal_item_update_time "
			    UPDATE cal_items
			    SET start_date=to_date(:valid_start_date, $date_format_string),
			        end_date=to_date(:valid_end_date, $date_format_string)
			    WHERE item_id=:item_id
			    "
			    if {[db_resultrows] <= 0} {
				db_abort_transaction
			    }
			}
			# Other fields require updating the current instance
			if {[db_string cal_item_changed_p "
			SELECT 1
			FROM cal_items i, cal_items ii
			WHERE i.item_id = :parent_id
			  AND ii.item_id = :item_id
			  AND (i.title != :title
			       OR i.description != :description
			       OR i.html_p != :html_p
			       OR ii.title != :title
			       OR ii.description != :description
			       OR ii.html_p != :html_p)
			" -default 0]} {
			    db_dml cal_item_update_changed "
			    UPDATE cal_items
			    SET title=:title,
			    description=:description,
			    html_p = :html_p
			    WHERE item_id = :item_id"
			    if {[db_resultrows] <= 0} {
				db_abort_transaction
			    }
			}
		    } 
		} else {
		    set security_error_p "t"
		}
	    } else {
		ns_log Error "$message"
		set succeeded 0
	    }
	}
	if $succeeded {
	    ## deal with type_id_list
	    db_dml cal_item_type_map_delete "
	    delete from cal_item_type_map 
	    where item_id = :item_id"
	    db_dml cal_item_type_map_insert "
	    insert into cal_item_type_map
	    (item_id, type_id)
	    select :item_id, type_id
	    from cal_item_types
	    where enabled_p = 't'
	    and type_id in ('[join $type_id_list "', '"]')"
	}
    }
    db_release_unused_handles

    set current_view $pass_through_current_view
    set current_date $pass_through_current_date
    if {[string compare $group_id "[db_null]"] == 0} {
	set group_id ""
    }
    if {[string compare $user_id "[db_null]"] == 0} {
	set user_id ""
    }
    if { $succeeded != 0 } {
	if [empty_string_p $return_url] {
	    if {$repeat_p == "t"} {
		ad_returnredirect "index.tcl?group_id=$group_id&action=item-repeat&item_id=$item_id&parent_id=$parent_id&current_view=$pass_through_current_view&current_date=$pass_through_current_date"
	    } else {
		ad_returnredirect "index.tcl?group_id=$group_id&action=item-view&item_id=$item_id&parent_id=$parent_id&current_view=$pass_through_current_view&current_date=$pass_through_current_date"
	    }
	} else {
	    ad_returnredirect $return_url
	}
    } else {
	set return_string "<A HREF=\"?[export_url_vars current_view current_date group_id user_id]\">Return to calendar</A>"
	if {$security_error_p == "t"} {
	    ns_log Notice "calendar_form_process access restriction - user #$logged_in_user_id tried to modify calendar item #$item_id without permission"
	    ad_return_error "Security error" "<blockquote><H4>Insufficient permission to edit this calendar item - must be item owner or group administrator</H4></blockquote>$return_string"
	} else {
	    ad_return_error "Database error" "We were unable to properly submit this information.<P>$return_string"
	}
    }
}

# This a modified form of ad_util_queryget from the templates package
# However, I didn't want to have a dependency on the templates package
# and there doesn't seem to be a direct equivalent in the core package.
ad_proc -private cal_queryget { name } {
  Returns a list of values from a form submission.  If no values
  are found, returns a single empty string unless the -none
  option is specified.
} {
  set form [ns_getform]
  set values [list]

  if { $form == "" } { return $values }

  for { set i 0 } { $i < [ns_set size $form] } { incr i } {
    if { [ns_set key $form $i] == $name } {
      set value [ns_set value $form $i]
      if { [string length $value] > 0 } {
        lappend values $value
      }
    }
  }
  return $values
}

# Stolen and slightly modified from templating system
# use_as_fallback is a quick hack added to let end_date pull some of its data from start_date if date wasn't
# displayed.
ad_proc -private calendar_form_transform_datetime { {-truncate_p "f"
                                            -use_as_fallback ""} name} {
    set dates [list]
    set years [cal_queryget "$name.year"]
    if {[empty_string_p $years] && ![empty_string_p $use_as_fallback]} {
	set years [cal_queryget "$use_as_fallback.year"]
    }
    set months [cal_queryget "$name.month"]
    if {[empty_string_p $months] && ![empty_string_p $use_as_fallback]} {
	set months [cal_queryget "$use_as_fallback.month"]
    }
    set days [cal_queryget "$name.day"]
    if {[empty_string_p $days] && ![empty_string_p $use_as_fallback]} {
	set days [cal_queryget "$use_as_fallback.day"]
    }

    if {$truncate_p == "t"} {
	set hours 0
	set minutes 0
	set seconds 0
	set ampm 0
    } else {
	set hours [cal_queryget "$name.hours"]
	if {[empty_string_p $hours] && ![empty_string_p $use_as_fallback]} {
	    set hours [cal_queryget "$use_as_fallback.hour"]
	}
	set minutes [cal_queryget "$name.minutes"]
	if {[empty_string_p $minutes] && ![empty_string_p $use_as_fallback]} {
	    set minutes [cal_queryget "$use_as_fallback.minute"]
	}
	set seconds [cal_queryget "$name.seconds"]
	if {[empty_string_p $seconds] && ![empty_string_p $use_as_fallback]} {
	    set seconds [cal_queryget "$use_as_fallback.second"]
	}
	set ampm [cal_queryget "$name.ampm"]
	if {[empty_string_p $ampm] && ![empty_string_p $use_as_fallback]} {
	    set ampm [cal_queryget "$use_as_fallback.ampm"]
	}
    }

  # In case there are multiple dates w/ same name - this won't happen
  # on calendar, but might as well keep the functionality.
  for { set i 0 } { $i < [llength $years] } { incr i } {

      set year [lindex $years $i]
      set month [lindex $months $i]
      set day [lindex $days $i]
      set hours [lindex $hours $i]
      set minutes [lindex $minutes $i]
      set seconds [lindex $seconds $i]
      set ampm [lindex $ampm $i]

      if {$ampm == 0} {
	  # 12 am is 00 hours
	  if {$hours != 12} {
	      set date "${year}-${month}-${day} $hours:$minutes:$seconds"
	  } else {
	      set date "${year}-${month}-${day} 00:$minutes:$seconds"
	  }
      } else {
	  # 12 pm is 12 hours, but 1 is 13
	  if {$hours != 12} {
	      set date "${year}-${month}-${day} [expr $hours+12]:$minutes:$seconds"
	  } else {
	      set date "${year}-${month}-${day} $hours:$minutes:$seconds"
	  }
      }

      lappend dates "to_date('$date', 'YYYY:MM:DD HH24:MI:SS')"
  }

  return $dates
}

ad_proc calendar_round_to_precision { {} number precision } {
    Rounds the given number to the given precision.
    i.e. <tt>calendar_round_to_precision 44 5</tt> will round to the nearest
    5 and return 45, while <tt>calendar_round_to_precision 32.678 .1</tt> will round
    to 32.7.
} {
    # We multiply by 1.0 to make sure we're in floating point for the divide.
    # This way we get a round instead of a truncate.
    return [expr $precision * round(1.0 * $number / $precision)]
}

ad_proc calendar_trim_leading_zeros { {} string } {
    Returns a string w/ leading zeros trimmed.
    Used to get around TCL interpreter problems w/ thinking leading zeros are octal.
    We could just use validate_integer, but it runs one extra regexp that we don't need to run.
} {
    # trim leading zeros, so as not to confuse Tcl
    set string [string trimleft $string 0]
    if { [empty_string_p $string] } {
	# but not all of the zeros
	return "0"
    }
    return $string
}

ad_proc calendar_parse_date_time_string { {} str } {
    Parses the passed in string into a list of year, month, day, hour, minutes, seconds.
    Assumes AD standard "YYYY-MM-DD HH24:MI:SS", allows specifying single-digit values
    and leaving off information to the right side of the string (i.e. you can leave off
    the seconds).
    Values default to 1 for MM/DD and 0 for HH/MI/SS if not found in string.
    Leading zeroes will be stripped to avoid TCL problems interpreting these in Octal.
    NOTE that no validation is done - it's entirely possible to enter an invalid date such
    as Feb 31.  Oracle will check for this sort of stupidity on insert and throw an error
    if it happens.  This is a parser, not a validator.
} {
    set year ""
    set month 1
    set day 1
    set hours 0
    set minutes 0
    set seconds 0
    # Split into year/month/day/hours/minutes/seconds
    # This ugly regexp uses
    # non-reporting parens to group stuff that may not be included on the right hand side.  It's ugly, but
    # basically it lets the user terminate the string early.
    # This requires TCL 8.3.  It doesn't work in TCL 8.0, 7.6, 7.5, etc
    if {! [regexp {^\s*([0-9][0-9][0-9][0-9])(?:-([0-1]?[0-9])(?:-([0-3]?[0-9])(?:\s*([0-2]?[0-9])(?::([0-5]?[0-9])(?::([0-5]?[0-9]))?)?)?)?)?\s*$} $str x year month day hours minutes seconds]} {
	ns_log Error "calendar_parse_date_time_string: Regexp Fails pulling apart default time ${str} - Either this isn't a valid time (YYYY-MM-DD HH24:MI:SS, with a bit of flexibility for single digits and leaving information off the end), or this TCL version ([info tclversion]) can't support non-reporting parenthesis"
	# We could fall back on Oracle here and throw this at their date parser... or we could give up.
	# For now, we're giving up, to limit the scope creep in this proc.
    }
    return [list [calendar_trim_leading_zeros $year]\
                 [calendar_trim_leading_zeros $month]\
                 [calendar_trim_leading_zeros $day]\
                 [calendar_trim_leading_zeros $hours]\
                 [calendar_trim_leading_zeros $minutes]\
                 [calendar_trim_leading_zeros $seconds]]
}

# Simple stupid utility macro for preparing SQL string for next statement
ad_proc calendar_util_add_to_string { str value} {
    if {[empty_string_p $str]} {
	return "select $value"
    } else {
	return "$str, $value"
    }
}

# Formats date/time
# See description of calendar_widget_datetime.
ad_proc -public calendar_format_date_time_string { {-show_date 1
                                            -date_time_sep "&nbsp;"
                                            -use_am_pm 0}
					    value granularity } {
    switch -exact -- $granularity {
	months {set to_precision 40000}
	days {set to_precision 1440}
	hours {set to_precision 60}
	halves {set to_precision 30}
	quarters {set to_precision 15}
        fives {set to_precision 5}
	minutes {set to_precision 1}
	seconds {set to_precision 0}
	default {set to_precision 15}
    }

    set show_day [expr $to_precision<1441]
    set show_hours [expr $to_precision<61]
    set show_minutes [expr $to_precision<60]
    set show_seconds [expr $to_precision<1]

    if {$to_precision==0} { set to_precision 1 }

    set parsed_date [calendar_parse_date_time_string $value]
    set year [lindex $parsed_date 0]
    set month [lindex $parsed_date 1]
    set day [lindex $parsed_date 2]
    set hours [lindex $parsed_date 3]
    set minutes [lindex $parsed_date 4]
    set seconds [lindex $parsed_date 5]

    if {$show_date} {
	set format_string "%4d-%02d"
	if {$show_day} {
	    append format_string "-%02d"
	}
	if {$show_hours} {
	    append format_string "$date_time_sep%02d"
	}
	if {$show_minutes} {
	    append format_string ":%02d"
	}
	if {$show_seconds} {
	    append format_string ":%02d"
	}
    } else {
	# These format strings are weird.  Here's what's up:
	# %...d indicates a number
	# 4$ indicates "take the 4th parameter"
	# 02 indicates "Use two digits, and pad the left side with zeros"
	# The \ is to escape the $ for the TCL interpreter
	if {$show_hours} {
	    set format_string "%4\$02d"
	}
	if {$show_minutes} {
	    append format_string ":%5\$02d"
	}
	if {$show_seconds} {
	    append format_string ":%6\$02d"
	}
    }

    if {! $use_am_pm} {
        return [format $format_string $year $month $day $hours $minutes $seconds]
    } else {
	if {$hours < 12} {
	    set show_am_pm am
	} else {
	    set show_am_pm pm
	}
	if { $hours > 12 } {
	    set show_hours [expr {$hours - 12}]
	} elseif {$hours == 0} {
	    set show_hours 12
	} else {
	    set show_hours $hours
	}
	return [format "$format_string $show_am_pm" $year $month $day $show_hours $minutes $seconds]
    }
}


# WARNING: This method does not do *any* security checking.  The reason for 
# this decision was that it would be inefficient to repeat queries that 
# were already conducted by the caller.
# The variables are passed into this method, but it is relying
# on the caller having conducted the queries properly.
#
# Presently the security queries are group_member_p, group_admin_p, and good
# old logged_in_user_id in initialize_calendar_parameters.
#
# Since def_start_date, def_end_date, and duration can be passed in directly
# from user input, we make sure that they are passed as bind vars before being
# wrapped in the SQL fragments that are eventually used to construct queries.
ad_proc -private calendar_item_aev {{-form_name ""
                            -action "item-view"
                            -item_id 0
                            -parent_id 0
                            -pass_through_current_view ""
                            -pass_through_current_date ""
                            -def_title ""
                            -def_start_date ___LOOKUP_DEFAULT
                            -def_end_date ___LOOKUP_DEFAULT
                            -duration ___LOOKUP_DEFAULT
                            -def_description ""
                            -user_id ""
			    -group_id 0
			    -group_member_p "f"
			    -group_admin_p "f"
                            -read_only_p "f"
                            -show_debug_info_p "f"
                            -return_url ""
                            -default_type_id_list ""
                            -all_day_checked_string ""}
                           form_destination
			   logged_in_user_id } {
Generates a calendar item add/edit/view page, or an error string if there was a problem
(i.e. non-numeric duration, bad date format, etc)
The only non-obvious parameter is that if you want to have the no time
checkbox default to checked you must set the def_end_date to "".
See calendar_item_aev_2 for code.
} {
    if {[catch {set retval [calendar_item_aev_2 $form_name $action $item_id $parent_id $pass_through_current_view $pass_through_current_date $def_title $def_start_date $def_end_date $duration $def_description $user_id $group_id $form_destination $logged_in_user_id $group_member_p $group_admin_p $read_only_p $show_debug_info_p $return_url $default_type_id_list $all_day_checked_string]} result]} {
	# We just want to intercept an exception to release the DB handles.

	# Then, we re-throw it.
	# Nevermind the re-throw - we log it and return the errormessage.
	global errorInfo
	set savedInfo $errorInfo
	db_release_unused_handles
	#error $result $savedInfo
	ns_log Error "${savedInfo}"
	return "$result"
    }
    db_release_unused_handles
    return $retval
}

# Do not call directly!  Use calendar_item_aev instead.
# They were split up to allow for error catching - basically we need to ensure that
# the db_release_unused_handles function is called before control returns to the
# template processor in order to keep from having "could not allocate 1 handle" error
# messages disguise the real problem.
ad_proc calendar_item_aev_2 { 
    form_name action item_id parent_id pass_through_current_view 
    pass_through_current_date def_title def_start_date def_end_date 
    duration def_description user_id group_id form_destination 
    logged_in_user_id group_member_p group_admin_p read_only_p 
    show_debug_info_p return_url default_type_id_list all_day_checked_string} {

    set date_format_string "'YYYY-MM-DD HH24:MI'"
    
    set SQL ""

    # Temporarily hard-coded to 0
    set supports_javascript 0
    
    if {$parent_id == 0} {
	set new_item_p "t"
	set parent_id [db_nextval "cal_id_sequence"]
	set item_id [db_nextval "cal_id_sequence"]
    } else {
	# Since these parameters don't go inside quotes, they could be 
	# dangerous.  We'll validate them.
	set item_id [validate_integer "calendar ID" $item_id]
	set parent_id [validate_integer "instance ID" $parent_id]
	set new_item_p "f"
    }
    if { [empty_string_p $group_id] || $group_id==0} {
	# it's personal calendar
	set scope "user"
	set id $logged_in_user_id
    } else {
	# group calendar
	set scope "group"
	set id $group_id
    }
    
    # Do we have write access to this calendar?  Are we even allowed to view it?
    # We do this now because we want to be checking against raw database values.
    # write_access_p implies read_access_p
    if {$new_item_p == "t"} {
	# we have to check whether user can add new item; 
	# this will be true in two cases:
	# 1. user adding item to his/hew own calendar
	# 2. user is administrator of the group to which the calendar belongs
	if { [empty_string_p $group_id] || $group_id==0} {
	    # it's personal calendar
	    set write_access_p 1
	} else {
	    # group calendar
	    set write_access_p [ad_user_group_authorized_admin_or_site_admin $logged_in_user_id $group_id]
	}
    } else {
	set write_access_p [calendar_item_write_access_p $parent_id $group_id]
    }

    if {!$write_access_p} {
	set read_access_sql "SELECT 't'
	                       FROM cal_user_read_privilege
                              WHERE user_id=:logged_in_user_id
	                        AND item_id=:parent_id"
	set read_access_p [db_string cal_read_access_p $read_access_sql -default "f"]
    } else {
	if {$read_only_p == "t"} {
	    set write_access_p "0"
	}
	set read_access_p "t"
    }

    # if we can't read or write, return error.
    if {!$write_access_p && $read_access_p != "t"} {
	# Note that this condition will also occur if there is no calendar item with this item_id...
	# Do we need another error message?
	db_release_unused_handles
	return "<blockquote><h4>Insufficient permissions</h4></blockquote>"
    }

    # We get defaults and *then* data.  Why?  We might need to get a item_id.
    # We could get fewer defaults sometimes, for instance if a duration doesn't
    # need to be calculated we could sometimes skip default lookup entirely,
    # but in practice we usually need to calculate the duration so there's no point
    # in adding the complexity right now.
    if {[string compare $def_start_date ___LOOKUP_DEFAULT] == 0} {
	set SQL_start_date "trunc(sysdate, 'MI')"
	set SQL [calendar_util_add_to_string $SQL "to_char($SQL_start_date, $date_format_string) as def_start_date"]
    } else {
	if [regexp {^[0-9][0-9][0-9][0-9]-[0-9][0-9]-[0-9][0-9]$} \
		$def_start_date match] {
	    # Just date specified, default to 12 noon rather than 12 midnight
	    set def_start_date "$def_start_date 12:00"
	}
	set SQL_start_date "trunc(to_date(:def_start_date, $date_format_string), 'MI')"
    }
    if {[string compare $def_end_date ___LOOKUP_DEFAULT] == 0} {
	if {[string compare $duration ___LOOKUP_DEFAULT] == 0} {
	    # Add one hour if no default duration specified"
	    set sql_duration "1/24"
	} else {
	    # Add duration # of minutes.
	    set use_duration $duration
	    set sql_duration ":use_duration/(24*60)"
	}

	if {[string compare $def_start_date ___LOOKUP_DEFAULT] == 0} {
	    set sql_end_date_source "sysdate"
	} else {
            set sql_end_date_source "to_date(:def_start_date, $date_format_string)"
	}
	set SQL_end_date "trunc($sql_end_date_source, 'MI') + $sql_duration"
	set SQL [calendar_util_add_to_string $SQL "to_char($SQL_end_date, $date_format_string) as def_end_date"]
    } else {
	set SQL_end_date "trunc(to_date(:def_end_date, $date_format_string), 'MI')"
    }

    # We only initialize duration if javascript is supported, since
    # otherwise it will take over the calculation.
    if {$supports_javascript || $read_only_p} {
	if {[string compare $duration ___LOOKUP_DEFAULT] == 0} {
	    if {[string compare $def_end_date ___LOOKUP_DEFAULT] == 0} {
		set duration 60
	    } else {
		# This calculates the # of minutes between end date and start date
		# Note that this will come up null if the start date or end date is blank, which is good, because
		# that's an no time event and duration would be silly.
		set SQL [calendar_util_add_to_string $SQL "($SQL_end_date - $SQL_start_date)*24*60 as duration"]
	    }
	}
    } else {
	set duration ""
    }

    # Go fetch defaults ...
    if {! [empty_string_p $SQL]} {
	append SQL " FROM dual"
	# return $SQL
	db_1row cal_get_defaults "$SQL"
    }

    db_0or1row cal_get_item "
    select nvl(ii.title, i.title) as title, 
           nvl(ii.description, i.description) as description, 
           nvl(ii.html_p, i.html_p) as html_p,
           ug.group_id as item_group_id, ug.short_name as item_group_name,
           ug.short_name as item_group_short_name, ug.group_type as item_group_type, ugt.pretty_name as item_group_type_pretty_name,
           i.creator_id, 
           nvl(ii.related_link_url, i.related_link_url) as related_link_url, 
           nvl(ii.related_link_text, i.related_link_text) as related_link_text, 
           to_char(ii.start_date, $date_format_string) as start_date,
           to_char(ii.end_date, $date_format_string) as end_date,
           to_char(ii.creation_date, $date_format_string) as creation_date,
           i.editable_p,
           nvl(i.interval_type, 'never') as interval_type, 
           i.every_nth_interval as nth_interval,
           i.days_of_week, i.repeat_until
    from cal_items i, cal_items ii, cal_user_group_item_map m,
         user_groups ug, user_group_types ugt
    where ug.group_id(+) = m.group_id
      and i.item_id = ii.parent_id
      and ugt.group_type(+) = ug.group_type
      and i.item_id = :parent_id
      and m.item_id = i.item_id
      and ii.item_id = :item_id"

    if {! [exists_and_not_null default_type_id_list]} {
	set default_type_id_list [db_list cal_get_types "
	select type_id 
	from cal_item_type_map 
	where item_id = :item_id"]
    }

    # Passed security...

    # Setup defaults if this is a new item
    if { $new_item_p == "t" } {
	if {! [exists_and_not_null title]} {
	    set title $def_title
	}
	if {! [exists_and_not_null start_date]} {
	    set start_date $def_start_date
	}
	if {! [exists_and_not_null description]} {
	    set description $def_description
	}
#  	if ![info exists related_link_url] {
#  	    set related_link_url ""
#  	}
#  	if ![exists_and_not_null redirect_to_related_link_p] {
#  	    set redirect_to_related_link_p "f"
#  	}
#  	if ![info exists interval_type] {
#  	    set interval_type ""
#  	}
	set submit_value "Add Item"
    } else {
	set submit_value "Update Item"
    }

    # Figure out whether all_day should be checked.
    # We only setup the end_date default if we found an empty end_date in DB.
    if { [info exists end_date] } {
	if {[empty_string_p $end_date]} {
	    set all_day_checked_string "CHECKED"
	    set all_day_checked_string_for_view "CHECKED"
	} else {
	    set all_day_checked_string_for_view ""
	}
    } else {
	if {[exists_and_not_null def_end_date]} {
	    set end_date $def_end_date
	    set all_day_checked_string_for_view ""
	} else {
	    set sd_parsed [calendar_parse_date_time_string $start_date]
	    set end_date "[lindex $sd_parsed 0]-[lindex $sd_parsed 1]-[lindex $sd_parsed 2]"
	    set all_day_checked_string "CHECKED"
	    set all_day_checked_string_for_view "CHECKED"
	}
    }
#ns_log Notice "all_day_checked_string: $all_day_checked_string"

    set show_end_date 0
    # If the two dates don't match, show end date.
    if {[empty_string_p $all_day_checked_string_for_view]} {
	set start_info [calendar_parse_date_time_string $start_date]
	set end_info [calendar_parse_date_time_string $end_date]
	if { ! ([lindex $start_info 0] == [lindex $end_info 0] && [lindex $start_info 1] == [lindex $end_info 1] && [lindex $start_info 2] == [lindex $end_info 2]) } {
	    set show_end_date 1
	}
    }

    switch $action {
	item-add -
	item-edit {
	    if { $write_access_p } {
		set title_widget "<input name=\"title\" size=\"60\" maxlength=\"60\" type=\"text\" value=\"$title\">"

 		# Figure out formatting
 		if ![exists_and_not_null html_p] {
 		    set html_p "t"
 		    set desc_html "pre"
 		} elseif { $html_p == "t" } { 
 		    if {[regexp {^<pre[^>]*>(.*)</pre>$} $description dummy description]} { 
 			set description [util_expand_entities $description]
 			set desc_html pre
 		    } else { 
 			set desc_html html
 		    }
 		} else { 
 		    set desc_html plain
 		}
 		set description_widget "
 		<tt>
 		<textarea name=\"description\" rows=\"4\" cols=\"60\">$description</textarea>
 		</tt><br>
 		[cal_radio_widget desc_html plain "wrap text"]
 		[cal_radio_widget desc_html pre "keep formatting"]
 		[cal_radio_widget desc_html html "allow HTML"]
 		"
		set duration_widget "<input name=\"duration\" size=\"4\" onChange=\"updateEndTime()\" size=\"24\" type=\"text\" value=\"$duration\">"
		if {! $supports_javascript} {
		    set duration_widget_warning " (overrides end time if set)"
		} else {
		    set duration_widget_warning ""
		}
		if { $action == "item-add" } {
		    set start_date_widget [calendar_widget_datetime -date_time_sep "<br><input name=\"all_day_p\" type=\"checkbox\" value=\"t\" $all_day_checked_string> <i>Note: overrides start time and end time selections.</i> <br>" -use_am_pm 1 start_date $start_date fives]
		} else {
		    set start_date_widget [calendar_widget_datetime -date_time_sep "<br><input name=\"all_day_p\" type=\"checkbox\" value=\"t\" $all_day_checked_string_for_view> <i>Note: overrides start time and end time selections.</i><br>" -use_am_pm 1 start_date $start_date fives]
		}
		    set end_date_widget [calendar_widget_datetime -show_date $show_end_date -use_am_pm 1 end_date $end_date fives]

		set type_widget [calendar_types_widget -default_type_id_list $default_type_id_list -default_return "<i>None</i>" $scope $id]
		# Always show types label even if no types
		set type_widget "
		<tr>
		<td valign=top align=\"right\"><b>Types</b></td>
		<td valign=top>$type_widget</td>
		</tr>
		"

		# Check for repeating item
		if {[exists_and_not_null interval_type] && 
	            $interval_type != "never" } {
		    set repeat_yes_checked " checked"
		    set repeat_no_checked ""
		    set update_current_or_all "<tr>
		    <td align=right><b>Update?</b></td>
		    <td>
		    <input name=\"update_current_or_all\" type=radio value=\"current\"> Current instance only &nbsp;&nbsp;
		    <input name=\"update_current_or_all\" type=radio value=\"all\"> All repeating instances
		    </td>
		    </tr>
		    "
		    set ask_to_repeat ""
		} else {
		    set repeat_yes_checked ""
		    set repeat_no_checked " checked"
		    set update_current_or_all ""
		    set ask_to_repeat "
		<tr>
		<td align=\"right\">
		<b>Repeat?</b>
		</td>
		<td>
		<input type=radio name=\"repeat_p\" value=\"t\" $repeat_yes_checked> Yes 
		<input type=radio name=\"repeat_p\" value=\"f\" $repeat_no_checked> No 
		</td>
		</tr>
		    "
		}
		set result "<form name=\"$form_name\" action=\"$form_destination\" method=\"post\">
		<table border=0 cellpadding=2>
		<tr>
		<td align=\"right\">
		<font color=\"#AA0000\">
		<b>Title</b></font>
		</td>
		<td>$title_widget</td>
		</tr>
		<tr>
		<td align=\"right\">
		<font color=\"#AA0000\">
		<b>Date<br><br>No Time<br>or<br>
		Start Time</b>
		</font></td>
		<td>$start_date_widget
		</td>
		</tr>
		<tr>
		<td align=\"right\">
		<b>End Time<br>or<br>Duration</b>
		</td>
		<td>$end_date_widget<br>
		$duration_widget minutes $duration_widget_warning
		</td>
		</tr>
                $type_widget
		<tr>
		<td align=\"right\" valign=\"top\"><br>
		<b>Description</b>
		</td>
		<td>$description_widget
		</td>
		</tr>
		$ask_to_repeat
		$update_current_or_all
		<tr>
		<td>&nbsp;</td>
		<td align=\"center\"><br>
		<input type=submit value=\"$submit_value\">
		</td>
		<td>&nbsp;
		</td>
		</tr>
		</table>
		[calendar_widget_hidden pass_through_current_view $pass_through_current_view]
		[calendar_widget_hidden group_id $group_id]
		[calendar_widget_hidden item_id $item_id]
		[calendar_widget_hidden parent_id $parent_id]
		[calendar_widget_hidden pass_through_current_date $pass_through_current_date]
		[calendar_widget_hidden user_id $user_id]
		[calendar_widget_hidden scope $scope]
		[calendar_widget_hidden new_item_p $new_item_p]
		[export_form_vars current_view current_date return_url]
		</form>"
	    } else {
		set result "
		<h2>$title</h2>
		<table border=0 cellpadding=2>
		<tr>
		<th>Date</th>
		<td>[calendar_format_date_time_string $start_date days]</td>
		</tr>
		"
		
		if {[empty_string_p $all_day_checked_string_for_view]} {
		    append result "
		    <tr>
		    <th>Time</th>
                    <td>[calendar_format_date_time_string -use_am_pm 1 -show_date 0 $start_date minutes] to 
		    [calendar_format_date_time_string -use_am_pm 1 -show_date $show_end_date $end_date minutes] </td>
		    </tr>"
		} else {
		    append result "
		    <tr>
		    <th>Time</th>
                    <td>No Time</td>
		    </tr>
		    "
		}
		append result "
		</table>
		<br>
		<table>
		<tr>
	        <th align=left>Description</th>
		</tr>
		<tr>
		<td>$description</td>
		</tr>
		</table>
		"
	    }
	}
	item-repeat {
	    append result "
	    <h2>$title</h2>
	    <form name=\"$form_name\" action=\"$form_destination\" method=\"post\">"
	    append result [cal_item_view $start_date $title \
		    $group_id $all_day_checked_string \
		    $end_date $show_end_date $description]
	    
	    # Fetch month date and month day pieces
	    # See DATE-NOTE at top of file
	    # Consider making proc and memoizing
	    set SQL_start_date "trunc(to_date(:start_date, $date_format_string), 'MI')"
	    db_1row cal_get_repeat_dates "select to_char($SQL_start_date, 'fmDdth') as month_date,
	    to_char($SQL_start_date, 'wth') as nth_week, 
	    to_char($SQL_start_date, 'd') - 1 as day_num, 
	    to_char($SQL_start_date, 'fmDay') as week_day
	    from dual"

	    # Set defaults
	    set every_nth_interval 1
	    set never_checked ""
	    set day_checked ""
	    set week_checked ""
	    set month_by_date_checked ""
	    set month_by_day_checked ""
	    set last_of_month_checked ""
	    set year_checked ""
	    set indefinitely_checked ""
	    set nth_week_string ""
	    set last_weekday_string ""

	    # Fill in info
	    if {![exists_and_not_null interval_type]} {
		set interval_type "never"
	    }
	    if {![exists_and_not_null repeat_until]} {
		set repeat_until $start_date
	    }
	    set repeat_until_widget \
		    [calendar_widget_datetime repeat_until $repeat_until days]

	    if {![exists_and_not_null days_of_week]} {
		set days_of_week [list $day_num]
	    }

	    set ${interval_type}_checked " checked"

	    set repeat_rowspan 6
	    set month_rowspan 2

	    if {$nth_week != "5th"} {
		incr repeat_rowspan
		incr month_rowspan
		set nth_week_string "
		<tr>
		<td>
		&nbsp;&nbsp;<input name=\"interval_type\" type=\"radio\" value=\"month_by_day\"${month_by_day_checked}>
		Month(s) on the $nth_week $week_day of the month
		</td>
		</tr>
		"
	    }

	    if {$nth_week == "4th" || $nth_week == "5th"} {
		incr repeat_rowspan
		incr month_rowspan
		set last_weekday_string "
		<tr>
		<td>
		&nbsp;&nbsp;<input name=\"interval_type\" type=\"radio\" value=\"last_of_month\"${last_of_month_checked}>
		Month(s) on the Last $week_day of the month
		</td>
		</tr>"
	    }

	    append result "
	    <p>
	    <table border=0 cellspacing=0 cellpadding=0>
	    <tr bgcolor=lavender>
	    <td colspan=3>
	    <b>Repeat Every<b>
	     <input name=nth_interval type=text size=3 value=${every_nth_interval}><b>:</b>
	    </td>
	    </tr>
	    <tr>
	    <td>
	    &nbsp;&nbsp;<input name=\"interval_type\" type=\"radio\" value=\"day\"${day_checked}>
	    Day(s)</td>
	    <td>&nbsp;</td>
	    </tr>
	    <tr bgcolor=lavender>
	    <td>
	    &nbsp;&nbsp;<input name=\"interval_type\" type=\"radio\" value=\"week\"${week_checked}>
	    Week(s) on
	    [calendar_days_of_week $days_of_week]
	    </td>
	    </tr>
	    <tr>
	    <td>
	    &nbsp;&nbsp;<input name=\"interval_type\" type=\"radio\" value=\"month_by_date\"${month_by_date_checked}>
	    Month(s) on the $month_date of the month
	    </td>
	    </tr>
	    $nth_week_string
	    $last_weekday_string
	    <tr bgcolor=lavender>
	    <td>
	    &nbsp;&nbsp;<input name=\"interval_type\" type=\"radio\" value=\"year\"${year_checked}>
	    Year(s)</td>
	    </tr>
	    </table>
	    <table>
	    <tr>
	    <td align=right><b>Repeat<br>Until</b></td>
	    <td>$repeat_until_widget 
	    <!-- Disabled
	    <b>or</b>
	    <input name=indefinitely_p type=checkbox value=t${indefinitely_checked}> Indefinitely
	    -->
	    </td>
	    </tr>
	    <tr>
	    <td>&nbsp;</td>
	    <td align=\"center\"><br>
	    <input type=submit value=\"Set Repeat\">
	    </td>
	    </table>
	    [calendar_widget_hidden pass_through_current_view $pass_through_current_view]
	    [calendar_widget_hidden group_id $group_id]
	    [calendar_widget_hidden item_id $item_id]
	    [calendar_widget_hidden parent_id $parent_id]
	    [calendar_widget_hidden pass_through_current_date $pass_through_current_date]
	    [calendar_widget_hidden user_id $user_id]
	    [calendar_widget_hidden new_item_p $new_item_p]
	    [export_form_vars return_url]
	    </form>"	
	}
	item-view {
	    # Display related_link_url
	    set related_link_widget ""
	    if ![empty_string_p $related_link_url] {
		set related_link_widget "
		<tr>
	     <th align=right bgcolor=\"#FFFFFF\"><font size=-1 face=\"Arial, Helvetica, sans-serif\">URL:</font></th>
	     <td><font size=-1 face=\"Arial, Helvetica, sans-serif\">
		<a href=\"$related_link_url\">
		"
		if [empty_string_p $related_link_text] {
		    append related_link_widget "$related_link_url"
		} else {
		    append related_link_widget "$related_link_text"
		}
		append related_link_widget "</a></font></td>"
	    }

	    # Pull out repeat information 
	    # Display if it exists
	    set repeat_info ""
	    if {$interval_type != "never"} {
		# Generate the display string
		set interval_detail ""
		switch $interval_type {
		    day {
			set interval_name "Day"
		    }
		    week {
			set interval_name "Week"
			set interval_detail "on \
				[translate_numeric_list $days_of_week \
				{Sunday Monday Tuesday Wednesday Thursday Friday Saturday}]"
		    }
		    month_by_date -
		    month_by_day -
		    last_of_month {
			# Fetch month date and month day pieces
			# See DATE-NOTE at top of file
			# Consider making proc and memoizing
			set SQL_start_date "trunc(to_date(:start_date, $date_format_string), 'MI')"
			db_1row cal_get_repeat_dates "
			SELECT to_char($SQL_start_date, 'fmDdth') as month_date,
			to_char($SQL_start_date, 'wth') as nth_week, 
			to_char($SQL_start_date, 'fmDay') as week_day
			FROM dual"
			set interval_name "Month"
			switch $interval_type {
			    month_by_date {
				set interval_detail "on the $month_date."
			    }
			    month_by_day {
				set interval_detail "on the $nth_week $week_day"
			    }
			    last_of_month {
				set interval_detail "on the last $week_day"
			    }
			}
		    }
		    year {
			set interval_name "Year"
		    }
		}
		set interval_plural [util_decode $nth_interval 1 "" "s"]
		# Eventually have to handle "indefinitely"
		append repeat_info "
		<tr>
		<td align=right><b>Repeat</b></td> <td>Every $nth_interval $interval_name$interval_plural $interval_detail</td>
		</tr>
		<tr>
		<td align=right><b>Until</b></td> <td>$repeat_until</td>
		</tr>
		"
	    }
	    # This needs to be updated to deal with single vs. all instances
 	    if { $write_access_p } {
		# Need to add a link to update repeat info

		# Delete link depends on repeating or not
		if {$interval_type == "never"} {
		    set delete_link [list "?action=item-delete&[export_url_vars group_id item_id parent_id title]&repeat_p=0&current_view=$pass_through_current_view&current_date=$pass_through_current_date\" onclick=\"if(confirm('Are you sure you want to delete \\\'$title\\\'?'))this.href='/calendar/item-delete-2?&[export_url_vars group_id item_id parent_id title]&current_view=$pass_through_current_view&current_date=$pass_through_current_date'; else this.href='javascript:()';" "Delete"]		    
		} else {
		    set delete_link [list "?action=item-delete&[export_url_vars group_id item_id parent_id title]&repeat_p=1&current_view=$pass_through_current_view&current_date=$pass_through_current_date" "Delete"]
		}
		# Certain special items, e.g. those inserted by absences
		# or reservations, will not be editable, but can be deleted
		# and have attachments.
		if {$editable_p != "f"} {
		    set navbar "[ad_navbar [list "?action=item-edit&[export_url_vars group_id item_id parent_id return_url]&current_view=$pass_through_current_view&current_date=$pass_through_current_date" Edit] \
			[list "?action=attachment-add&[export_url_vars group_id item_id parent_id return_url]&current_view=$pass_through_current_view&current_date=$pass_through_current_date" "Add Attachment"]\
			$delete_link]" 
		} else {
		    set navbar "[ad_navbar \
			[list "?action=attachment-add&[export_url_vars group_id item_id parent_id return_url]&current_view=$pass_through_current_view&current_date=$pass_through_current_date" "Add Attachment"]\
			$delete_link]"
		}
		append result "
		<p><font face=arial,helvetica size=-1 color=black> 
		$navbar
		</font></p>"
	    }
	    set item_date [calendar_format_date_time_string $start_date days]
	    set current_date $item_date
	    set type_list [calendar_display_types $item_id]
	    set num_types [llength $type_list]


 	    # Deal with description formatting
 	    if {$html_p == "f"} {
 		set description_html [ad_text_to_html $description]
	    } else {
		## right now, no difference, because it confuses font display
		set description_html "$description"
	    }

	    if {[ad_calendar type] != "group"} {
		set group_html "
		<tr>
		<th align=right>$item_group_type_pretty_name</th>
		<td>
		[cal_group_home_link $item_group_id $item_group_type $item_group_short_name $item_group_name]
		</td>
		</tr>"
	    } else {
		set group_html ""
	    }

	    if {[empty_string_p $all_day_checked_string_for_view]} {
		set date_header "Date and Time"
		set time_html " ; [calendar_format_date_time_string -use_am_pm 1 -show_date 0 $start_date minutes] to [calendar_format_date_time_string -use_am_pm 1 -show_date $show_end_date $end_date minutes]"
	    } else {
		set date_header "Date"
		set time_html ""
	    }

	    # Add type information
	    if !$num_types {
		set type_cell "
		<th align=right><font size=-1 face=\"Arial, Helvetica, sans-serif\">Type:</font></th>
		<td><font size=-1 face=\"Arial, Helvetica, sans-serif\">none</font></td>"
	    } elseif {$num_types == 1} {
		set type_cell "
		<th align=right><font size=-1 face=\"Arial, Helvetica, sans-serif\">Type:</font></th>
		<td><font size=-1 face=\"Arial, Helvetica, sans-serif\">$type_list</font></td>"
	    } else {
		set type_cell "
		<th align=right><font size=-1 face=\"Arial, Helvetica, sans-serif\">Types:</font></th>
		<td><font size=-1 face=\"Arial, Helvetica, sans-serif\">[join $type_list ", "]</font></td>"
	    }

	    append result "
	    <b><font face=arial,helvetica size=3 color=black>$description_html</font></b>
	    <table border=0 cellpadding=2>
	    $group_html
	    <tr valign=top>
	     <th align=right bgcolor=\"#FFFFFF\"><font size=-1 face=\"Arial, Helvetica, sans-serif\">$date_header:</font></th>
	     <td><font size=-1 face=\"Arial, Helvetica, sans-serif\"><a href=?current_view=day&[export_url_vars current_date group_id]>$item_date</a>$time_html</font></td>
	    </tr>
	    <tr valign=top>
	     <th align=right bgcolor=\"#FFFFFF\"><font size=-1 face=\"Arial, Helvetica, sans-serif\">Title:</font></th>
	     <td><font size=-1 face=\"Arial, Helvetica, sans-serif\">$title</font></td>
	    </tr>
	    $related_link_widget
	    <tr>
	     $type_cell
	    </tr>
	    "

	    ## commented out for now
	    # Why don't we display this?
	    #append result "$repeat_info<br>"

	    set file_attachments_html ""
	    # This query is another case where the performance of the 
	    # cost-based optimizer is sub-optimal.
	    db_foreach cal_item_db_row_map_fs_select "
	    select /*+ RULE */ c.on_what_id as file_id, 
	           fs.file_title as file_title,
	           fs.parent_id as parent_id,
	           fs2.file_title as parent_title,
	           fsvl.version_id as version_id,
	           fsvl.client_file_name,
	           fsvl.n_bytes,
	           fsvl.url,
	           lower(fsvl.file_type) as file_type
	    from cal_item_db_row_map c, fs_files fs, 
	         fs_versions_latest fsvl, fs_files fs2
	    where fs2.file_id(+) = fs.parent_id
	    and fs2.deleted_p = 'f'
	    and c.item_id=:item_id
	    and c.on_what_id=fs.file_id
	    and fs.file_id=fsvl.file_id
	    and c.on_which_table='FS_FILES'
            and (ad_general_permissions.user_has_row_permission_p ( :logged_in_user_id, 'read', fsvl.version_id, 'FS_VERSIONS' ) = 't'
                 or fs.owner_id = :logged_in_user_id)
	    and fs.deleted_p='f'
	    " {
		# write out folder name
		if {[info exists parent_id] && ![empty_string_p $parent_id]} {
#		    append file_attachments_html "
#		    <a href=/file-storage/one-folder?file_id=$parent_id&[export_url_vars group_id return_url]>$parent_title</a>:"		    
#Not putting in the link the the folder right now. Too confusing to the user.
		    append file_attachments_html "$parent_title:"		    
		}
		if { [empty_string_p $n_bytes] } {
		    append file_attachments_html "
		    <a href=$url>$file_title</a>"
		} else {
		    regsub -all {[^-_.0-9a-zA-Z]+} $client_file_name "_" pretty_file_name
		    append file_attachments_html "
		    <a href=/file-storage/download/$version_id/$pretty_file_name>$file_title</a>"
		}
		#giving file adminstration priveleges to people who can write to it
		if { $write_access_p } {
		    if {$group_id==0} {
			set export_url_vars [export_url_vars file_id]
		    } else {
			set export_url_vars [export_url_vars group_id file_id]
		    }
		    append file_attachments_html " \[<a href=\"/file-storage/one-file?$export_url_vars\">manage</a>\]"
		    if {![string compare $file_type "text/html"] || ![string compare $file_type "text/plain"]} {
			set return_url [ns_conn url]
			set query [ns_conn query]
			if {[info exists query] && ![empty_string_p $query]} {
			    append return_url "?$query"
			}
			append file_attachments_html " \[<a href=\"/calendar/edit-html-attachment?[export_url_vars file_id group_id return_url]\">edit</a>\]"
		    }
		}
		append file_attachments_html "<br>"
	    }
	    if ![empty_string_p $file_attachments_html] {
		
		append result "
		<tr>
		<th align=right valign=top bgcolor=\"#FFFFFF\"><font size=-1 face=\"Arial, Helvetica, sans-serif\">Attachments:</font></th>
		<td align=left valign=top bgcolor=\"#FFFFFF\">
		 <font size=-1 face=\"Arial, Helvetica, sans-serif\">$file_attachments_html</font>
		</td>
		</tr>
		"
	    }
	append result "</table>"
	}
    
	db_release_unused_handles
	return $result
    }
}

ad_proc -private calendar_days_of_week {day_nums {day_abbrevs "S M T W T F S"}} {
    This is a helper function for outputting a set of checkboxes for days
    of the week.  @param day_nums indicates days to be checked.  Optional
    @param day_abbrevs allows for alternate abbrevs, e.g. for different
    languages. } {
    set html ""
    set count 0
    foreach abbrev $day_abbrevs {
	set checked ""
	if {[lsearch $day_nums $count] != -1} {
	    set checked " checked"
	}
	append html "<input name=\"days_of_week\" type=checkbox value=$count$checked>&nbsp;[lindex $day_abbrevs $count]&nbsp;&nbsp;"
	incr count
    }
    return $html
}

ad_proc -private translate_numeric_list {indices replacements {joiner ", "}} {
    For each index in @param indices, substitute corresponding value in 
    @param replacements, joining with @param joiner. } {

    foreach index $indices {
	lappend result [lindex $replacements $index]
    }
    return [join $result $joiner]
}

ad_proc cal_item_view { {} start_date title group_id all_day_checked_string 
                           end_date show_end_date description } {
Display the basic information for an item.  Used by both item-view
and item-repeat.
} {
    set item_date [calendar_format_date_time_string $start_date days]
    set current_date $item_date
    append result "
    <table border=0 cellpadding=2>
    <tr>
    <th>Date</th>
    <td><a href=?current_view=day&[export_url_vars current_date group_id]>$item_date</a></td>
    </tr>
    "
    
    if {[empty_string_p $all_day_checked_string]} {
	append result "
	<tr>
	<th>Time</th>
	<td>[calendar_format_date_time_string -use_am_pm 1 -show_date 0 $start_date minutes] to 
	[calendar_format_date_time_string -use_am_pm 1 -show_date $show_end_date $end_date minutes] </td>
	</tr>"
    } else {
	append result "
	<tr>
	<th>Time</th>
	<td>No Time</td>
	</tr>
	"
    }
    append result "
    </table>
    <br>
    <table>
    <tr>
    <th align=right>Description</th>
    <td>$description</td>
    </tr>
    </table>
    "
    
    return $result
}

ad_proc -public calendar_widget_datetime { {-show_date 1
                                    -date_time_sep "&nbsp;"
                                    -use_am_pm 0}
				    name value granularity } {
  Yet another date/time widget!  You'd think we'd have a super-duper canonical version somewhere in the ACS.
  Based on First Tuesday's ad_form_widget_datetime (/web/ftdev/tcl/template/ad-form-widget.tcl
  on dev0103-001 as of 6/29/00) which is the templating system's standard
  datetime but with granularity control.
  granularity can be "months" "days" "hours" "halves" "quarters" "fives" "minutes" or "seconds"
  You cannot ask for granularity less fine than months (presently)
  Presently no support for timezones - all times are just taken and spewed directly, do
  any TZ conversion on the other side of this proc.
  Parameters other than granularity are obvious.
  End result is a text string containing HTML widgets with names
  "$name.year", "$name.month", "$name.day", "$name.hours", "$name.minutes", "$name.seconds",
  and "$name.ampm".  These will be numeric ("ampm" is 0 for am, 1 for pm)
  All HTML widgets will always be output *except* if show_date is 0; they will be hidden with if not needed
  to satisfy the current granularity level.
  Values default to 1 for MM/DD and 0 for HH/MI/SS/AM if not found in input
  string or if below granularity threshold.  This means
  you can pass the results straight to a stupid processor and it can readily
  convert to the proper Oracle date string just by stringing everything together.
} {
    switch -exact -- $granularity {
	months {set to_precision 40000}
	days {set to_precision 1440}
	hours {set to_precision 60}
	halves {set to_precision 30}
	quarters {set to_precision 15}
        fives {set to_precision 5}
	minutes {set to_precision 1}
	seconds {set to_precision 0}
	default {set to_precision 15}
    }

    set show_day [expr $to_precision<1441]
    set show_hours [expr $to_precision<61]
    set show_minutes [expr $to_precision<60]
    set show_seconds [expr $to_precision<1]

    if {$to_precision==0} { set to_precision 1 }

    if { [string match $value {}] } {
	# Shouldn't happen, but might as well default to something nice if it does.
	set value [db_string cal_sysdate_time "select to_char(sysdate, 'YYYY-MM-DD HH24:MI:SS') from dual"]
    }

    # Need to do some timezone conversion, take a look at FirstTuesday

    set parsed_date [calendar_parse_date_time_string $value]
    set year [lindex $parsed_date 0]
    set month [lindex $parsed_date 1]
    set day [lindex $parsed_date 2]
    set hours [lindex $parsed_date 3]
    set minutes [lindex $parsed_date 4]
    set seconds [lindex $parsed_date 5]
    # ns_log Notice "Formatting: $parsed_date broke into $year,$month,$day,$hours,$minutes,$seconds"

    # Kludge to get minutes rounded.  Should make general-purpose for the other values too...
    if {$to_precision < 60} {
	set minutes [calendar_round_to_precision $minutes $to_precision]
    }

    if {$show_date} {
	append input [calendar_widget_month "$name.month" $month]
	append input [calendar_widget_maybe_range $show_day "$name.day" 1 31 $day 1 0 1]
	append input "<input name=\"$name.year\" size=5 maxlength=4 value=\"$year\"> $date_time_sep "
    }
    if {$use_am_pm} {
	if { $hours > 12 } {
	    append input [calendar_widget_maybe_range $show_hours "$name.hours" 1 12 [expr {$hours - 12}] 1 0]
	} elseif {$hours == 0} {
	    append input [calendar_widget_maybe_range $show_hours "$name.hours" 1 12 12 1 0]
	} else {
	    append input [calendar_widget_maybe_range $show_hours "$name.hours" 1 12 $hours 1 0]
	}
    } else {
	append input [calendar_widget_maybe_range $show_hours "$name.hours" 0 23 $hours 1 1]
    }
    if {$show_minutes} { append input ":" }
    append input [calendar_widget_maybe_range $show_minutes "$name.minutes" 0 59 $minutes $to_precision 1]
    if {$show_seconds} { append input ":" }
    append input [calendar_widget_maybe_range $show_seconds "$name.seconds" 0 59 $seconds 1 1]

    if {$use_am_pm} {
	if {$hours < 12 || ! $show_hours} {
	    set am_selected " selected"
	    set pm_selected ""
	} else {
	    set am_selected ""
	    set pm_selected " selected"
	}
	append input "
	<select name=\"${name}.ampm\">
	<option value=\"0\"${am_selected}>AM\n
	<option value=\"1\"${pm_selected}>PM\n
	</select>"
    } else {
	append input [calendar_widget_hidden "${name}.ampm" "AM"]
    }
    # First Tuesday's widget displayed a hidden field w/ the timezone in it here.  We
    # may want to do something similar some day.

    return $input
}

ad_proc calendar_widget_hidden { {} name value } {
  Makes a hidden form item w/ given name and value
} {
    set result "<input name=\"$name\" type=\"hidden\" value=\"$value\">"
    # TEMP: Use non-hidden in order to debug
    # append result "(hidden: '$value' is $name)"
    return $result
}

ad_proc calendar_widget_month { {} name default } {
  Builds a picklist of months of the year.  Taken from First Tuesday's ad_widget_month.
  Presently hard-wired to just use English - more time should be spent to figure out
  ad_locale.
} {

  # Modified by akatwala to allow for I18N

  #set month_names [ad_util_month_names_by_locale [ad_locale user language]]
  # Removing locale selection to avoid dependency on publishing system.
  #set month_names [calendar_util_month_names_by_locale en]
  # Removing month_names_by_locale use because 'en' wasn't standard.
  set month_names [calendar_util_month_names]

  set default [string trimleft $default "0"]

  set input "<select name=\"$name\">\n"

  for { set i 1 } { $i <= 12 } { incr i } {

    set month_name [lindex $month_names [expr $i - 1]]

    if { $i == $default } {
      set selected " selected"
    } else {
      set selected ""
    }

    append input "<option$selected value=$i>$month_name\n"
  }

  append input "</select>\n"

  return $input
}

ad_proc calendar_util_month_names_by_locale { {} locale_abbrev } {
Taken and modified from FirstTuesday's ad_util_month_names_by_locale

Returns a list with the month names for the given locale from Oracle, memoising appropriately.
Presupposes that the ad_locales table has an nls_language column the contents of which Oracle will understand
} {
    # This doesn't seem to be setup right by default on most ACS installations, we're
    # not going to use it for calendar...
    set nls_language [util_memoize "db_string ad_locales_nls_language \"select nls_language as nls_language from ad_locales where locale_abbrev = '$locale_abbrev'\""]
    #set nls_language 'American'

    return [util_memoize "calendar_util_month_names_by_locale_helper $nls_language"]
}

ad_proc calendar_util_month_names {} {
Non-internationalized version of calendar_util_month_names_by_locale.
Hopefully this will work on more systems.
Returns a list of month names from Oracle.
} {
    return [util_memoize "calendar_util_month_names_helper"]
}

ad_proc calendar_util_month_names_helper {} {
Helper for non-internationalized version of calendar_util_month_names_by_locale.
} {
    for {set i 1} {$i<=12} {incr i} {
	set selection [db_1row month_name_select "select to_char(to_date('1999[ec_leading_zeros $i 2]01','YYYYMMDD'), 'Month') as month_name from dual"]
	lappend months $month_name
    }
#    ns_log Notice "Months: $months"
    return $months
}

ad_proc calendar_util_month_names_by_locale_helper { {} nls_language } {
Taken from FirstTuesday's ad_util_month_names_by_locale_helper

Returns a list of month names from Oracle. Thus nls_language needs to be something Oracle understands.

} {

    # Preserves previous value of NLS_DATE_LANGUAGE for restoration below.
    set selection [db_1row nls_language_select "select value as save_nls_language from v\$nls_parameters where parameter='NLS_DATE_LANGUAGE'"]

    if {[catch {

	db_dml alter_seesion "alter session set NLS_DATE_LANGUAGE = :nls_language"

	for {set i 1} {$i<=12} {incr i} {
	    set selection [db_1row month_name_select "select to_char(to_date('1999[ec_leading_zeros $i 2]01','YYYYMMDD'), 'Month') as month_name from dual"]
	    lappend months $month_name
	}

    } err_value]} {
	ns_log Warning "calendar_util_month_names_by_locale_helper failed : $errmsg"
    }

    # Could alter session back to prior value, thusly ...
    db_dml alter_session_2 "alter session set NLS_DATE_LANGUAGE = :save_nls_language"
    #
    # ... but since this is called only once per lang per server restart it is sound to simply
    # expire the whole pool; which might be worth putting back, if it were to work. ?akatwala?
    #    ns_db bouncepool $mydb

    return $months
}

# Taken from templating system
# Returns form numeric range, or hidden 00 if as_for_value is false.
ad_proc calendar_widget_maybe_range {ask_for_value name start_range end_range default_value { interval 1 } {with_leading_zeros 0} {hidden_value "00"}} {
    if {!$ask_for_value} {
	# Note that this flattens to zero for hidden fields, may be better to use default_value for some things.
	if {$with_leading_zeros} {
	    return [calendar_widget_hidden $name $hidden_value]
	} else {
	    return [calendar_widget_hidden $name [calendar_trim_leading_zeros $hidden_value]]
	}
    } else {
	return [calendar_widget_numericrange "$name" $start_range $end_range $default_value $interval $with_leading_zeros]
    }
}

# Taken (unmodified, for once!) from First Tuesday's templating system
ad_proc calendar_widget_numericrange { name begin end { default "" } {interval 1} {with_leading_zeros 0} } {
  append input "<select name=\"$name\">\n"

  for { set i $begin } { $i <= $end } { incr i $interval} {

    if { $i == $default || [string match "0$i" $default]} {
      set selected " selected"
    } else {
      set selected ""
    }
# With no leading zeros now. Previously just first case of following:
    if {$with_leading_zeros} {
	append input "<option$selected>[format %02d $i]\n"
    } else {
	append input "<option$selected>$i\n"
    }
}

  append input "</select>"

  return $input
}


# Ugh.  The next two procs duplicate a lot of code.  Consider cleaning up.

# returns file_id of the folder in file-storage where calendar items are kept 
# for the user calendar identified with the user_id
# folder with name equal to FileAttachmentsFolder from parameters file will
# be created in the user's personal tree if it doesn't already exist
ad_proc user_calendar_folder_id {user_id} {
    set attachments_folder_name \
	    [ad_parameter FileAttachmentsFolder calendar "CalendarAttachments"]

    if [db_0or1row cal_file_id "
    select file_id as folder_id, deleted_p
    from fs_files 
    where file_title=:attachments_folder_name 
    and owner_id=:user_id
    and group_id IS NULL"] {
		
	if ![empty_string_p $folder_id] {
	    # the folder could have been deleted; in this case, let's restore it without restoring its deleted contents
	    if ![string compare $deleted_p "t"] {
		db_dml fs_update "
		update fs_files
		set deleted_p='f'
		where file_id=:folder_id"
	    }
	    return $folder_id
	}
    }

    #folder doesn't exist and must be created

    set file_seq_id [db_string fs_file_id_seq_nextval "select fs_file_id_seq.nextval from dual"]
    set version_seq_id [db_string fs_version_id_seq_nextval "select fs_version_id_seq.nextval from dual"]

    db_transaction {

	db_dml fs_insert "
	insert into fs_files
	(file_id, file_title, sort_key, depth, folder_p, owner_id)
	values 
	(:file_seq_id, :attachments_folder_name, 10000, 0, 't', :user_id)
	"

	# now we want to insert a "dummy" version so that we can also create the permission
	# records
	db_dml fs_versions_insert "
	insert into fs_versions
	(version_id, file_id, creation_date, author_id)
	values
	(:version_seq_id, :file_seq_id, sysdate, :user_id)"
    }

    fs_order_files 
    
    return $file_seq_id
}
 
# returns file_id of the folder in file-storage where calendar items are kept 
# for the group calendar identified with the group_id
# folder with name equal to FileAttachmentsFolder from parameters file will
# be created in the file-storage tree for that group if it doesn't already exist
ad_proc group_calendar_folder_id {group_id} {
    set attachments_folder_name [ad_parameter FileAttachmentsFolder calendar "CalendarAttachments"]

    if [db_0or1row cal_file_id "
    select file_id as folder_id, deleted_p
    from fs_files 
    where file_title=:attachments_folder_name
    and group_id=:group_id"] {
    	
	if ![empty_string_p $folder_id] {
	    # the folder could have been deleted; in this case, let's restore it without restoring its deleted contents
	    if ![string compare $deleted_p "t"] {
		db_dml fs_update "
		update fs_files
		set deleted_p='f'
		where file_id=:folder_id"
	    }
	    return $folder_id
	}
    }

    #folder doesn't exist and must be created

    set file_seq_id [db_string fs_file_id_seq_nextval "select fs_file_id_seq.nextval from dual"]
    set version_seq_id [db_string fs_version_id_seq_nextval "select fs_version_id_seq.nextval from dual"]
    set user_id [ad_verify_and_get_user_id]

    if ![set root_folder_id [portals_fs_group_root_folder $group_id]] {
	set parent_id [db_null]
    } else {
	set parent_id $root_folder_id
    }


    db_transaction {
	db_dml fs_insert "
	insert into fs_files
	(file_id, file_title, sort_key, depth, folder_p, owner_id, group_id, parent_id)
	values 
	(:file_seq_id, :attachments_folder_name, 10000, 0, 't', :user_id, :group_id, :parent_id)
	"

	# now we want to insert a "dummy" version so that we can also create the permission
	# records
	db_dml fs_versions_insert "
	insert into fs_versions
	(version_id, file_id, creation_date, author_id)
	values
	(:version_seq_id, :file_seq_id, sysdate, :user_id)"
    }

    fs_order_files

    return $file_seq_id
} 

ad_proc initialize_calendar_parameters { {} {view_mode "modular"} } {
    Initializes global variable ad_calendar and populates it with information
    used on calendar pages.    view_mode can be "full" or "modular": view 
    page with the minicalendar for navigation  or view page in isolation
} {
    global ad_calendar 
    # check if initialization was already done
    if {[info exists ad_calendar(initialized_p)] && $ad_calendar(initialized_p) } {
	return
    }
    if {[apm_package_enabled_p "portals"] && [portals_info portal_page_p]} {
	set group_id_from_portals [portals_info group_id]
    } else {
	set group_id_from_portals ""
    }

    ad_page_variables {
	{group_id $group_id_from_portals }
	{current_view ""}
	{current_date ""}
	{compress_day_view_p ""}
	{view_mode $view_mode}
	{action view}
    }
#ns_log notice "view_mode: $view_mode"

    # To support publicly viewable (i.e. without logging in) calendars
    # start by replacing the following line with this line: 
    # set logged_in_user_id [ad_verify_and_get_user_id]
    set logged_in_user_id [ad_maybe_redirect_for_registration]


    set user_id $logged_in_user_id


    # Date wrangling

    # Set up date limits
    set past_limit [ad_parameter CalendarPastLimit calendar 10]
    set future_limit [cal_future_limit]
    db_1row cal_get_date_limits "
    select add_months(sysdate, -12 * :past_limit) as past_date,
           add_months(sysdate, 12 * :future_limit) as future_date
    from dual"
    set ad_calendar(past_date) $past_date
    set ad_calendar(future_date) $future_date

    if [empty_string_p $current_date] {
	db_1row cal_pretty_sysdate "
	select trunc(sysdate) as current_date, 
	       to_char(sysdate, 'Month DD, YYYY') as pretty_current_date
	from dual"
    } else {
#	db_1row cal_pretty_current_date "select :current_date as current_date, to_char(to_date(:current_date), 'Month DD, YYYY') as pretty_current_date from dual"
	if [catch {util_memoize "cal_check_and_pretty_current_date $current_date $past_limit $future_limit"}  pretty_current_date] {
	    # Bad date, complain
	    return -code return [ad_return_complaint 1 "<li>$current_date is an invalid date.  It should be of the form YYYY-MM-DD"]
	}
	if [empty_string_p $pretty_current_date] {
	    # Date out of range, complain
	    return -code return [ad_return_complaint 1 "<li>$current_date is out of range. The date must be between $past_limit year[ad_decode $past_limit 1 "" "s"] in the past and $future_limit year[ad_decode $future_limit 1 "" "s"] in the future"]
	}
    }

    set ad_calendar(current_date) $current_date
    set ad_calendar(pretty_current_date) $pretty_current_date

    set ad_calendar(user_id) $user_id
    set ad_calendar(logged_in_user_id) $logged_in_user_id
    if { ![exists_and_not_null group_id] } {
	set group_id 0
    }
    if {$group_id==0} {
	set ad_calendar(type) user
	set ad_calendar(group_name) ""
	set ad_calendar(pretty_name) ""
	set ad_calendar(group_member_p) 0
	set ad_calendar(group_admin_p) 0
    } else {
	set ad_calendar(type) group
	set ad_calendar(group_name) [db_string cal_short_name "select short_name from user_groups where group_id=:group_id"]
	set ad_calendar(pretty_name) [db_string cal_group_name "select group_name from user_groups where group_id=:group_id"]
	set ad_calendar(group_member_p) [ad_user_group_member $group_id]
	set ad_calendar(group_admin_p) [ad_user_group_authorized_admin_or_site_admin $user_id $group_id]
    }
    set ad_calendar(group_id) $group_id

    # Consider memoizing
    db_1row cal_get_user_info "
    select first_names, last_name, email 
    from users 
    where user_id=:user_id"
    set ad_calendar(user.first_names) $first_names
    set ad_calendar(user.last_name) $last_name
    set ad_calendar(user.email) $email

    db_1row cal_get_user_prefs "
    select nvl(default_view, 'day') as default_view,
           compress_day_view_p as default_compress_day_view_p,
           new_items_age
    from cal_user_prefs 
    where user_id = :user_id"

    if [empty_string_p $current_view] {
	set current_view $default_view
    }
    set ad_calendar(current_view) $current_view

    if [empty_string_p $compress_day_view_p] {
	set compress_day_view_p $default_compress_day_view_p
    }
    set ad_calendar(compress_day_view_p) $compress_day_view_p

    set ad_calendar(new_items_age) $new_items_age

    # Track the current view but only update if the view actually changed
    # because otherwise it could be a performance hit.
    if {$current_view != $default_view} {
	db_dml cal_set_current_view "
	update cal_user_prefs
	set default_view = :current_view
	where user_id = :logged_in_user_id"
    }
    set ad_calendar(title) "Calendar"
    if {$group_id != 0} {
	append ad_calendar(title) " for $ad_calendar(pretty_name)"
    } else {
	if {![empty_string_p $first_names]} {
	    append ad_calendar(title) " for $first_names ${last_name}"
	}
    }
    set ad_calendar(view_mode) $view_mode
    set ad_calendar(view_mode_var) "view_mode=$view_mode&"

    set ad_calendar(action) $action
    
    # To where should the add link go?
    if {[string compare $view_mode "full"] == 0} {
	set ad_calendar(add_link) "?current_date=$current_date&"
    } else {
	set ad_calendar(add_link) "/calendar/?current_date=$current_date&"
    }

    if {$group_id} {
	append ad_calendar(add_link) "scope=group&group_id=$group_id&"
    }

    if {$view_mode == "full"} {
	set current_view_link "?current_view=$ad_calendar(current_view)&view_mode=full&"
    } else {
	set current_view_link "?view_mode=modular&"
    }
    set ad_calendar(current_view_link) $current_view_link

    if {$view_mode == "full"} {
	set day_link "/calendar/?current_view=day&"
    } else {
	set day_link "/calendar/?current_view=day&view_mode=modular&"
    }
    set ad_calendar(day_link) $day_link

    set ad_calendar(initialized_p) 1
}

ad_proc ad_calendar { {} which } {
    Returns calendar property.
} {
    global ad_calendar
    if { [info exists ad_calendar($which)] } {
	return $ad_calendar($which)
    } else {
	error "ad_calendar $which is invalid"
    }
}

ad_proc calendar_view_title {} {
    Returns view title
} {
    set compress_link ""
    set current_date [ad_calendar current_date]
    switch [ad_calendar current_view] {
	day {
	    set compress_day_view_p [ad_calendar compress_day_view_p]
	    
	    set compress_link "&compress_day_view_p=$compress_day_view_p"
	    db_1row cal_get_day_items "
	    select
	    to_char(to_date(:current_date), 'fmDay, fmMonth fmDD, YYYY') as pretty_current_item,
	    to_date(:current_date) - 1 as previous_item,
	    to_date(:current_date) + 1 as next_item
	    from dual"
	}
	week {
	    db_1row cal_get_week_items "
	    select
	    to_char(next_day(to_date(:current_date) - 7, 'SUNDAY'), 'Month fmDD, YYYY') || ' - ' || to_char(next_day(to_date(:current_date), 'SATURDAY'), 'Month fmDD, YYYY') as pretty_current_item,
	    to_date(:current_date) - 7 as previous_item,
	    to_date(:current_date) + 7 as next_item
	    from dual"
	}
	month {
	    db_1row cal_get_month_items "select 
	    to_char(to_date(:current_date), 'fmMonth') || ' ' || to_char(to_date(:current_date), 'YYYY') as pretty_current_item,
	    trunc(add_months(to_date(:current_date), 1)) as next_item,
	    trunc(add_months(to_date(:current_date), -1)) as previous_item
	    from dual"
	}
	year {
	    db_1row cal_get_year_items "select
	    to_char(to_date(:current_date), 'YYYY') as pretty_current_item,
	    to_char(add_months(to_date(:current_date), -12), 'YYYY-MM-DD') as previous_item,
	    to_char(add_months(to_date(:current_date), 12), 'YYYY-MM-DD') as next_item
	    from dual"
	}
	default {
	    error "calendar_view_title: Invalid view"
	}
    }
    
    return "
    <table cellpadding=3 cellspacing=0 border=0 width=90%>
    <tr bgcolor=lavender>
    <td align=center>
    <a href=[ad_calendar current_view_link]group_id=[ad_calendar group_id]&current_view=[ad_calendar current_view]&current_date=$previous_item$compress_link>
    <img border=0 width=11 height=11 src=\"/calendar/graphics/left\"></a>
    <FONT face=\"Arial,Helvetica\" SIZE=-1>
    <B>
    $pretty_current_item
    </B>
    </FONT>
    <a href=[ad_calendar current_view_link]group_id=[ad_calendar group_id]&current_view=[ad_calendar current_view]&current_date=$next_item$compress_link>
    <img border=0 width=11 height=11 src=\"/calendar/graphics/right\"></a>
    </td>
    </tr>
    </table>
    "
}

ad_proc calendar_no_items {} {
    returns the string, informing user that there are no calendar items
} { 
    return "<i>No Calendar Items to Display</i>"
}

ad_proc calendar_user_group_sql {
    {-group_id}
    {-user_id}
} {
    This clause is used to control display of group vs. user items for 
    all the views.  This defines it in one central location.
} {
    if ![info exists group_id] {
	set group_id [ad_calendar group_id]
    }
    if ![info exists user_id] {
	set user_id [ad_calendar user_id]
    }

    return [subst "
      (($group_id = 0
        and i.item_id = m.item_id 
	and (m.user_id = $user_id
	     or m.group_id in 
	     (select ugm.group_id
	      from user_group_map ugm, cal_user_update_prefs ugp
	      where ugm.user_id = $user_id
	      and ugp.user_id = ugm.user_id
	      and ugm.group_id = ugp.group_id
	      and ugp.update_type = 'automatically')
	      )
	    )
         or (i.item_id = m.item_id
	     and m.group_id in (select group_id
                                from user_groups
                                start with group_id= $group_id
                                connect by prior group_id = parent_group_id))
       )"]
}

ad_proc calendar_types_widget {{
    -select_size 2
    -var_name "type_id_list"
    -default_return ""
    -default_type_id_list ""}
    {scope "public"}
    {id ""}} {
	generates a select box of calendar types
    } {
    if { $scope == "public" } {
	set id_clause ""
    } else {
	set id_clause "and ${scope}_id = :id"
    }

    set options ""
    set n_types 0
    db_foreach type "select type_id, type
    from cal_item_types
    where enabled_p = 't'
    and scope = :scope
    $id_clause
    order by type" {
	incr n_types
	if {[lsearch -exact $default_type_id_list $type_id] != -1} {
	    append options "<option value=\"$type_id\" SELECTED>$type</option>"
	} else {
	    append options "<option value=\"$type_id\">$type</option>"
	}
    }

    if { $n_types == 0 } {
	return "$default_return"
    } else {
	return "<select multiple size=$select_size name=$var_name>
	$options
	</select>"
    }
}

# ejc@arsdigita.com 7-27-00
ad_proc calendar_display_types { {} item_id } {
    Given a calendar instance id, returns a tcl list of all type names for that item.
} {
    set type_list [db_list type_select "select type
                                from cal_item_type_map ctm, cal_item_types ct
                                where item_id = :item_id
                                and ctm.type_id = ct.type_id"]
    db_release_unused_handles
    return $type_list
}

ad_proc calendar_item_write_access_p { {} item_id {group_id ""} } {
    checks write access
} {
    set user_id [ad_verify_and_get_user_id]

    set write_access_p [db_string cal_write_privilege_p "
    select 1
    from cal_user_write_privilege
    where user_id = :user_id
      and item_id = :item_id
    " -default "0"]

    if !$write_access_p {
	set write_access_p [ad_administrator_p $user_id]

	if {!$write_access_p && $group_id} {
	    set write_access_p [ad_user_group_authorized_admin $user_id $group_id]
	}
    }

    return $write_access_p
}

ad_proc cal_group_home_link { group_id group_type short_name group_name } {
    Possibly generate an HTML link on group_name to the preferred home for
    the group.  If portals are enabled, link to group portal, otherwise 
    don't link.
} {
    if [apm_package_enabled_p "portals"] {
	set domain_id [portals_get_domain_id group $group_type]
	
	if { $domain_id == 0 } {
	    # there is no portal for this group type 
	    return "$group_name"
	} else {
	    if { ![empty_string_p $group_id] && $group_id>0 } {
		return "<a href=\"[portals_get_portal_property base_url [portals_get_portal_id group $group_id $domain_id]]/\">$group_name</a>"
	    } else {
		return ""
	    }
	}
    } else {
  	return "$group_name"
    }
}

# This is completely dependent on portals and should either be rewritten to
# be more independent (e.g. pass in type and group_id) or should be in the
# calendar-portals package.
ad_proc cal_group_schedule { {} } {
    Displays group schedule - all events for a group, categorized by type
} { 
    if ![apm_package_enabled_p "portals"] {
	return
    }
    ad_page_contract {
	This isn't really a page.
    } {
	{orderby ""}
    }
    # workaround setting orderby up there, because it might be different for different portals on the same page
    if {![info exists orderby] || [empty_string_p $orderby]} {
	set orderby "c_type,pretty_start_day"
    }

    set user_id [ad_verify_and_get_user_id]

    if {[ad_portals type]=="group"} {
	set group_id [ad_portals group_id]
	set id_clause "m.group_id = :group_id"
	set calendar_name "[db_string get_calendar_name "select group_name from user_groups where group_id = :group_id"] Schedule"
	set scope "group"
    } else {
	set group_id ""
	set id_clause "[calendar_user_group_sql -group_id 0 -user_id $user_id]"
	set calendar_name "Personal Schedule"
	set scope "user"
    }
    initialize_calendar_parameters
    
    set table_def {
	{day_start_day {Day of Week} {} {<td align=center>$day_start_day</td>} }
	{pretty_start_day {Date} {start_date $order} {<td align=center>$pretty_start_day</td>} }
	{pretty_start_time {Start Time} {orderby_start_time $order} {<td align=center>$pretty_start_time</td>} }
	{pretty_end_time {End Time} {orderby_end_time $order} {<td align=center>$pretty_end_time</td>} }
	{title {Title} {} {<td><a href="/calendar/?current_date=$start_date&group_id=$group_id&scope=$scope&action=item-view&item_id=$item_id&parent_id=$parent_id">$title</a></td>} }
	{c_type {Type} {} {<td align=center>$c_type</td>} }
    }

    # I think this query could be rewritten more cleanly...
    set sql_query "
    select c.*, NVL(type, 'Untyped') as c_type, 
           $group_id as group_id, '$scope' as scope
    from (select ci.item_id, ci.parent_id,
          ci.start_date,
          to_char(ci.start_date, 'MM/DD/YY') as pretty_start_day,
	  to_char(ci.start_date, 'Day') as day_start_day,
          to_char(ci.end_date, 'fmMonth fmDD, YYYY') as pretty_end_day,
          to_char(ci.end_date, 'Day') as day_end_day,
          to_char(ci.start_date, 'fmHH:fmMIam') as pretty_start_time,
          to_char(ci.end_date, 'fmHH:fmMI am') as pretty_end_time,
          to_char(ci.start_date, 'amHH:MI') as orderby_start_time,
          to_char(ci.end_date, 'amHH:MI') as orderby_end_time,
	  NVL(ci.title, i.title) as title
          from cal_items i, cal_user_group_item_map m, cal_items ci
	  where $id_clause
          and i.item_id = m.item_id
          and i.item_id = ci.parent_id) c, cal_item_type_map ctm, cal_item_types ct
    where c.item_id = ctm.item_id(+)
    and ctm.type_id = ct.type_id(+)
    [ad_order_by_from_sort_spec $orderby $table_def]
    "

    set rowcode {}
    set remove_column [list]
    switch [ad_sort_primary_key $orderby] {
	c_type {
	    lappend remove_column c_type
	    set rowcode {
		[if {![ad_table_same c_type]} {
		    ad_table_span [switch $c_type {
			"" {subst {<b>No Type</b>}}
			default {subst {<b>Type: $c_type</b>}}
		    }] "bgcolor=cccccc"
		}
		]
	    }
	}
    }
    
    #
    # generate the columns we really want to see given the list of removes
    #
    
    set all_columns {}
    foreach column $table_def {
	lappend all_columns [lindex $column 0]
    }
    set col [list]
    foreach acol $all_columns { 
	if {[lsearch -exact $remove_column $acol] < 0} { 
	    lappend col $acol
	}
    }
	set bind_vars [ns_set create]
	ns_set put $bind_vars group_id $group_id
    set table [ad_table -Taudit {item_id} -Tcolumns $col -Tpre_row_code $rowcode -Torderby $orderby -Tmissing_text "" -Textra_vars group_id -bind $bind_vars schedule_table $sql_query $table_def]

    if [empty_string_p $table] {
	set page_content "
	<ul><li><em>No items on schedule.</em></ul>"
    } else {
	set page_content "
	<b>$calendar_name</b>
	<br>
	<blockquote>
	$table
	</blockquote>
	"
    }
    return $page_content
}

# For memoization
proc cal_check_and_pretty_current_date {current_date past_limit future_limit} {
    
    set pretty_current_date [db_string cal_pretty_current_date "
    select to_char(to_date(:current_date), 'Month DD, YYYY')
    from dual
    where to_date(:current_date) 
          between add_months(sysdate, -12 * :past_limit)
              and add_months(sysdate, 12 * :future_limit)" -default ""]
    
    return $pretty_current_date
}

proc cal_radio_widget {varname value description} {
    upvar $varname current_value

    if {[info exists current_value] && $current_value == $value} {
	set checked " checked"
    } else {
	set checked ""
    }
    return "<input type=radio name=$varname value=\"$value\"$checked> $description"
}

proc cal_future_limit {} {
    return [ad_parameter CalendarFutureLimit calendar 10]
}

util_report_successful_library_load

