--
-- /www/doc/sql/education.sql
--
-- by randyg@mit.edu and aileen@mit.edu on September 29, 1999
-- with much help from philg@arsdigita.com on October 28, 1999

-- the system is centered around the classes
-- as it is too difficult to have 2 types of queries to support 
-- individual users

-- instead of having a classes table, we just define a user group
-- type of "edu_class" 

-- for the class name we'll use "group_name" (a default field from
-- the user_groups table); everything else will have to be in
-- edu_class_info; this is a bit tricky since we need to
-- keep the definitions for the helper edu_classes_info table in
-- sync with what we insert into user_group_type_fields (used
-- to generate UI)

-- we don't store much contact info in the classes table; if we need
-- to send out a report on system usage, we send it to all the people
-- with the admin role in this user group


-- this table holds the terms for the classes (e.g. Fall 1999)
 
create sequence edu_term_id_sequence start with 1;

create table edu_terms (
     term_id         integer not null primary key,
     term_name       varchar(100) not null,
     short_name	     varchar(100) unique,
     start_date      date not null,
     end_date        date not null
);

-- we want the above table to automatically start with a term that extends over all time
-- (or at least 100 years) for classes that people take at their own pace

insert into edu_terms (term_id, term_name, start_date, end_date) 
select edu_term_id_sequence.nextval, 'No Term', sysdate, add_months(sysdate,1200)
from dual
where 0 = (select count(*) from edu_terms);

-- for a multi-department university, we need to this to sort courses
-- by department; we're going to want private discussion groups, etc. 
-- for people who work in departments, so we make this a user group

-- to find the department head and other big staffers, we look at people with
-- particular roles in the user_group_map

create table edu_department_info (
	group_id 		integer primary key references user_groups,
	-- for schools like MIT where each department has a number
	department_number	varchar(100),
	-- we'll generate a home page for them but if they have one already
	-- we can provide a link
	external_homepage_url	varchar(200),
	mailing_address		varchar(200),
	phone_number		varchar(20),
	fax_number		varchar(20),
	inquiry_email		varchar(50),
	description		clob,
	mission_statement	clob,
	last_modified          	date default sysdate not null,
        last_modifying_user    	references users,
        modified_ip_address    	varchar(20)
);


-- we want to audit the department information

create table edu_department_info_audit (
	group_id 		integer,
	department_number	varchar(100),
	external_homepage_url	varchar(200),
	mailing_address		varchar(200),
	phone_number		varchar(20),
	fax_number		varchar(20),
	inquiry_email		varchar(50),
	description		clob,
	mission_statement	clob,
	last_modified          	date,
        last_modifying_user    	integer,
        modified_ip_address    	varchar(20)
);


-- we create a trigger to keep the audit table current

create or replace trigger edu_department_info_audit_tr
before update or delete on edu_department_info
for each row
begin
   insert into edu_department_info_audit (
	group_id,
	department_number,
	external_homepage_url,
	mailing_address,
	phone_number,
	fax_number,
	inquiry_email,
	description,
	mission_statement,
	last_modified,
        last_modifying_user,
        modified_ip_address)
     values (
	:old.group_id,
	:old.department_number,
	:old.external_homepage_url,
	:old.mailing_address,
	:old.phone_number,
	:old.fax_number,
	:old.inquiry_email,
	:old.description,
	:old.mission_statement,
	:old.last_modified,
        :old.last_modifying_user,
        :old.modified_ip_address);
end;
/
show errors


-- now, lets create a group of type department and insert all of
-- the necessary rows to generate the user interface on the /admin pages

declare	
 n_departments_group_types integer;
begin
 select count(*) into n_departments_group_types from user_group_types where group_type = 'edu_department';
  if n_departments_group_types = 0 then
    insert into user_group_types
    (user_group_types_id, group_type, pretty_name, pretty_plural, approval_policy, default_new_member_policy, group_module_administration)
    values
   (user_group_types_seq.nextval, 'edu_department','Department','Departments','wait','open','full');

   insert into user_group_type_fields (group_type, column_name, pretty_name, column_type, column_actual_type, column_extra, sort_key) 
   values
   ('edu_department', 'department_number', 'Department Number', 'text', 'varchar(100)', '', 1);

   insert into user_group_type_fields (group_type, column_name, pretty_name, column_type, column_actual_type, column_extra, sort_key) 
   values
   ('edu_department', 'external_homepage_url', 'External Homepage URL', 'text', 'varchar(200)', '', 2);

   insert into user_group_type_fields (group_type, column_name, pretty_name, column_type, column_actual_type, column_extra, sort_key) 
   values
   ('edu_department', 'mailing_address', 'Mailing Address', 'text', 'varchar(200)', '', 3);

   insert into user_group_type_fields (group_type, column_name, pretty_name, column_type, column_actual_type, column_extra, sort_key) 
   values
   ('edu_department', 'phone_number', 'Phone Number', 'text', 'varchar(20)', '', 4);

   insert into user_group_type_fields (group_type, column_name, pretty_name, column_type, column_actual_type, column_extra, sort_key) 
   values
   ('edu_department', 'fax_number', 'Fax Number', 'text', 'varchar(20)', '', 5);

   insert into user_group_type_fields (group_type, column_name, pretty_name, column_type, column_actual_type, column_extra, sort_key) 
   values
   ('edu_department', 'inquiry_email', 'Inquiry Email', 'text', 'varchar(50)', '', 6);

   insert into user_group_type_fields (group_type, column_name, pretty_name, column_type, column_actual_type, column_extra, sort_key) 
   values
   ('edu_department', 'description', 'Description', 'text', 'clob', '', 7);

   insert into user_group_type_fields (group_type, column_name, pretty_name, column_type, column_actual_type, column_extra, sort_key) 
   values
   ('edu_department', 'mission_statement', 'Mission Statement', 'text', 'clob', '', 8);

  end if;
end;
/


-- now we want to create a view to easily select departments

create or replace view edu_departments
as
select
	user_groups.group_id as department_id,
	group_name as department_name,
	department_number,
	external_homepage_url,
	mailing_address,
	phone_number,
	fax_number,
	inquiry_email,
	description,
	mission_statement
from user_groups, edu_department_info
where user_groups.group_id = edu_department_info.group_id
and group_type = 'edu_department'
and active_p = 't'
and approved_p = 't';



-- we model the subjects offered by departments

create sequence edu_subject_id_sequence;

-- we don't store the subject number in edu_subjects because a joint subject
-- may have more than one number 

create table edu_subjects (
	subject_id		integer primary key,
	subject_name		varchar(100) not null,
	description		varchar(4000),
	-- at MIT this will be a string like "3-0-9"
	credit_hours		varchar(50),	
	long_description	clob,
	long_description_html_p	char(1) default 'f' 
                                constraint edu_sub_lg_des_html_p_ck 
                                check(long_description_html_p in ('t','f')),
	prerequisites		varchar(4000),
	professors_in_charge	varchar(200),
	last_modified          	date default sysdate not null,
        last_modifying_user    	not null references users,
        modified_ip_address    	varchar(20) not null
);


-- we want to audit edu_subjects

create table edu_subjects_audit (
	subject_id		integer,
	subject_name		varchar(100),
	description		varchar(4000),
	credit_hours		varchar(50),	
	prerequisites		varchar(4000),
	professors_in_charge	varchar(200),
	last_modified          	date,
        last_modifying_user    	integer,
        modified_ip_address    	varchar(20)
);


-- we create a trigger to keep the audit table current

create or replace trigger edu_subjects_audit_trigger
before update or delete on edu_subjects
for each row
begin
   insert into edu_subjects_audit (
	subject_id,
	subject_name,
	description,
	credit_hours,
	prerequisites,
	professors_in_charge,
	last_modified,
        last_modifying_user,
        modified_ip_address)
   values (
	:old.subject_id,
	:old.subject_name,
	:old.description,
	:old.credit_hours,
	:old.prerequisites,
	:old.professors_in_charge,
	:old.last_modified,
        :old.last_modifying_user,
        :old.modified_ip_address);
end;
/
show errors	



create table edu_subject_department_map (
	department_id		integer references user_groups,
	subject_id		integer references edu_subjects,
	-- this would be the full '6.014' or 'CS 101'
	subject_number		varchar(20),
        grad_p                  char(1) default 'f' check(grad_p in ('t','f')),
	primary key ( department_id, subject_id )
);


-- now we create classes.  A class is a particular subject being taught in a particular
-- term.  However, we can also have special cases where a class is not associated with 
-- a term and we can even have classes that stand by themselves and aren't associated with
-- subjects, e.g., an IAP knitting course  (IAP = MIT's Independent Activities Period)

-- the PL/SQL statement cannot create the table so we do it here.
-- create a table to hold the extra info for each group of type
-- 'edu_classes'

create table edu_class_info (
	group_id		integer not null primary key references user_groups,
	term_id			integer references edu_terms,
	subject_id		integer references edu_subjects,
	-- if the class doesn't start or end on the usual term boundary, fill these in
	start_date		date,
	end_date		date,
	description 		varchar(4000),
	-- at MIT, something like 'Room 4-231, TR 1-2:30'
	where_and_when		varchar(4000),
	-- I still don't agree with this column.  I think we should use
	-- the file system to hold this and just keep a pointer to the 
	-- syllabus.  That way we would have versioning which we do not 
	-- have now (randyg@arsdigita.com, November, 1999)
	syllabus_id		integer references fs_files,
	-- we keep references to the class folders so that we can link to them directly
	-- from various different parts of the system.
	root_folder_id			references fs_files,
	assignments_folder_id		references fs_files,
	projects_folder_id		references fs_files,
	lecture_notes_folder_id		references fs_files,
	handouts_folder_id		references fs_files,
	exams_folder_id			references fs_files,
	-- will the class web page and the documents on it be open to the public?
	public_p			char(1) default 'f' check(public_p in ('t','f')),
	-- do students receive grades?
	grades_p  		        char(1) default 'f' check(grades_p in ('t','f')),
	-- will the class be divided into teams?
	teams_p			 	char(1) default 'f' check(teams_p in ('t','f')),
	exams_p                  	char(1) default 'f' check (exams_p in ('t', 'f')),
	-- does the class have a final exam?
	final_exam_p                	char(1) default 'f' check (final_exam_p in ('t','f')),
	last_modified           	date default sysdate not null,
        last_modifying_user     	references users,
        modified_ip_address     	varchar(20)
);

-- this table audits edu_class_info
create table edu_class_info_audit (
	group_id		integer,
	term_id			integer,
	subject_id		integer,
	start_date		date,
	end_date		date,
	description 		varchar(4000),
	where_and_when		varchar(4000),
	syllabus_id		integer,
	root_folder_id          integer,
	assignments_folder_id	integer,
	projects_folder_id	integer,
	lecture_notes_folder_id integer,
	handouts_folder_id	integer,
	exams_folder_id		integer,
	public_p		char(1),
	grades_p  		char(1),
	teams_p			char(1),
	exams_p                 char(1),
	final_exam_p            char(1),
	last_modified          	date,
        last_modifying_user     integer,
        modified_ip_address     varchar(20)
);



-- we create a trigger to keep the audit table current

create or replace trigger edu_class_info_audit_trigger
before update or delete on edu_class_info
for each row
begin
   insert into edu_class_info_audit (
	group_id,		
	term_id,			
	subject_id,		
	start_date,		
	end_date,		
	description, 		
	where_and_when,		
	syllabus_id,		
	root_folder_id,
	assignments_folder_id,	
	projects_folder_id,	
	lecture_notes_folder_id, 
	handouts_folder_id,	
	exams_folder_id,		
	public_p,		
	grades_p,  		
	teams_p,			
	exams_p,                 
	final_exam_p,            
	last_modified,          	
        last_modifying_user,     
        modified_ip_address)
    values (
	:old.group_id,		
	:old.term_id,			
	:old.subject_id,		
	:old.start_date,		
	:old.end_date,		
	:old.description, 		
	:old.where_and_when,		
	:old.root_folder_id,
	:old.syllabus_id,		
	:old.assignments_folder_id,	
	:old.projects_folder_id,	
	:old.lecture_notes_folder_id, 
	:old.handouts_folder_id,	
	:old.exams_folder_id,		
	:old.public_p,		
	:old.grades_p,  		
	:old.teams_p,			
	:old.exams_p,                 
	:old.final_exam_p,            
	:old.last_modified,          	
        :old.last_modifying_user,     
        :old.modified_ip_address);
end;
/
show errors


-- @author Eric J. Cholankeril (ejc@arsdigita.com)
-- @creation-date January 29, 2000

create table edu_subgroup_info (
	group_id	integer primary key
			constraint edu_sub_info_group_id_fk references user_groups,
	root_folder_id  integer
			constraint edu_sub_info_root_id_fk references fs_files
);

create table edu_subgroup_info_audit (
	group_id integer,
	root_folder_id integer
);

create or replace trigger edu_sub_info_audit_trigger
before update or delete on edu_subgroup_info
for each row
begin
	insert into edu_subgroup_info_audit (
		group_id,
		root_folder_id)
	values (:old.group_id,
		:old.root_folder_id);
end;
/
show errors

declare
 n_classes_group_types integer;
begin
 select count(*) into n_classes_group_types from user_group_types where group_type = 'edu_class';
  if n_classes_group_types = 0 then
	insert into user_group_types
  (user_group_types_id, group_type, pretty_name, pretty_plural, approval_policy, default_new_member_policy, group_module_administration)
  values
  (user_group_types_seq.nextval, 'edu_class','Class','Classes','wait','open','full');


   insert into user_group_type_fields (group_type, column_name, pretty_name, column_type, column_actual_type, column_extra, sort_key) 
   values
   ('edu_class', 'term_id', 'Term Class is Taught', 'text', 'integer', 'not null references edu_terms', 1);


   insert into user_group_type_fields (group_type, column_name, pretty_name, column_type, column_actual_type, column_extra, sort_key) 
   values
   ('edu_class', 'subject_id', 'Subject', 'text', 'integer', 'not null references edu_subjects', 2);


   insert into user_group_type_fields 
   (group_type, column_name, pretty_name, column_type, column_actual_type, column_extra, sort_key) 
   values
  ('edu_class', 'start_date', 'Date to Start Displaying Class Web Page', 'date', 'date', '', 3);


   insert into user_group_type_fields (group_type, column_name, pretty_name, column_type, column_actual_type, column_extra, sort_key) 
   values
   ('edu_class', 'end_date', 'Date to Stop Displaying Class Web Page', 'date', 'date', '', 4);


   insert into user_group_type_fields (group_type, column_name, pretty_name, column_type, column_actual_type, column_extra, sort_key) 
   values
   ('edu_class', 'description', 'Class Description', 'text', 'varchar(4000)', '', 5);


   insert into user_group_type_fields (group_type, column_name, pretty_name, column_type, column_actual_type, column_extra, sort_key) 
   values
   ('edu_class', 'where_and_when', 'Where and When', 'text', 'varchar(4000)', '', 6);


   insert into user_group_type_fields (group_type, column_name, pretty_name, column_type, column_actual_type, column_extra, sort_key) 
   values
   ('edu_class', 'syllabus_id', 'Syllabus ID', 'integer', 'integer', 'references fs_files', 7);


   insert into user_group_type_fields (group_type, column_name, pretty_name, column_type, column_actual_type, column_extra, sort_key) 
   values
   ('edu_class', 'assignments_folder_id', 'Assignments Folder', 'integer', 'integer', 'references fs_files', 8);


   insert into user_group_type_fields (group_type, column_name, pretty_name, column_type, column_actual_type, column_extra, sort_key) 
   values
   ('edu_class', 'projects_folder_id', 'Projects Folder', 'integer', 'integer', 'references fs_files', 8.5);


   insert into user_group_type_fields (group_type, column_name, pretty_name, column_type, column_actual_type, column_extra, sort_key) 
   values
   ('edu_class', 'lecture_notes_folder_id', 'Lecture Notes Folder', 'integer', 'integer', 'references fs_files', 9);


   insert into user_group_type_fields (group_type, column_name, pretty_name, column_type, column_actual_type, column_extra, sort_key) 
   values
   ('edu_class', 'handouts_folder_id', 'Handouts Folder', 'integer', 'integer', 'references fs_files', 10);


   insert into user_group_type_fields (group_type, column_name, pretty_name, column_type, column_actual_type, column_extra, sort_key) 
   values
   ('edu_class', 'public_p', 'Will the web page be open to the public?', 'boolean', 'char(1)', 'default ''t'' check(public_p in (''t'',''f''))', 11);


   insert into user_group_type_fields (group_type, column_name, pretty_name, column_type, column_actual_type, column_extra, sort_key) 
   values
   ('edu_class', 'grades_p', 'Do students recieve grades?', 'boolean', 'char(1)','default ''f'' check(grades_p in (''t'',''f''))', 12);


   insert into user_group_type_fields (group_type, column_name, pretty_name, column_type, column_actual_type, column_extra, sort_key) 
   values
   ('edu_class', 'teams_p', 'Will the class be divided into teams?', 'boolean', 'char(1)','default ''f'' check(teams_p in (''t'',''f''))', 13);


   insert into user_group_type_fields (group_type, column_name, pretty_name, column_type, column_actual_type, column_extra, sort_key) 
   values
   ('edu_class', 'exams_p', 'Will the class have exams?', 'boolean', 'char(1)','default ''f'' check(exams_p in (''t'',''f''))', 14);


   insert into user_group_type_fields (group_type, column_name, pretty_name, column_type, column_actual_type, column_extra, sort_key) 
   values
   ('edu_class', 'final_exam_p', 'Will the class have a final exam?', 'boolean', 'char(1)','default ''f'' check(final_exam_p in (''t'',''f''))', 15);

   insert into user_group_type_fields (group_type, column_name, pretty_name, column_type, column_actual_type, column_extra, sort_key) 
   values
   ('edu_class', 'exams_folder_id', 'Exams Folder', 'integer', 'integer', 'references fs_files', 16);

  end if;
end;
/

-- create a view for all active classes in the system - these are so
-- professors can access the admin pages even though students don't see
-- these classes

create or replace view edu_classes
as
select
	user_groups.group_id as class_id,
	group_name as class_name,
	user_groups.short_name as class_short_name,
	edu_class_info.term_id,
	subject_id,
	decode(edu_class_info.start_date, NULL, edu_terms.start_date, edu_class_info.start_date) as start_date,
	decode(edu_class_info.end_date, NULL, edu_terms.end_date, edu_class_info.end_date) as end_date,
	description,
	where_and_when,
	syllabus_id,
	root_folder_id,
	lecture_notes_folder_id,
	handouts_folder_id,
	assignments_folder_id,
	projects_folder_id,
	exams_folder_id,
	public_p,
	grades_p,
	teams_p,
	exams_p,
	final_exam_p
from user_groups, edu_class_info, edu_terms
where user_groups.group_id = edu_class_info.group_id
and edu_class_info.term_id = edu_terms.term_id
and group_type = 'edu_class'
and active_p = 't'
and approved_p = 't';

-- create a view for current classes whose webpages we should display 
-- to students

create or replace view edu_current_classes
as
select * 
from edu_classes
where sysdate<end_date and sysdate>=start_date;


-- now, we want to be able to store information about each individual in
-- a class so we create an entry in user_group_type_member_fields

insert into user_group_type_member_fields 
(group_type, role, field_name, field_type, sort_key) 
values 
('edu_class', 'student', 'Institution ID', 'short_text', 1);

insert into user_group_type_member_fields 
(group_type, role, field_name, field_type, sort_key) 
values 
('edu_class', 'dropped', 'Institution ID', 'short_text', 2);

insert into user_group_type_member_fields 
(group_type, role, field_name, field_type, sort_key) 
values 
('edu_class', 'student', 'Student Account', 'short_text', 3);

insert into user_group_type_member_fields 
(group_type, role, field_name, field_type, sort_key) 
values 
('edu_class', 'dropped', 'Student Account', 'short_text', 4);

insert into user_group_type_member_fields 
(group_type, role, field_name, field_type, sort_key) 
values
('edu_class', 'ta', 'Office', 'short_text', 5);

insert into user_group_type_member_fields 
(group_type, role, field_name, field_type, sort_key) 
values
('edu_class', 'professor', 'Office', 'short_text', 6);

insert into user_group_type_member_fields 
(group_type, role, field_name, field_type, sort_key) 
values
('edu_class', 'professor', 'Phone Number', 'short_text', 7);

insert into user_group_type_member_fields 
(group_type, role, field_name, field_type, sort_key) 
values
('edu_class', 'ta', 'Phone Number', 'short_text', 8);

insert into user_group_type_member_fields 
(group_type, role, field_name, field_type, sort_key) 
values
('edu_class', 'ta', 'Office Hours', 'short_text', 9);

insert into user_group_type_member_fields 
(group_type, role, field_name, field_type, sort_key) 
values
('edu_class', 'professor', 'Office Hours', 'short_text', 10);


-- we want to be able to divide classes further into sections.
-- this is nice for tutorials and recitations.  

-- you can get the class for the section from the parent_group_id from user_groups

create table edu_section_info (
	group_id		integer not null references user_groups,
	section_time		varchar(100),
	section_place		varchar(100)
);

declare	
 n_section_group_types integer;
begin
 select count(*) into n_section_group_types from user_group_types where group_type = 'edu_section';
  if n_section_group_types = 0 then
    insert into user_group_types
    (user_group_types_id, group_type, pretty_name, pretty_plural, approval_policy, default_new_member_policy,group_module_administration)
    values
   (user_group_types_seq.nextval, 'edu_section','Section','Sections','wait','open','full');


   insert into user_group_type_fields (group_type, column_name, pretty_name, column_type, column_actual_type, column_extra, sort_key) 
   values
   ('edu_section', 'section_time', 'Section Time', 'text', 'varchar(100)', '', 2);


   insert into user_group_type_fields (group_type, column_name, pretty_name, column_type, column_actual_type, column_extra, sort_key) 
   values
   ('edu_section', 'section_place', 'Section Place', 'text', 'varchar(100)', '', 3);

  end if;
end;
/

create or replace view edu_sections
as
select
	user_groups.group_id as section_id,
	group_name as section_name,
	parent_group_id as class_id,
	section_time,
 	section_place
from user_groups, edu_section_info
where user_groups.group_id = edu_section_info.group_id
and group_type = 'edu_section'
and active_p = 't'
and approved_p = 't';

declare	
 n_classes_group_types integer;
begin
 select count(*) into n_classes_group_types from user_group_types where group_type = 'edu_department';
  if n_classes_group_types = 0 then
    insert into user_group_types
    (user_group_types_id, group_type, pretty_name, pretty_plural, approval_policy, default_new_member_policy, group_module_administration)
    values
   (user_group_types_seq.nextval, 'edu_department','Department','Departments','wait','open','none');
  end if;
end;
/

-- Create edu_subgroup group type
declare	
 n_teams_group_types integer;
begin
 select count(*) into n_teams_group_types from user_group_types where group_type = 'edu_subgroup';
  if n_teams_group_types = 0 then
    insert into user_group_types
    (user_group_types_id, group_type, pretty_name, pretty_plural, approval_policy, default_new_member_policy,group_module_administration)
    values
   (user_group_types_seq.nextval, 'edu_subgroup','Subgroup','Subgroups','wait','open','none');
  end if;
end;
/

-- we want to allow classes to rename their roles.  That is,
-- some people want to be called Professor where others want
-- to be called Instructor and still others want to be called
-- Lecturer.  We don't want to just use the 'role' column
-- in user_group_roles because then we would not have a way
-- to "spam all professors and TAs" because we would not know
-- which role was a prof and which was a TA.  Also, we want to
-- have a sort_key so that we know which order to display these
-- items when they are shown to the user.  So, we have the following
-- table

-- so, for the case where a class wants to call the prof a Lecturer, 
-- we would have role = Professor and pretty_role = Lecturer

create table edu_role_pretty_role_map (
	group_id		not null references user_groups,
	-- role should reference user_group_roles(role)
	role			varchar(200),
	-- what the class wants to call the role
	pretty_role		varchar(200),
	pretty_role_plural 	varchar(300),
	-- sort key for display of columns.
	sort_key		integer not null,
	-- this is to capture info about the hierarchy of role permissions
	priority		integer,
	primary key (group_id, role)
);

-- now, we need a trigger to update the table we just created
-- this is included in case people want to add new roles to
-- the class all they have to do insert into user_group_roles
-- and this will take care of the rest

-- I. temporary table: holds rowids so that we know which rows
--    have changed/been inserted

--create global temporary table edu_role_change_state_info (
--	role_rowid	rowid,
--        old_role	varchar(200)
--);


-- II. row-level trigger: stores what changes in temporary table.

--CREATE OR REPLACE TRIGGER edu_role_before_update_tr
--BEFORE UPDATE OF role ON user_group_roles
--FOR EACH ROW
--BEGIN
--	insert into edu_role_change_state_info(role_rowid, old_role)
--	values (:new.rowid, :old.role);
--END;
--/
--show errors


-- III. update the edu_role_pretty_role table

CREATE OR REPLACE TRIGGER edu_class_role_update_tr
AFTER UPDATE OF role ON user_group_roles
FOR EACH ROW
DECLARE
	v_class_p	integer;
BEGIN
	select count(group_id) into v_class_p
	from user_groups
	where group_type = 'edu_class'
        and group_id = :old.group_id
	and group_type = 'edu_class';

	IF v_class_p > 0 THEN
		-- we want to update the existing row
		update edu_role_pretty_role_map
               	set role = :new.role
	        where group_id = :new.group_id
        	and role = :old.role;
	END IF;

END;
/
show errors


-- for every row that is inserted into the user_group_roles, if
-- the group is of type edu_class then we want to insert a corresponding
-- role into edu_role_pretty_role_map

CREATE OR REPLACE TRIGGER edu_class_role_insert_tr
AFTER INSERT ON user_group_roles
FOR EACH ROW
DECLARE
	v_class_p	integer;
BEGIN
	select count(group_id) into v_class_p
	from user_groups
	where group_type = 'edu_class'
        and group_id = :new.group_id;

	IF v_class_p < -1 THEN

		insert into edu_role_pretty_role_map (
        	       group_id, 
	               role, 
        	       pretty_role,
                       pretty_role_plural, 
	               sort_key,
                       priority) 
        	    select
	               :new.group_id, 
  		       :new.role, 
		       :new.role, 
                       :new.role || 's',
    		       nvl(max(sort_key),0) + 1,
    		       nvl(max(priority),0) + 1
	             from edu_role_pretty_role_map
        	    where group_id = :new.group_id;
	END IF;
END;
/
show errors


-- if a role is deleted from user_group_roles and the group
-- is of type edu_class then we also want to delete it from
-- edu_role_pretty_role_map

CREATE OR REPLACE TRIGGER edu_class_role_delete_tr
BEFORE DELETE ON user_group_roles
FOR EACH ROW
DECLARE
	v_class_p	integer;
BEGIN
	select count(group_id) into v_class_p
	from user_groups
	where group_type = 'edu_class'
        and group_id = :old.group_id;

	IF v_class_p > 0 THEN

		delete from edu_role_pretty_role_map 
		where group_id = :old.group_id
                and role = :old.role;

	END IF;
END;
/
show errors


insert into user_groups
    (group_id, group_type, group_name, short_name, admin_email, creation_user, 
     creation_ip_address, approved_p, new_member_policy, spam_policy, 
     parent_group_id, registration_date)
    values
    (user_group_sequence.nextval,'administration','Limited Access',
    'limited_access', null, 1, '216.34.106.252', 't', 'closed',
    'closed', null, sysdate);
    

insert into administration_info (select group_id, 'Education', null,
       null from user_groups where group_name like 'Limited Access'); 

insert into content_sections	
	(section_id, scope, section_type, requires_registration_p, 
	visibility, group_id, section_key, module_key, 
	section_pretty_name, enabled_p)
	select content_section_id_sequence.nextval, 'group', 
	section_type_from_module_key(module_key), 'f', 'public', ug.group_id,
	module_key, module_key, pretty_name_from_module_key(module_key), 't'
	from user_group_type_modules_map, user_groups ug 
	where ug.group_type='administration' and ug.group_name like 'Limited Access';



--trigger for dropping bboard subscriptions for users who leave a
--group


CREATE OR REPLACE TRIGGER drop_bboard_subscription_tr
BEFORE DELETE ON user_group_map
FOR EACH ROW
DECLARE
	
	CURSOR topic IS
		select topic_id from gc_bboard_topics
		where group_id = :old.group_id;
BEGIN
	FOR one_topic in topic LOOP
	   delete from gc_topic_user_map where user_id = :old.user_id
	   and topic_id = one_topic.topic_id;
	END LOOP;
END;
/
show errors

