-- we want to be able to keep track of assignemnts within the class.
create sequence edu_task_sequence;

-- includes assignments, projects, exams, any tasks a student might be graded on
create table edu_student_tasks (
	task_id		integer primary key,
	class_id 	not null references user_groups,
	grade_id	references edu_grades,
        -- we have to have a task type so we can categorize tasks in the
        -- user pages
	task_type	varchar(100) check (task_type in ('assignment', 'exam', 'project')),
	task_name	varchar(100),
	description	varchar(4000),
	-- the date we assigned/created the task
	date_assigned	date,
	-- we want to know the last time the task was modified
	-- (the permissions were changed or a new version was uploaded, etc)
        last_modified   date,
	-- could be date assignment is due, or date of an exam
	due_date	date,
	-- this references the fs_files that holds either the 
	-- actual assignment available for download or the url of the
	-- assignment
	file_id		references fs_files,
	-- who assigned this?
	assigned_by	not null references users,
        -- This column is for projects where students can
        -- assign themselves to teams.
        self_assignable_p char(1) default 'f' check (self_assignable_p in ('t','f')),
        self_assign_deadline    date,
	-- how much is this assignment worth compared to the others with
        -- the same grade_id (e.g. under the same grade group)?
	-- weight is a percentage	
	weight			number check (weight between 0 and 100),
	requires_grade_p char(1) check (requires_grade_p in ('t','f')),
	-- whether the task is submitted/administered online
	online_p char(1) check (online_p in ('t','f')),
	-- if an assignment has been deleted we mark it as inactive
	active_p char(1) default 't' check (active_p in ('t','f'))
);

-- views for assignments, exams, and projects
create or replace view edu_projects
as 
  select
  task_id as project_id,
  class_id,
  task_type,
  assigned_by as teacher_id,
  grade_id,
  task_name as project_name,
  description,
  date_assigned,
  last_modified,
  due_date,
  file_id,
  self_assignable_p,
  self_assign_deadline,
  weight,
  requires_grade_p,
  online_p as electronic_submission_p
from edu_student_tasks 
where task_type='project'
and active_p='t';

create or replace view edu_exams
as
  select
  task_id as exam_id,
  task_type,
  class_id,
  assigned_by as teacher_id,
  grade_id,
  task_name as exam_name,
  description as comments,
  date_assigned as creation_date,
  last_modified,
  due_date as date_administered,
  file_id,
  weight,
  requires_grade_p,
  online_p
from edu_student_tasks 
where task_type='exam'
and active_p='t';

create or replace view edu_assignments 
as 
  select 
  task_id as assignment_id,
  task_type,
  class_id,
  assigned_by as teacher_id,
  grade_id,
  task_name as assignment_name,
  description,
  date_assigned,
  last_modified,
  due_date,
  file_id,
  weight,
  requires_grade_p,
  online_p as electronic_submission_p
from edu_student_tasks
where task_type = 'assignment'
and active_p='t';


-- we want to be able to post the solutions and associate the solutions
-- to a given file

create table edu_task_solutions (
	task_id			references edu_student_tasks,
	file_id			references fs_files,
	primary key(task_id, file_id)
);



-- we want a table to map student solutions to assignments
-- this is what allows students to upload their finished papers, etc.

create table edu_student_answers (
        -- either student_id or team_id must be non-empty
	student_id		references users,
	team_id			references user_groups,
	task_id			references edu_student_tasks,
	file_id			references fs_files,
	-- this is the date of the last time the solutions were changed
	last_modified           date default sysdate not null,
        last_modifying_user     not null references users,
	-- modified_ip_address is stored as a string separated by periods.
        modified_ip_address     varchar(20) not null
);

create table edu_student_answers_audit (
	student_id		integer,
	task_id			integer,
	file_id			integer,
	-- this is the date of the last time the solutions were changed
	last_modified           date,
        last_modifying_user     integer,
	-- modified_ip_address is stored as a string separated by periods.
        modified_ip_address     varchar(20)
);


-- we create a trigger to keep the audit table current
create or replace trigger edu_student_answers_audit_tr
before update or delete on edu_student_answers
for each row
begin
	insert into edu_student_answers_audit (
	    	student_id,
		task_id,
		file_id,
		last_modified,
		last_modifying_user,
		modified_ip_address)
           values (
	    	:old.student_id,
		:old.task_id,
		:old.file_id,
		:old.last_modified,
		:old.last_modifying_user,
		:old.modified_ip_address);                     
end;
/
show errors

-- now, we want to hold information about each task.  It is possible
-- to have one term task but many instances of that task.  For
-- instance, "Final Project for 6.916" is a term task that would
-- be kept in the edu_tasks table but ArfDigita.org is a task
-- instance that would be kept in this table.  There is a many to 
-- one mapping

-- we make task_id not null because every task has to be part of
-- some sort of task (either an assignment or a project)
-- we make it a task because all evaluations are done on tasks

create sequence edu_task_instance_id_seq start with 1;
create table edu_task_instances  (
	task_instance_id 	integer not null primary key,
	task_instance_name	varchar(200),
	task_instance_url	varchar(500),
	-- which task is this an instance of?
	task_id		integer not null references edu_student_tasks,
	description		varchar(4000),
	approved_p		char(1) default 'f' check(approved_p in ('t','f')),
        approved_date           date,
        approving_user          references users(user_id),
	-- we want to be able to generate a consistent user interface so
	-- we record the type of task.
	-- (aileen 4/00) renamed this from task_type because task_type is
	-- a reserved column name in edu_student_tasks  
	team_or_user 		varchar(10) default 'team' check(team_or_user in ('user','team')),
	min_body_count		integer,
	max_body_count		integer,
	-- we want to be able to "delete" task instances so we have active_p
	active_p		char(1) default 't' check(active_p in ('t','f'))
);


-- we want to be able to assign students and teams to tasks
-- we use an index instead of a multi-column primary key because
-- team_id and student_id an both be null

create table edu_task_user_map (
	task_instance_id	integer not null references edu_task_instances,
	team_id			integer references user_groups,
	student_id		integer references users,
	constraint edu_task_user_map_check check ((team_id is null and student_id is not null) or (team_id is not null and student_id is null))
);

create index edu_task_map_idx on edu_task_user_map(task_instance_id, team_id, student_id);

-- now create a view for project instances 
-- note the renaming of project_type to team_or_user as task_type is a 
-- reserved column name that references edu_student_tasks 
-- rename to project_type for now to keep code from breaking but we should 
-- eventually make the code reference team_or_user instead
create or replace view edu_project_instances 
as
select 
i.task_instance_id as project_instance_id,
i.task_instance_name as project_instance_name,
i.task_instance_url as project_instance_url,
i.task_id as project_id,
i.description, i.approved_p, i.approved_date, i.approving_user,
team_or_user as project_type,
min_body_count, max_body_count, i.active_p
from edu_task_instances i, edu_student_tasks t
where i.task_id=t.task_id 
and t.task_type='project';

create or replace view edu_project_user_map 
as
select 
m.task_instance_id as project_instance_id,
m.team_id, m.student_id from edu_task_user_map m, 
edu_student_tasks t, edu_task_instances i
where t.task_type='project'
and i.task_id=t.task_id
and m.task_instance_id=i.task_instance_id;
