ad_library {
    Routines for manipulating portals.

    @creation-date January 2001
    @author tarik@alatovic.com
    @cvs-id $Id: portal-api-procs.tcl,v 1.3 2001/02/01 17:46:46 tarik Exp $
}

ad_proc -public portals_get_portal_id { type id domain_id } {
    @return returns portal_id for portal specified by $type and $id. 
            if portal specified by $type and $id doesn't already exist, 
            then the new one is created and portal_id of new portal is returned.
    @param type type is either user or group
    @param id id is either $user_id or $group_id

    @author Tarik Alatovic (tarik@alatovic.com)
    @date 09/01/2000
} {
    return [db_exec_plsql portals_get_portal_id "
    begin
    :1 := portal.get_portal_id(v_portal_type => :type, v_id => :id, v_domain_id => :domain_id);
    end;
    "]
}

ad_proc -public portals_get_portal_property { {-cached_p 1} which portal_id } {
    @return returns portal property
    @param which specifies property (type, user_id, group_id, url_identifier, parent_portal_id, domain_id, portal_name, control_panel_exists_p, page_count, page_id_list, bae_url, ancestor_list)
    @param portal_id specifies portal

    @error throws an error if requested property is not defined

    @author Tarik Alatovic (tarik@alatovic.com)
    @date 09/01/2000
} {
    switch $which {
	type {
	    if $cached_p {
		return [portals_memoize "portals_get_portal_property -cached_p 0 $which $portal_id"]
	    } else {
		db_1row get_portal_type "
		select decode(user_id, null, 1, 0) as user_id_null_p, decode(default_p, 't', 1, 0) as default_p
		from portals
		where portal_id=:portal_id"
		
		if $default_p {
		    return default_portal
		} else {
		    if $user_id_null_p {
			return group
		    } else {
			return user
		    }
		}
	    }
	}
	url_identifier - parent_portal_id {
	    return [db_string get_portal_property "
	    select $which 
	    from portals
	    where portal_id=:portal_id"]
	}
	user_id - group_id - domain_id {
	    if $cached_p {
		return [portals_memoize "portals_get_portal_property -cached_p 0 $which $portal_id"]
	    } else {
		return [db_string get_portal_property "
		select $which 
		from portals
		where portal_id=:portal_id"]
	    }
	}
	portal_name {
	    set portal_name [db_string get_portal_name "
	    select portal_name
	    from portals
	    where portal_id=:portal_id"]

	    if [empty_string_p $portal_name] {
		set portal_type [portals_get_portal_property type $portal_id]

		switch $portal_type {
		    default_portal {
			set domain_id [portals_get_portal_property domain_id $portal_id]
			return [portals_get_domain_property domain_name $domain_id]
		    }
		    group {
			set group_id [portals_get_portal_property group_id $portal_id]
			return [db_string group_portal_name "
			select group_name from user_groups where group_id=:group_id"]
		    }
		    user {
			set user_id [portals_get_portal_property user_id $portal_id]
			return [db_string user_portal_name "
			select first_names||' '||last_name from users where user_id=:user_id"]
		    }
		}
	    }

	}
	control_panel_exists_p {
	    set domain_id [portals_get_portal_property domain_id $portal_id]
	    if [db_0or1row control_panel "
	    select 1
	    from portal_domain_c_panel_map pdcpm
	    where pdcpm.domain_id=:domain_id"] {
		return 1
	    } else {
		return 0
	    }
	}
	page_count {
	    return [db_string portal_page_count "select count(*) from portal_pages where portal_id=:portal_id"]
	}
	page_id_list {
	    return [db_list portal_page_id_list "
	    select page_id 
	    from portal_pages 
	    where portal_id=:portal_id
	    order by page_number"]
	}
	base_url {
	    return [db_exec_plsql portal_base_url "
	    begin
	    :1 := portal.get_base_url(v_portal_id => :portal_id);
	    end;
	    "]
	}
	control_panel_url {
	    return "[portals_get_portal_property base_url $portal_id]/control-panel/"
	}
	ancestor_list {
	    return [db_exec_plsql portal_ancestor_list "
	    begin
	    :1 := portal.get_portal_ancestor_list(v_portal_id => :portal_id);
	    end;
	    "]
	}
	default {
	    error "portals_get_portal_property: $which parameter is not supported."
	}
    }    
}

ad_proc -public portals_set_portal_property { which portal_id value} {
    @param which specifies portal_property (user_id, group_id, domain_id, url_identifier)
    @param portal_id specifies portal
    @param value specifies property value
    
    @error throws an error if property specifed by which does not exist
    @error results in database error if portal_id does not exist
    
    @author Tarik Alatovic (tarik@alatovic.com)
    @date 01/30/2001
} {
    switch $which {
	url_identifier {
	    set encoded_url_identifier [ad_urlencode $value]
	    db_dml update_portal_property "
	    update portals
	    set url_identifier = :encoded_url_identifier
	    where portal_id= :portal_id"
	}
	user_id - group_id - domain_id {
	    db_dml update_portal_property "
	    update portals
	    set $which = :value
	    where domain_id= :portal_id"

	    portals_memoize_flush_for_portal_property $portal_id
	}
	default {
	    error "portals_set_portal_property: $which is invalid portal property"
	}
    }
}

ad_proc -public portals_create_portal { type id domain_id {portal_name ""}} {
    @return returns portal_id of newly created portal
    @param type specifes type of the new portals (user, group)
    @param id specifes user_id or group_id for which portal is created
    @param domain_id specifes domain_id for which portal is created
    @param optionally, name can be associated with the portal
    
    @author Tarik Alatovic (tarik@alatovic.com)
    @date 09/01/2000
} {
    # id_name is v_user_id for user portals and v_group_id for group portals
    set id_name v_${type}_id

    set portal_id [db_exec_plsql portal_create "
    begin
    :1 := portal.create_new(v_domain_id => :domain_id, v_portal_name => :portal_name, $id_name => :id);
    end;
    "]
    
    return $portal_id
}

ad_proc -public portals_create_default_portal {domain_id} {
    @return returns portal_id of newly created portal for the domain specified by domain_id
    @param domain_id specifes domain_id for which default portal is created
    
    @author Tarik Alatovic (tarik@alatovic.com)
    @date 12/19/2000
} {
    set portal_id [db_exec_plsql default_portal_create "
    begin
    :1 := portal.create_new(v_domain_id => :domain_id, v_default_p => 't');
    end;
    "]
    
    return $portal_id
}

ad_proc -public portals_delete_portal {{} portal_id} {
    Deletes a portal.

    @param portal_id specified portal to be deleted 
    
    @author Tarik Alatovic (tarik@alatovic.com)
    @date 09/01/2000
} {
    set page_id_list [portals_get_portal_property page_id_list $portal_id]
    foreach page_id $page_id_list {
	portals_memoize_flush_for_page_property $page_id
    }
    portals_memoize_flush_for_portal_property $page_id

    db_exec_plsql portal_remove "
    begin
    portal.remove(v_portal_id => :portal_id);
    end;"
}

ad_proc -public portals_set_to_default_configuration { portal_id } {
    Sets portal identified by portal_id to be identical to default portal for that domain.

    @author tarik@alatovic.com
    @date 01/11/2001
    @param portal_id portal_id for the portal we want to set to default portal.
} {
    db_exec_plsql portals_set_to_default_configuration "
    begin
        portal.set_to_default_configuration(v_portal_id => :portal_id);
    end;"

    set page_id_list [portals_get_portal_property page_id_list $portal_id]
    foreach page_id $page_id_list {
	portals_memoize_flush_for_page_property $page_id
    }

    return
}

ad_proc -public portals_make_page_numbers_sequential {portal_id} {
    Makes page numbers sequential starting from 1 for portal identified by portal_id.

    @param portlet_id identifies portlet that whose page_numbers are being put into sequential order

    @author Tarik Alatovic (tarik@alatovic.com)
    @date 11/29/2000
} {
    db_exec_plsql portals_make_page_numbers_sequential "
    begin
    portal_page.make_page_numbers_sequential(v_portal_id => :portal_id);
    end;"

    set page_id_list [portals_get_portal_property page_id_list $portal_id]
    foreach page_id $page_id_list {
	portals_memoize_flush_for_page_property $page_id
    }
}

ad_proc -private portals_memoize_init_for_portal_properties {} {
    Initializes memoize cache for portal properties.
    @author Tarik Alatovic
    @date 01/29/2001
} {
    ns_log Notice "Initializing Portals Properties Cache ..."
    db_foreach portal_property "
    select portal_id, user_id, group_id, domain_id,
           decode(user_id, null, 1, 0) as user_id_null_p, decode(default_p, 't', 1, 0) as default_p
    from portals
    " {
	portals_memoize_init "portals_get_portal_property -cached_p 0 user_id $portal_id" $user_id
	portals_memoize_init "portals_get_portal_property -cached_p 0 group_id $portal_id" $group_id
	portals_memoize_init "portals_get_portal_property -cached_p 0 domain_id $portal_id" $domain_id

	if $default_p {
	    set type default_portal
	} else {
	    if $user_id_null_p {
		set type group
	    } else {
		set type user
	    }
	}
	portals_memoize_init "portals_get_portal_property -cached_p 0 type $portal_id" $type
    }
    ns_log Notice "Done initializing portals properties cache"
}

ad_proc -private portals_memoize_flush_for_portal_property { portal_id } {
    Flushes memoize cache for portals_get_portal_property.

    @author Tarik Alatovic
    @date 01/29/2001

    @param portal_id
} {
    portals_memoize_flush "portals_get_portal_property -cached_p 0 user_id $portal_id"
    portals_memoize_flush "portals_get_portal_property -cached_p 0 group_id $portal_id"
    portals_memoize_flush "portals_get_portal_property -cached_p 0 domain_id $portal_id"
    portals_memoize_flush "portals_get_portal_property -cached_p 0 type $portal_id"
}

