# /packages/survey/www/process-response.tcl
ad_page_contract {

    Insert user response into database.
    This page receives an input for each question named
    response_to_question.

    @param   survey_id             survey user is responding to
    @param   return_url            optional redirect address
    @param   group_id              
    @param   response_to_question  since form variables are now named as response_to_question.$question_id, this is actually array holding user responses to all survey questions.
    @param   edit_previous_response_p This tells whether the user is editing a previous response or adding a new response
    
    @author  dparker@arsdigita.com
    @date    Jan 3, 2001
    @cvs-id  $ID$
} {
    response_id:integer,notnull
    survey_id:integer,notnull
    {return_url "index.tcl"}
    response_to_question:array,optional,multiple
    edit_previous_response_p:notnull
    scope:optional
    group_id:optional,integer
}


# Get the names and settings for this survey from the database

db_0or1row survsimp_name_from_id "
select name as survey_name, enabled_p, response_editable_p, single_response_p 
from survsimp_surveys
where survey_id = :survey_id 
  and [ad_scope_sql]"

# check the scope settings
set user_id [ad_verify_and_get_user_id]
ad_scope_error_check
ad_scope_authorize $scope all group_member none 

# check to make sure the survey is still active
# We wouldn't want to insert a response after the survey has been closed
if {![info exist enabled_p] || $enabled_p == "f"} {
    ad_return_error "Not Allowed" "This survey has been turned of by survey administrator.  
    Your survey was not processed."
    return
}


# Check the response ID for a double click
# If they are adding a new response, then this will keep to
# entrys from being inserted into the database.  If they are updateing their
# answers it will be "updated" twice, but with the same values so it's ok.

set count_responses [db_string check_double_click "
select count(*) 
from survsimp_responses 
where response_id=:response_id"]

if {$count_responses > 0 && $edit_previous_response_p == "f" } {
    # There was a double click.  return them to index or return url
    ad_returnredirect $return_url?[export_url_scope_vars]
    return
}

if {$edit_previous_response_p == "t" && $response_editable_p == "f"} {
    # there was some error in the logic.  The administrator must have changed the 
    # survey properties
    ad_return_error "Error" "You are not allowed to edit this response"
    return
} 


set question_info_sql "
select question_id, question_text, abstract_data_type, presentation_type, required_p
from survsimp_questions
where survey_id = :survey_id
and active_p = 't'
order by sort_key"

## Validate input.
set questions_with_missing_responses [list]
set exception_count 0
set exception_text ""

db_foreach get_question $question_info_sql {
    #  Need to clean-up after mess with :array,multiple flags
    #  in ad_page_contract.  Because :multiple flag will sorround empty
    #  strings and all multiword values with one level of curly braces {}
    #  we need to get rid of them for almost any abstract_data_type
    #  except 'choice', where this is intended behaviour.  Why bother
    #  with :multiple flag at all?  Because otherwise we would lost all
    #  but first value for 'choice' abstract_data_type - see ad_page_contract
    #  doc and code for more info.
    #
    if { [exists_and_not_null response_to_question($question_id)] } {
	if {$abstract_data_type != "choice"} {
	set response_to_question($question_id) [join $response_to_question($question_id)]
	}
    }
    
    
    if { $abstract_data_type == "date" } {
	if [catch  { set response_to_question($question_id) [validate_ad_dateentrywidget "" response_to_question.$question_id [ns_getform]]} errmsg] {
	    incr exception_count
	    append exception_text "<li>$errmsg: Please make sure your dates are valid."
	}
    }
    
    if { [exists_and_not_null response_to_question($question_id)] } {
	set response_value [string trim $response_to_question($question_id)]
    } elseif {$required_p == "t"} {
	lappend questions_with_missing_responses $question_text
	continue
    } else {
	set response_to_question($question_id) ""
	set response_value ""
    }
    
    if {![empty_string_p $response_value]} {
	if { $abstract_data_type == "number" } {
	    if { ![regexp {^(-?[0-9]+\.)?[0-9]+$} $response_value] } {
		incr exception_count
		append exception_text "<li>The response to \"$question_text\" must be a number. Your answer was \"$response_value\".\n"
		continue
	    }
	} elseif { $abstract_data_type == "integer" } {
	    if { ![regexp {^[0-9]+$} $response_value] } {
		incr exception_count
		append exception_text "<li>The response to \"$question_text\" must be an integer. Your answer was \"$response_value\".\n"
		continue
	    }
	}
    }
    if { $abstract_data_type == "blob" } {
        set tmp_filename $response_to_question($question_id.tmpfile)
        set n_bytes [file size $tmp_filename]
        if { $n_bytes == 0 && $required_p == "t" } {
            incr exception_count
            append exception_text "Your file is zero-length. Either you attempted to upload a zero length file, a file which does not exist, or something went wrong during the transfer."
        }
    }

}

set missing_expression_count [llength $questions_with_missing_responses]

if { $missing_expression_count > 0 } {
    incr exception_count $missing_expression_count
    append exception_text "<li>You didn't respond to all required sections. You skipped:
    <ul>
    <li>[join $questions_with_missing_responses "\n<li>"]
    </ul>
    "
}

if { $exception_count > 0 } {
    ad_return_complaint $exception_count $exception_text
    return
}


    

# Generate the insert and update sql
# For each case there can be an update or on insert 
# we can't have transactions with nested db statements inside of them 
# so this is a little messy but it works.
set count 0
db_transaction {
db_foreach get_question $question_info_sql {
    
    set peeraddr [ns_conn peeraddr]
	if {$edit_previous_response_p == "f" && $count == 0} {
	    # we want this to be inside the transaction and since we can't have
	    # nested transactions we have to put it here.
	    # to maek sure that we only have one entry for responce, we
	    # check the count
	    set survsimp_response_insert_sql "insert into survsimp_responses 
	    (response_id, survey_id, user_id, ip_address, [ad_scope_cols_sql])
	    values 
	    ($response_id, $survey_id, $user_id, '$peeraddr', [ad_scope_vals_sql])"
	    
	    db_dml survsimp_response_insert $survsimp_response_insert_sql
	    ns_log notice "insert response $survsimp_response_insert_sql"
	}
	
	incr count
	
	set response_value [string trim $response_to_question($question_id)]
	
	
	switch -- $abstract_data_type {
	    "choice" {
		if { $presentation_type == "checkbox"  || $presentation_type == "select"} {
		    # Deal with multiple responses. 
		    set checked_responses $response_to_question($question_id)
		    foreach response_value $checked_responses {
			if { [empty_string_p $response_value] } {
			    set response_value [db_null]
			}
			
			set survsimp_question_response_insert_sql "
			insert into survsimp_question_responses 
			(response_id, question_id, choice_id)
			values 
			(:response_id, :question_id, :response_value)" 
		    
			set survsimp_question_response_update_sql "
			update survsimp_question_responses
			set choice_id=:response_value 
			where response_id=:response_id
			and question_id=:question_id"
			
			if {$edit_previous_response_p == "f"} {
			    db_dml survsimp_question_response_insert $survsimp_question_response_insert_sql 
			} else {
			    db_dml survsimp_question_response_update $survsimp_question_response_update_sql
			}
			
			
		    }
		} else {
		    if { [empty_string_p $response_value] } {
			set response_value [db_null]
		    }
		    
		    set survsimp_question_response_insert_sql "
		    insert into survsimp_question_responses 
		    (response_id, question_id, choice_id)
		    values 
		    (:response_id, :question_id, :response_value)"
		    
		    
		    set survsimp_question_response_update_sql "
		    update survsimp_question_responses
		    set choice_id=:response_value 
		    where response_id=:response_id
		    and question_id=:question_id"
		    
		    if {$edit_previous_response_p == "f"} {
			db_dml survsimp_question_response_insert $survsimp_question_response_insert_sql 
		    } else {
		    db_dml survsimp_question_response_update $survsimp_question_response_update_sql
		    }
		    
		}
	    }
	    "shorttext" {
		
		set survsimp_question_response_insert_sql "
		insert into survsimp_question_responses 
		(response_id, question_id, varchar_answer)
		values 
		(:response_id, :question_id, :response_value)" 
		
		set survsimp_question_response_update_sql "
		update survsimp_question_responses
		set varchar_answer='$response_value' 
		where response_id=$response_id
		and question_id=$question_id"
		
		if {$edit_previous_response_p == "f"} {
		    db_dml survsimp_question_response_insert $survsimp_question_response_insert_sql 
		} else {
		    
		    db_dml survsimp_question_response_update $survsimp_question_response_update_sql
		}
	    
	    }
	    "boolean" {
		if { [empty_string_p $response_value] } {
		    set response_value [db_null]
		}
		
		set survsimp_question_response_insert_sql "
		insert into survsimp_question_responses 
		(response_id, question_id, boolean_answer)
		values 
		(:response_id, :question_id, :response_value)"
		
		set survsimp_question_response_update_sql "
		update survsimp_question_responses
		set boolean_answer=:response_value 
		where response_id=:response_id
		and question_id=:question_id"
		
		if {$edit_previous_response_p == "f"} {
		    db_dml survsimp_question_response_insert $survsimp_question_response_insert_sql 
		} else {
		    db_dml survsimp_question_response_update $survsimp_question_response_update_sql
		}
		
	    }
	    "number" {
		if { [empty_string_p $response_value] } {
		    set response_value [db_null]
		} 
		
		set survsimp_question_response_insert_sql "
		insert into survsimp_question_responses 
		(response_id, question_id, number_answer)
		values 
		(:response_id, :question_id, :response_value)"
		
		set survsimp_question_response_update_sql "
		update survsimp_question_responses
		set number_answer=:response_value 
		where response_id=:response_id
		and question_id=:question_id"
		
		if {$edit_previous_response_p == "f"} {
		    db_dml survsimp_question_response_insert $survsimp_question_response_insert_sql 
		} else {
		    db_dml survsimp_question_response_update $survsimp_question_response_update_sql
		}
		
	    }
	    "integer" {
		if { [empty_string_p $response_value] } {
		    set response_value [db_null]
		} 
		
		set survsimp_question_response_insert_sql "
		insert into survsimp_question_responses 
		(response_id, question_id, number_answer)
		values 
		(:response_id, :question_id, :response_value)"
		
		set survsimp_question_response_update_sql "
		update survsimp_question_responses
		set number_answer=:response_value 
		where response_id=:response_id
		and question_id=:question_id"
		
		if {$edit_previous_response_p == "f"} {
		    db_dml survsimp_question_response_insert $survsimp_question_response_insert_sql 
		} else {
		    db_dml survsimp_question_response_update $survsimp_question_response_update_sql
		}
		
	    }
	    "text" {
		if { [empty_string_p $response_value] } {
		    set response_value [db_null]
		}
	    
		set survsimp_question_response_insert_sql "
		insert into survsimp_question_responses
		(response_id, question_id, clob_answer)
		values 
		(:response_id, :question_id, empty_clob())
		returning clob_answer into :1" 
		
		set survsimp_question_response_update_sql "
		update survsimp_question_responses
		set clob_answer=empty_clob() 
		where response_id=:response_id
		and question_id=:question_id
		returning clob_answer into :1
		"
		if {$edit_previous_response_p == "f"} {
		    db_dml survsimp_question_response_insert $survsimp_question_response_insert_sql -clobs [list $response_value]
		} else {
		    db_dml survsimp_question_response_update $survsimp_question_response_update_sql -clobs [list $response_value]
		}
	    }
	    "date" {
		if { [empty_string_p $response_value] } {
		    set response_value [db_null]
		}
		
		set survsimp_question_response_date_insert_sql "
		insert into survsimp_question_responses 
		(response_id, question_id, date_answer)
		values 
		(:response_id, :question_id, :response_value:)"
		
		set survsimp_question_response_update_sql "
		update survsimp_question_responses
		set date_answer=:response_value 
		where response_id=:response_id
		and question_id=:question_id"
		
		if {$edit_previous_response_p == "f"} {
		    db_dml survsimp_question_response_insert $survsimp_question_response_insert_sql 
		} else {
		    db_dml survsimp_question_response_update $survsimp_question_response_update_sql
		}
		
	    }   
	    "blob" {
		if { ![empty_string_p $response_value] } {
		    # this stuff only makes sense to do if we know the file exists
		    set tmp_filename $response_to_question($question_id.tmpfile)
		    set file_extension [string tolower [file extension $response_value]]
		    # remove the first . from the file extension
		    regsub {\.} $file_extension "" file_extension
		    set guessed_file_type [ns_guesstype $response_value]
		    
		    set n_bytes [file size $tmp_filename]
		    # strip off the C:\directories... crud and just get the file name
		    if ![regexp {([^/\\]+)$} $response_value match client_filename] {
			# couldn't find a match
			set client_filename $response_value
		    }
		    if { $n_bytes == 0 } {
			error "This should have been checked earlier."
		    } else {
			
			set survsimp_question_response_insert_sql "
			insert into survsimp_question_responses 
			(response_id, question_id, attachment_answer, attachment_file_name, 
			attachment_file_type, attachment_file_extension)
			values 
			(:response_id, :question_id, empty_blob(), :response_value, :guessed_file_type, :file_extension) 
			returning attachment_answer into :1" 
			
			set survsimp_question_response_update_sql "
			update survsimp_question_responses
			set attachment_answer=empty_blob(), attachment_file_name=:response_value, 
			attachment_file_type=:guessed_file_type, attachment_file_extension=:file_extension
			where response_id=:response_id
			and question_id=:question_id
			returning attachment_answer into :1"
			
			if {$edit_previous_response_p == "f"} {
			    db_dml survsimp_question_response_insert $survsimp_question_response_insert_sql -blob_files [list $tmp_filename]
			} else {
			    db_dml survsimp_question_response_update $survsimp_question_response_update_sql -blob_files [list $tmp_filename]
			}
		    
			
		    }
		}
	    }
	}
    }
}

if {[info exists return_url] && ![empty_string_p $return_url]} {
    ad_returnredirect "$return_url"
    return
} else {
    doc_return 200 text/html "
    [ad_scope_header "Response Submitted"]
    [ad_scope_page_title "$survey_name"]
    [ad_scope_context_bar [list "index.tcl?[export_url_scope_vars]" "Surveys"] "One Survey"]

    <hr>
    
    <blockquote>
    Response submitted. Thank you.
    </blockquote>
    
    [ad_scope_footer]
    "
}
















