alter table portal_domains add (
	top_level_domain_p	char(1) default 't' check (top_level_domain_p in ('t','f'))
);

alter table portals add (
	parent_portal_id	constraint portals_parent_portal_id_fk references portals on delete cascade,
	url_identifier		varchar(100)
);

--------------------------------------------------------------
--                       PORTAL PACKAGE                     --
--------------------------------------------------------------

-- portals portal package contains PL/SQL API for manipulation of
-- portal objects

create or replace package portal
as

-- creates new portal portal object
function create_new (
	v_domain_id		in integer,
	v_default_p		in char default 'f',
	v_portal_name		in varchar default null,
	v_group_id		in integer default null,
	v_user_id		in integer default null
)
return integer;

-- deletes portal object identified by v_portal_id
procedure remove (
	v_portal_id		in integer
);

-- returns unique url identifier for a portal
function generate_unique_url_identifier(
	v_domain_id		in integer, 
	v_parent_portal_id	in integer default null,
	v_user_id 		in integer default null,
	v_group_id		in integer default null,
	v_default_p		in char default 'f'
)
return varchar;

function url_identifier_unique_p (
	v_domain_id		in integer, 
	v_parent_portal_id	in integer default null,
	v_url_identifier	in varchar
)
return varchar;

end portal;
/
show errors


create or replace package body portal as

-- creates new portal portal object
function create_new (
	v_domain_id		in integer,
	v_default_p		in char default 'f',
	v_portal_name		in varchar default null,
	v_group_id		in integer default null,
	v_user_id		in integer default null
)
return integer
is
	v_portal_id		integer;
begin

	select portal_id_sequence.nextval into v_portal_id from dual;

	insert into portals
	(portal_id, default_p, domain_id, portal_name, group_id, user_id)
	values
	(v_portal_id, v_default_p, v_domain_id, v_portal_name, v_group_id, v_user_id);
	
	return v_portal_id;
end create_new;

-- deletes portal object identified by v_portal_id
procedure remove (
	v_portal_id		in integer
)
is
begin
	delete from portals where portal_id=v_portal_id;
end remove;

function generate_unique_url_identifier(
	v_domain_id		in integer, 
	v_parent_portal_id	in integer default null,
	v_user_id 		in integer default null,
	v_group_id		in integer default null,
	v_default_p		in char default 'f'
)
return varchar
is
	v_portal_type		varchar(20);
	v_url_identifier_base	portals.url_identifier%TYPE;
	v_url_identifier	portals.url_identifier%TYPE;
	v_url_identifier_number integer;
begin
	if ((v_default_p='t') and (v_user_id is null) and (v_group_id is null)) then
		v_portal_type:='default_portal';
	elsif ((v_default_p='f') and (v_user_id is not null) and (v_group_id is null)) then
		v_portal_type:='user';
	elsif ((v_default_p='f') and (v_group_id is not null) and (v_user_id is null)) then
		v_portal_type:='group';
	else
		raise_application_error(-20000, 'portals.generate_unique_url_identifier: Invalid portal type');
	end if;

	if v_portal_type='default_portal' then
		return 'default-portal';
	end if;

	if v_portal_type='user' then
		-- user portal
		select user_id into v_url_identifier_base
		from users 
		where user_id=v_user_id;
	else
		select short_name into v_url_identifier_base
		from user_groups
		where group_id=v_group_id;
	end if;

	v_url_identifier_number:=0; 
	v_url_identifier:= v_url_identifier_base;

	while portal.url_identifier_unique_p(v_domain_id, v_parent_portal_id, v_url_identifier)=0 loop
		v_url_identifier_number:= v_url_identifier_number+1;
		v_url_identifier:= v_url_identifier_base || '-' || v_url_identifier_number;
	end loop;
	return v_url_identifier;
end generate_unique_url_identifier;

function url_identifier_unique_p (
	v_domain_id		in integer, 
	v_parent_portal_id	in integer default null,
	v_url_identifier	in varchar
)
return varchar
is
	v_url_identifier_unique_p	integer;
begin
	begin
		select 0 into v_url_identifier_unique_p from portals
		where domain_id= v_domain_id
		and ((parent_portal_id= v_parent_portal_id) or ((parent_portal_id is null) and (v_parent_portal_id is null)))
		and url_identifier= v_url_identifier;
	exception
		when no_data_found then
			v_url_identifier_unique_p:=1;
	end;
	return v_url_identifier_unique_p;
end url_identifier_unique_p;

end portal;
/
show errors

create table portal_pages_temp
as
select * from portal_pages;

create table portal_portlet_page_map_temp
as
select * from portal_portlet_page_map;

drop table portal_portlet_page_map;
drop table portal_pages;

create table portals_temp
as
select * from portals;

update portals_temp set url_identifier= portal.generate_unique_url_identifier(domain_id, NULL, user_id, group_id, default_p);
commit;

drop table portals;

create table portals (
	portal_id		integer 
	                        constraint portals_portal_id_pk primary key,
	-- we need parent_portal_id for supporting subportals
	parent_portal_id	constraint portals_parent_portal_id_fk references portals on delete cascade,
	url_identifier		varchar(100)
				constraint portals_url_identifier_nn not null,
	default_p		char(1) default 'f'
				constraint portals_default_p_ck check(default_p in ('t', 'f')),
	domain_id		constraint portals_domain_id_fk references portal_domains on delete cascade
				constraint portals_domain_id_nn not null,
	portal_name             varchar(300),
	-- define ownership of the page - either as a group page or individual page
	group_id		constraint portals_group_id_fk references user_groups on delete cascade,
	user_id			constraint portals_user_id_fk references users on delete cascade,
	-- if this is a default portal we store neither user_id nor group_id
	-- if this is a user portal then we store user_id only
	-- it this is a group portal then we store group_id only
	constraint portals_type_check_ck check(((default_p='t') and (user_id is null) and (group_id is null))
                                               or ((default_p='f') and (((user_id is not null) and (group_id is null))
                                                                        or ((group_id is not null) and (user_id is null))))),
	-- within domain every user can have exactly one portal
	-- within domain every group can have exactly one portal
	-- within domain there is at most one default portal 
	constraint portals_portal_unique_un unique(user_id, group_id, domain_id),
	-- url_identifier must be unique within a domain and parent portal
	constraint portals_url_identifier_un unique(domain_id, parent_portal_id, url_identifier)
);

insert into portals	
(portal_id, parent_portal_id, url_identifier, default_p, domain_id, portal_name, group_id, user_id)
select portal_id, parent_portal_id, url_identifier, default_p, domain_id, portal_name, group_id, user_id
from portals_temp;

commit;

drop table portals_temp;

create table portal_pages (
	page_id			integer 
	                        constraint portal_pages_page_id_pk primary key,
	portal_id		constraint portal_pages_portal_id_fk references portals on delete cascade
				constraint portal_pages_portal_id_nn not null,
	-- page_name may be null, in which case we use "Page #x" instead
        page_name		varchar(300),
	-- page_number determines the order of the page links in the nav bar
	-- the first page_number should be 1. 
        page_number             integer
);
create index portal_pages_portal_id_index on portal_pages(portal_id);

insert into portal_pages
(page_id, portal_id, page_name, page_number)
select page_id, portal_id, page_name, page_number
from portal_pages_temp;

commit;

drop table portal_pages_temp;

create table portal_portlet_page_map (
	portlet_id	   	constraint portal_portlet_page_map_ti_fk references portal_portlets on delete cascade
                                constraint portal_portlet_page_map_ti_nn not null,
	page_id			constraint portal_portlet_page_map_pi_fk references portal_pages on delete cascade
                                constraint portal_portlet_page_map_pi_nn not null,
	constraint portal_portlet_pk primary key(portlet_id, page_id),
	-- sort_key and page_side define location of the portlet on the page     	 
	-- this defines an order within this side of this page
	sort_key  	    	integer,
	-- side of the page the portlet will displayed on
	page_side       	char(1) 
                                constraint portal_portlet_page_map_ps_ck check (page_side in ('l','r'))
);

insert into portal_portlet_page_map
select portlet_id, page_id, sort_key, page_side
from portal_portlet_page_map_temp;

commit;

drop table portal_portlet_page_map_temp;


